   package Entitats.Actuacio;

   import Entitats.Temps.Data;
   import Entitats.Temps.Hora;
   import Entitats.Actuacio.RelacioPecesActuacio;
   import Entitats.Actuacio.RelacioParticipantsActuacio;
   import Entitats.Actuacio.RelacioImatgesActuacio;

/**
* Classe que permet crear i manipular actuacions, les quals tenen lloc en un poble
* o area determinats, una data concreta, ...
* No s'implementa en aquest punt cap validacio de les dades assignades a l'objecte, doncs
* es creu funcio indispensable de les capes superiors de l'aplicacio, les capes que 
* utilitzen aquesta classe. 
*@author Jordi Moles i Blanco, Mar 2005
*/

   public class Actuacio
   {
   
      private int identificadorActuacio; //Valor que identifica de manera unica l'actuacio
   												  //en el sistema.
      private Data data; //Data en que l'actuacio tindra/va tenir lloc
      private String poblacio; //Poblacio en que l'actuacio tindra/va tenir lloc
      private String lloc; //Lloc (dins la poblacio) en que l'actuacio tindra/va tenir lloc
      private Hora hora; //Hora d'inici de l'actuacio
      private RelacioPecesActuacio peces; //Relacio de peces que s'interpretaran o es
   													//van interpretar
      private RelacioParticipantsActuacio participants;	//Relacio de participants que s'interpretaran o es
   													               //van interpretar	
      private RelacioImatgesActuacio imatges; 		//Relacio d'imatges associades a l'actuacio																
      private String observacions; //Comentaris adicionals, per exemple la informacio de si ha plogut
   
   /**
   *Constructor de la classe.
   *Inicialitza tots els atributs numerics a 0 i els strings a "" (Blanc)
   *La data de l'actuacio s'estableix a 00/00/0000
   *L'hora s'estableix a 00:00:00
   */
      public Actuacio()
      {
      
         identificadorActuacio=0;
         data=new Data();
         data.estableixData(0,0,0);
         poblacio="";
         lloc="";
         hora=new Hora();
         hora.estableixHora(0,0,0);
         peces = new RelacioPecesActuacio();
         participants = new RelacioParticipantsActuacio();
         imatges = new RelacioImatgesActuacio();
         observacions ="";
      
      }
   
   	/**
   	*Destructor de la classe
   	*Metode que elimina les referencies als objectes manipulats per una actuacio
   	*per tal de que l'objecte "Gargabe Collector" del Java n'alliberi la memoria assignada
   	*quan descobreixi que l'objecte ja no te cap referencia.
   	*/
      public void destructorActuacio()
      {
      
         data.destructorData();
         hora.destructorHora();
         peces.destructorRelacioPecesActuacio();
         participants.destructorRelacioParticipantsActuacio();
         imatges.destructorRelacioImatgesActuacio();
      
      }
   
   /**
   * Metode que estableix l'identificador de l'objecte. El contingut previ de l'objecte en 
   * aquest camp es perdra totalment.
   *@param identificadorActuacio Valor que identifica l'actuacio de manera unica en el sistema
   */
      public void estableixIdentificador(int identificadorActuacio)
      {
      
         this.identificadorActuacio=identificadorActuacio;
      
      }
   
   /**
   * Metode que estableix quan tindra/va tenir lloc l'actuacio. El contingut previ de 
   * l'objecte en aquest camp es perdra totalment.
   *@param dia Enter amb el valor del dia del mes que ha de formar part de la data.
   *@param mes Enter amb el valor del mes de l'any que ha de formar part de la data.
   *@param any Enter amb el valor de l'any que ha de formar part de la data
   */
      public void estableixDataActuacio(int dia,int mes,int any)
      {
      
         data.estableixData(dia,mes,any);
      
      }
   
   
   /**
   * Metode que estableix la data en que tindra/va tenir lloc l'actuacio. El contingut previ de 
   * l'objecte en aquest camp es perdra totalment.
   *@param dataActuacio String que conte la data amb un dels seguents formats: dd/mm/aaaa o dd-mm-aaaa
   */
      public void estableixData(String dataActuacio)
      {
      
         int dia,mes,any;
         int posicio1,posicio2;
      
         posicio1=dataActuacio.indexOf("-");
         if(posicio1<0) posicio1=dataActuacio.indexOf("/");
         dia=Integer.parseInt(dataActuacio.substring(0,posicio1));
      
         posicio1++;
         posicio2=dataActuacio.indexOf("-",posicio1);
         if(posicio2<0) posicio2=dataActuacio.indexOf("/",posicio1);
         mes=Integer.parseInt(dataActuacio.substring(posicio1,posicio2));
      
         posicio2++;
         any=Integer.parseInt(dataActuacio.substring(posicio2).trim());
      
         estableixDataActuacio(dia,mes,any);
      
      }
   
   /**
   * Metode que estableix el lloc on tindra/va tenir lloc l'actuacio.
   *@param poblacio String amb el nom de la localitat o indret
   */
   
      public void estableixPoblacio(String poblacio)
      
      {
      
         this.poblacio=poblacio;
      
      }
   
   /**
   * Metode que estableix el lloc concret dins la poblacio on tindra/va tenir lloc l'actuacio.
   *@param lloc String amb el nom de la localitat o indret
   */
      public void estableixLloc(String lloc)
      {
      
         this.lloc=lloc;
      
      }
   
   /**
   * Metode que estableix la hora en que tindra/va tenir lloc l'actuacio. El contingut previ de 
   * l'objecte en aquest camp es perdra totalment.
   *@param hora Enter amb el valor de la hora d'inici.
   *@param minut Enter amb el valor del minut.
   */        
      public void estableixHoraInici(int hora,int minut)        
      {
      
         this.hora.estableixHora(hora,minut,0);
      
      }
   
   /**
   * Metode que estableix la hora en que tindra/va tenir lloc l'actuacio. El contingut previ de 
   * l'objecte en aquest camp es perdra totalment.
   *@param horaInici String amb el valor de la hora d'inici amb un dels seguents formats: hh:mm o hh.mm
   */
      public void estableixHoraInici(String horaInici)
      {
      
         int hores,minuts;
         int posicio1;
      
         posicio1=horaInici.indexOf(".");
         if(posicio1<0) posicio1=horaInici.indexOf(":");
         hores=Integer.parseInt(horaInici.substring(0,posicio1));
      
         posicio1++;
         minuts=Integer.parseInt(horaInici.substring(posicio1).trim());
      
         estableixHoraInici(hores,minuts);
      
      }
   
   
   
   /**
   * Metode que ens permet obtenir l'identificador de l'actuacio
   *@return Enter amb el valor de l'identificador de l'actuacio
   */
      public int obteIdentificador()
      {
      
         return identificadorActuacio;
      
      }
   
   /**
   * Metode que ens permet obtenir la data de l'actuacio que conte l'objecte
   *@return String amb el contingut de la data de l'actuacio en format dd/mm/aaaa
   */
      public String obteData()
      {
      
         return data.obteData();
      
      }
   
   /**
   * Metode que ens permet obtenir el lloc on es celebrara/va celebrar l'actuacio
   *@return String amb el nom de l'indret
   */           
   
      public String obtePoblacio()        
      
      {
      
         return poblacio;
      
      }	
   
   /**
   * Metode que ens permet obtenir el lloc concret dins una poblacio on es celebrara/va celebrar l'actuacio
   *@return String amb el nom de l'indret
   */           
      public String obteLloc()        
      {
      
         return lloc;
      
      }	
   
   /**
   * Metode que ens permet obtenir la hora de l'actuacio que conte l'objecte
   *@return String amb la hora de l'actuacio en format hh:mm
   */         
      public String obteHoraInici()         
      {
      
         String horaInici = hora.obteHora();
         int posicio1,posicio2;
         posicio1=horaInici.indexOf(":");
         posicio1++;
         posicio2=horaInici.indexOf(":",posicio1);
         horaInici = horaInici.substring(0,posicio2);
      
         return horaInici;
      
      }
   
   /**
   * Metode que ens permet obtenir les observacions que hem fet sobre una actuacio
   *@return String amb el contingut dels comentaris guardats
   */           
   
      public String obteObservacions()                   
      {
      
         return observacions;
      
      }	
   
   /**
   * Metode que estableix observacions sobre el desenvolupament de l'actuacio.
   *@param observacions String amb els comentaris a emmagatzemar sobre l'actuacio
   */
   
      public void estableixObservacions(String observacions)
      
      {
      
         this.observacions=observacions;
      
      }
   
   /**
   * Metode que ens permet afegir una pesa a la llista de peces que es duen a terme
   * en una actuacio
   *@param identificadorPesa Enter amb el valor de l'identificador de la pesa a afegir
   */
      public void afegeixPesa(int identificadorPesa)
      {
      
         peces.afegirPesaAActuacio(obteIdentificador(),identificadorPesa);
      
      }
   
   /**
   * Metode que ens permet eliminar la relacio d'una determinada actuacio amb totes les peces associades.
   */
   
      public void eliminaPeces()
      {
      
         peces.eliminaPeces(obteIdentificador());
      
      }
   
   //////Els seguents metodes representen l'iterador de peces associades a una actuacio\\\\\	
   
   /**
   * Metode que inicialitza l'iterador de les peces associades a una actuacio
   */
      public void obtePeces()
      {
      
         peces.obtePecesActuacio(obteIdentificador(),"ordre");
      
      }
   
   /**
   * Metode que ens informa si hi ha peces en l'iterador que encara no hem consultat
   *@return Boolea amb valor positiu si hi ha elements i negatiu si no n'hi ha.
   */
      public boolean hiHaPeces()
      {
      
         return peces.hiHaPeces();
      
      }
   
   /**
   * Metode que ens permet obtenir l'identificador de la primera pesa que encara
   * no hem consultat
   *@return Enter amb l'identificador de la pesa
   */
      public int obtePesa()
      {
      
         return peces.obtePesa();
      
      }
   
   /**
   * Metode que ens permet afegir un participant a la llista de participants que participen
   * en una actuacio
   *@param numSociParticipant Enter amb el numero de soci del participant a afegir
   */
      public void afegeixParticipant(int numSociParticipant)
      {
      
         participants.afegirParticipantAActuacio(obteIdentificador(),numSociParticipant);
      
      }
   
   /**
   * Metode que ens permet eliminar la relacio d'una determinada actuacio amb tots els participants d'aquesta.
   */
      public void eliminaParticipants()
      
      {
      
         participants.eliminaParticipants(obteIdentificador());
      
      }
   
   //////Els seguents metodes representen l'iterador de participants associats a una actuacio\\\\\	
   
   /**
   * Metode que inicialitza l'iterador dels participants associats a una actuacio
   */
   
      public void obteParticipants()
      
      {
         participants.obteParticipantsActuacio(obteIdentificador(),"ordre");
      }
   
   /**
   * Metode que ens informa si hi ha participants en l'iterador que encara no hem consultat
   *@return Boolea amb valor positiu si hi ha elements i negatiu si no n'hi ha.
   */
   
      public boolean hiHaParticipants()
      
      {
      
         return participants.hiHaParticipants();
      
      }
   
   /**
   * Metode que ens permet obtenir el numero de soci del primer participan que encara
   * no hem consultat
   *@return Enter amb el numero de soci del participant
   */
   
      public int obteParticipant()
      
      {
      
         return participants.obteParticipant();
      
      }
   
   	///////////////
   
   	/**
   * Metode que ens permet afegir una imatge a la llista
   *@param numImatge Enter amb el numero de imatge a afegir
   */
   
      public void afegeixImatge(int numImatge)
      
      {
      
         imatges.afegirImatgeAActuacio(obteIdentificador(),numImatge);
      
      }
   
   /**
   * Metode que ens permet eliminar la relacio d'una determinada actuacio amb totes les imatges d'aquesta.
   */
   
      public void eliminaImatges()
      
      {
      
         imatges.eliminaImatges(obteIdentificador());
      
      }
   
   //////Els seguents metodes representen l'iterador de imatges associades a una actuacio\\\\\	
   
   /**
   * Metode que inicialitza l'iterador de les imatges associades a una actuacio
   */
   
   
      public void obteImatges()
      
      
      {
         imatges.obteImatgesActuacio(obteIdentificador());
      }
   
   /**
   * Metode que ens informa si hi ha imatges en l'iterador que encara no hem consultat
   *@return Boolea amb valor positiu si hi ha elements i negatiu si no n'hi ha.
   */
   
   
      public boolean hiHaImatges()
      
      
      {
      
         return imatges.hiHaImatges();
      
      }
   
   /**
   * Metode que ens permet obtenir la primera imatge que encara
   * no hem consultat
   *@return Enter amb la imatge
   */
   
   
      public int obteImatge()
      
      
      {
      
         return imatges.obteImatge();
      
      }
   
   }