   package Entitats.Actuacio;

   import Entitats.Persistencia.Persistencia;
   import Entitats.Persistencia.Resultat;
   import Entitats.Actuacio.Actuacio;
   import Entitats.Persistencia.Llistat;
   import Entitats.Persistencia.ObjectePersistencia;

/**
* Classe que permet mantenir una relacio d'actuacions. Altes, baixes, modificacions i
* consultes d'actuacions per qualsevol camp son possibles. Implementa els elements necessaris
* per accedir a la BBDD o qualsevol altre sistema que utilitzem per conservar les dades
* que manipulem de forma permanent. Aixo es gracies a l'us de la classe Persistencia, que 
* ens permet treballar amb actuacions sense haver de preocupar-nos de com el sistema
* tracta les dades.
*@author Jordi Moles i Blanco, Mar 2005
*/

   public class RelacioActuacions
   {
   
      private Persistencia persistencia;
      private Resultat resultat;
      private Llistat llistat;
   
   /**
   * Constructor de la classe. Inicialitza els elements necessaris per tal d'accedir a la BBDD.
   */
      public RelacioActuacions()
      {
      
         persistencia = new Persistencia("entitats","actuacio");
      
      }
   
   /**
   * Destructor de la classe
   * Metode que elimina les referencies als objectes manipulats per una RelacioActuacions
   * per tal de que l'objecte "Gargabe Collector" del Java n'alliberi la memoria assignada
   * quan descobreixi que l'objecte ja no te cap referencia.
   */
      public void destructorRelacioActuacions()
      {
      
         persistencia.destructorPersistencia();
         resultat=null;
         llistat.destructorLlistat();
      
      }
   
   //////Els seguents metodes representen l'iterador d'actuacions recuperades del sistema\\\\\
                                                                                                                                                               
   /**
   * Metode que ens permet obtenir un conjunt d'actuacions que coincideixen amb els valors
   * donats. Es pot cercar una actuacio aportant valors de tots els seus camps
   * o nomes d'un si es desitja, i a mes, aquest camp pot ser qualsevol.
   *@param dadesActuacioAConsultar ObjectePersistencia amb tots els valors dels camps
   *per els que volem cercar una actuacio.
   */
      public void obteActuacioPerMultiplesCamps(ObjectePersistencia dadesActuacioAConsultar)
      {
      
         llistat = new Llistat(persistencia.obtePerDiversosCamps(dadesActuacioAConsultar));
      
      }
   
   /**
   * Metode que ens informa si hi ha actuacions en l'iterador que encara no hem consultat
   *@return Boolea amb valor positiu si hi ha elements i negatiu si no n'hi ha.
   */
      public boolean hiHaActuacions()
      {
      
         return llistat.hiHaElements();
      
      }
   
   /**
   * Metode que ens permet obtenir el primer objecte actuacio que encara
   * no hem consultat, amb totes les dades disponibles sobre aquesta en el sistema.
   *@return Objecte actuacio amb totes les dades disponibles
   */
      public Actuacio obteActuacio()
      {
      
         resultat=(Resultat)llistat.obtePrimerElement();
         Actuacio actuacio = new Actuacio();
      
         actuacio.estableixIdentificador(Integer.parseInt((String)resultat.obteElement("identificadorActuacio")));
         actuacio.estableixData((String)resultat.obteElement("dataActuacio"));
         actuacio.estableixPoblacio((String)resultat.obteElement("poblacio"));
         actuacio.estableixLloc((String)resultat.obteElement("lloc"));
         actuacio.estableixObservacions((String)resultat.obteElement("observacions"));
      
         String horaInici = (String)resultat.obteElement("horaInici");
         int posicio1,posicio2;
         posicio1=horaInici.indexOf(":");
         posicio1++;
         posicio2=horaInici.indexOf(":",posicio1);
         horaInici = horaInici.substring(0,posicio2);
         actuacio.estableixHoraInici(horaInici);
      
         return actuacio;
      
      }
   
   	//////////Fi iterador\\\\\\\
                                                                                                                                               
   /**
   * Metode que ens permet obtenir una actuacio a partir de l'identificador d'aquesta
   *@param identificadorActuacio Enter amb el valor que identifica una actuacio
   *de manera unica
   *@return Objecte actuacio amb totes les dades disponibles de l'actuacio sol.licitada
   */
      public static Actuacio obteActuacioPerIdentificadorActuacio(int identificadorActuacio)
      {
      
         Actuacio Actuacio= new Actuacio();
         Persistencia Persistencia = new Persistencia("entitats","actuacio");
         Resultat Resultat;
         Llistat ll;
         ll = new Llistat(Persistencia.obtePerCampNumeric("identificadorActuacio",identificadorActuacio));
         if(ll.hiHaElements())
         {
            Resultat=(Resultat)ll.obtePrimerElement();
            Actuacio=emplenaActuacio(Resultat);
         }
         Persistencia.destructorPersistencia();
         ll.destructorLlistat();
         Resultat=null;
      
         return Actuacio;
      
      }
   
      private static Actuacio emplenaActuacio(Resultat Resultat)
      {
      
         Actuacio Actuacio = new Actuacio();
      
         Actuacio.estableixIdentificador(Integer.parseInt((String)Resultat.obteElement("identificadorActuacio")));
         Actuacio.estableixData((String)Resultat.obteElement("dataActuacio"));
         Actuacio.estableixPoblacio((String)Resultat.obteElement("poblacio"));
         Actuacio.estableixLloc((String)Resultat.obteElement("lloc"));
         Actuacio.estableixObservacions((String)Resultat.obteElement("observacions"));
      
         String horaInici = (String)Resultat.obteElement("horaInici");
         int posicio1,posicio2;
         posicio1=horaInici.indexOf(":");
         posicio1++;
         posicio2=horaInici.indexOf(":",posicio1);
         horaInici = horaInici.substring(0,posicio2);
         Actuacio.estableixHoraInici(horaInici);
      
         return Actuacio;
      
      }
   
   /**
   * Metode que permet eliminar una actuacio del sistema a partir del seu identificador
   *@param identificadorActuacio Enter amb l'identificador de l'actuacio
   */
      public static void eliminaActuacioPerIdentificadorActuacio(int identificadorActuacio)
      {
      
         Persistencia Persistencia = new Persistencia("entitats","actuacio");
         Persistencia.eliminaPerCampNumeric("identificadorActuacio",identificadorActuacio);
         Persistencia.destructorPersistencia();
      
      }
   
   /**
   * Metode que permet actualitzar de cop diversos camps d'una actuacio
   *@param identificadorActuacio Enter amb el valor que ens permetra trobar l'actuacio
   *que volem modificar 
   *@param dadesActualitzadesActuacio ObjectePersistencia amb els noms dels camps a modificar, aixi com
   *els valors d'aquests
   */
      public static void actualitzaDiversosCampsActuacioPerIdentificadorActuacio(int identificadorActuacio,ObjectePersistencia dadesActualitzadesActuacio)
      {
      
         Persistencia Persistencia = new Persistencia("entitats","actuacio");
         Persistencia.actualitzaDiversosCampsPerCampNumeric("identificadorActuacio",identificadorActuacio,dadesActualitzadesActuacio);
         Persistencia.destructorPersistencia();
      
      }
   
   /**
   * Metode que ens permet afegir una actuacio a la relacio
   *@param actuacio Objecte Actuacio amb les dades de la nova actuacio que volem afegir
   *al sistema
   *@return Enter amb el valor que el sistema genera per a aquest nou element, valor
   *que l'identificara de manera unica en el sistema.
   */
      public static int afegirActuacio(Actuacio actuacio)
      {
      
         int identificador;
      
         Persistencia Persistencia = new Persistencia("entitats","actuacio");
         ObjectePersistencia dadesNovaActuacio = new ObjectePersistencia();
      
         dadesNovaActuacio.afegirCampData("dataActuacio",actuacio.obteData());
         dadesNovaActuacio.afegirCampText("poblacio",actuacio.obtePoblacio());
         dadesNovaActuacio.afegirCampText("lloc",actuacio.obteLloc());
         dadesNovaActuacio.afegirCampHoraSenseSegons("horaInici",actuacio.obteHoraInici());
         dadesNovaActuacio.afegirCampText("observacions",actuacio.obteObservacions());
      
         identificador=Persistencia.afegeixElement(dadesNovaActuacio);
      
         dadesNovaActuacio.destructorObjectePersistencia();
         Persistencia.destructorPersistencia();
      
         return identificador;
      
      }
   
   }