   package Entitats.Actuacio;

   import Entitats.Persistencia.Persistencia;
   import Entitats.Persistencia.Llistat;
   import Entitats.Persistencia.Resultat;
   import Entitats.Persistencia.ObjectePersistencia;

/**
* Classe que permet mantenir una relacio entre actuacions i les imatges associades a aquestes
*@author Jordi Moles i Blanco, Mar 2005
*/

   public class RelacioImatgesActuacio
   {
   
      private Persistencia persistencia; //Objecte que ens permet obtenir informacio de la BBDD.
      private Llistat llistat; 
      private Resultat resultat;
      private static int ordre=0; 
   
   /**
   * Constructor de la classe. Inicialitza els elements necessaris per tal d'accedir a la BBDD.
   */
      public RelacioImatgesActuacio()
      {
      
         persistencia = new Persistencia("entitats","imatgesActuacio");
         llistat=null;
      
      }
   
   /**
   * Destructor de la classe
   * Metode que elimina les referencies als objectes manipulats per una RelacioImatgesActuacio
   * per tal de que l'objecte "Gargabe Collector" del Java n'alliberi la memoria assignada
   * quan descobreixi que l'objecte ja no te cap referencia.
   */
      public void destructorRelacioImatgesActuacio()
      {
      
         persistencia.destructorPersistencia();
         if(llistat!=null)llistat.destructorLlistat();
         resultat=null;
      
      }
   
   //////Els seguents metodes representen l'iterador de imatges associades a una actuacio\\\\\
                                                            
   /**
   * Metode que cerca totes les imatges sobre actuacions   
   */
   
      public void obteImatgesActuacions()
      {
      
         llistat=new Llistat(persistencia.obteTots());
      
      }
   
   /**
   * Metode que cerca les imatges que composen una actuacio donada l'identificador d'aquesta
   *@param identificadorActuacio Enter amb el valor que identifica una actuacio de manera unica
   */
      public void obteImatgesActuacio(int identificadorActuacio)
      {
      
         llistat=new Llistat(persistencia.obtePerCampNumeric("actuacio_identificadorActuacio",identificadorActuacio));
      
      }
   
   
   /**
   * Metode que ens informa si hi ha imatges en l'iterador que encara no hem consultat
   *@return Boolea amb valor positiu si hi ha elements i negatiu si no n'hi ha.
   */
      public boolean hiHaImatges() 
      {
      
         return llistat.hiHaElements();
      
      }
   
   /**
   * Metode que ens permet obtenir l'identificador de la primera imatge que encara
   * no hem consultat. L'element no s'elimina del sistema pero si d'aquest iterador. Per tant,
   * haurem de tornar a executar obteImatgesActuacio si volem tornar a tractar tots els elements.
   *@return Enter amb l'identificador de la pesa
   */
      public int obteImatge()   
      {
      
         resultat=(Resultat)llistat.obtePrimerElement();
         return Integer.parseInt((String)resultat.obteElement("imatge_numImatge"));
      
      }
   
   //////////Fi iterador\\\\\\\
                                                                                                                                                                                             
   /**
   * Metode que permet afegir una imatge, es a dir, relacionar una nova imatge amb una actuacio.
   *@param actuacio Enter amb l'identificador de l'actuacio
   *@param imatge Enter amb l'idenificador de la imatge
   */
      public static void afegirImatgeAActuacio(int actuacio,int imatge)
      {
      
         Persistencia Persistencia = new Persistencia("entitats","imatgesActuacio");
         ObjectePersistencia dadesNovaImatgePerActuacio = new ObjectePersistencia();
      
         dadesNovaImatgePerActuacio.afegirCampNumeric("actuacio_identificadorActuacio",actuacio);
         dadesNovaImatgePerActuacio.afegirCampNumeric("imatge_numImatge",imatge);
      
         Persistencia.afegeixElement(dadesNovaImatgePerActuacio);
      
         dadesNovaImatgePerActuacio.destructorObjectePersistencia();
         Persistencia.destructorPersistencia();
      
      }
   
   /*
   * Metode que elimina la relacio d'una determinada actuacio amb totes les imatges relacionades.
   *@param identificadorActuacio Enter amb el valor que defineix a una actuacio de manera unica en el sistema. 
   */
      public static void eliminaImatges(int identificadorActuacio)
      {
      
         Persistencia Persistencia = new Persistencia("entitats","imatgesActuacio");
      
         Persistencia.eliminaPerCampNumeric("actuacio_identificadorActuacio",identificadorActuacio);
      
         Persistencia.destructorPersistencia();
      
      }
   
   }