   package Entitats.Actuacio;

   import Entitats.Persistencia.Persistencia;
   import Entitats.Persistencia.Llistat;
   import Entitats.Persistencia.Resultat;
   import Entitats.Persistencia.ObjectePersistencia;

/**
* Classe que permet mantenir una relacio entre actuacions i els participants associats a aquestes,
*@author Jordi Moles i Blanco, Mar 2005
*/


   public class RelacioParticipantsActuacio
   
   {
   
      private Persistencia persistencia; //Objecte que ens permet obtenir informacio de la BBDD.
      private Llistat llistat; 
      private Resultat resultat;
      private static int ordre=0; 
   
   /**
   * Constructor de la classe. Inicialitza els elements necessaris per tal d'accedir a la BBDD.
   */
   
      public RelacioParticipantsActuacio()
      
      {
      
         persistencia = new Persistencia("entitats","participantsActuacio");
         llistat=null;
      
      }
   
   /**
   * Destructor de la classe
   * Metode que elimina les referencies als objectes manipulats per una RelacioParticipantsActuacio
   * per tal de que l'objecte "Gargabe Collector" del Java n'alliberi la memoria assignada
   * quan descobreixi que l'objecte ja no te cap referencia.
   */
   
      public void destructorRelacioParticipantsActuacio()
      
      {
      
         persistencia.destructorPersistencia();
         if(llistat!=null)llistat.destructorLlistat();
         resultat=null;
      
      }
   
   //////Els seguents metodes representen l'iterador de participants associats a una actuacio\\\\\
                                                                                                                                                
   /**
   * Metode que cerca els participants que participen en una actuacio donat l'identificador d'aquesta
   *@param identificadorActuacio Enter amb el valor que identifica una actuacio de manera unica
   */
   
      public void obteParticipantsActuacio(int identificadorActuacio)
      
      {
      
         llistat=new Llistat(persistencia.obtePerCampNumeric("actuacio_identificadorActuacio",identificadorActuacio));
      
      }
   
   /**
   * Metode que cerca els participants que participen en una actuacio donat l'identificador d'aquesta
   * i ordenats per un camp determinat 
   *@param identificadorActuacio Enter amb el valor que identifica una actuacio de manera unica
   *@param campOrdre String amb el nom del camp per el que volem ordenar el resultat
   */
      public void obteParticipantsActuacio(int identificadorActuacio,String campOrdre)
      
      {
      
         ObjectePersistencia campActuacioABuscar = new ObjectePersistencia();
         campActuacioABuscar.afegirCampNumeric("actuacio_identificadorActuacio",identificadorActuacio);
         llistat=new Llistat(persistencia.obtePerDiversosCampsAscPer(campActuacioABuscar,campOrdre));
         campActuacioABuscar.destructorObjectePersistencia();
      }
   
   /**
   * Metode que ens informa si hi ha participants en l'iterador que encara no hem consultat
   *@return Boolea amb valor positiu si hi ha elements i negatiu si no n'hi ha.
   */
   
      public boolean hiHaParticipants() 
      
      {
      
         return llistat.hiHaElements();
      
      }
   
   /**
   * Metode que ens permet obtenir el numero de soci del primer participant que encara
   * no hem consultat. L'element no s'elimina del sistema pero si d'aquest iterador. Per tant,
   * haurem de tornar a executar obteParticipantsActuacio si volem tornar a tractar tots els elements.
   *@return Enter amb el numero de soci del participant
   */
   
      public int obteParticipant()   
      
      {
      
         resultat=(Resultat)llistat.obtePrimerElement();
         return Integer.parseInt((String)resultat.obteElement("participant_numSoci"));
      
      }
   
   //////////Fi iterador\\\\\\\
                                                                                                                                                
   /**
   * Metode que permet afegir un participant, es a dir, relacionar un nou participant amb una actuacio.
   * Per tant, el nou participant formara part del programa de l'actuacio.
   *@param actuacio Enter amb l'identificador de l'actuacio
   *@param numSoci Enter amb l'identificador del soci
   */
   
      public static void afegirParticipantAActuacio(int actuacio,int numSoci)
      
      {
      
         Persistencia Persistencia = new Persistencia("entitats","participantsActuacio");
         ObjectePersistencia dadesNouParticipantPerActuacio = new ObjectePersistencia();
      
         dadesNouParticipantPerActuacio.afegirCampNumeric("actuacio_identificadorActuacio",actuacio);
         dadesNouParticipantPerActuacio.afegirCampNumeric("participant_numSoci",numSoci);
         dadesNouParticipantPerActuacio.afegirCampNumeric("ordre",++ordre);
      
         Persistencia.afegeixElement(dadesNouParticipantPerActuacio);
      
         dadesNouParticipantPerActuacio.destructorObjectePersistencia();
         Persistencia.destructorPersistencia();
      
      }
   
   /*
   * Metode que elimina la relacio d'una determinada actuacio amb tots els seus participants
   *@param identificadorActuacio Enter amb el valor de l'identificador de l'actuacio	
   */
      public static void eliminaParticipants(int identificadorActuacio)
      
      {
      
         Persistencia Persistencia = new Persistencia("entitats","participantsActuacio");
      
         Persistencia.eliminaPerCampNumeric("actuacio_identificadorActuacio",identificadorActuacio);
      
         Persistencia.destructorPersistencia();
      
      }
   
   }