   package Entitats.Actuacio;

   import Entitats.Persistencia.Persistencia;
   import Entitats.Persistencia.Llistat;
   import Entitats.Persistencia.Resultat;
   import Entitats.Persistencia.ObjectePersistencia;

/**
* Classe que permet mantenir una relacio entre actuacions i les peces associades a aquestes,
* les peces del repertori que es duen a terme 
*@author Jordi Moles i Blanco, Mar 2005
*/

   public class RelacioPecesActuacio
   {
   
      private Persistencia persistencia; //Objecte que ens permet obtenir informacio de la BBDD.
      private Llistat llistat; 
      private Resultat resultat;
      private static int ordre=0; 
   
   /**
   * Constructor de la classe. Inicialitza els elements necessaris per tal d'accedir a la BBDD.
   */
      public RelacioPecesActuacio()
      {
      
         persistencia = new Persistencia("entitats","pecesActuacio");
         llistat=null;
      
      }
   
   /**
   * Destructor de la classe
   * Metode que elimina les referencies als objectes manipulats per una RelacioPecesActuacio
   * per tal de que l'objecte "Gargabe Collector" del Java n'alliberi la memoria assignada
   * quan descobreixi que l'objecte ja no te cap referencia.
   */
      public void destructorRelacioPecesActuacio()
      {
      
         persistencia.destructorPersistencia();
         if(llistat!=null)llistat.destructorLlistat();
         resultat=null;
      
      }
   
   //////Els seguents metodes representen l'iterador de peces associades a una actuacio\\\\\
                                                                                                                  
   /**
   * Metode que cerca les peces que composen una actuacio donada l'identificador d'aquesta
   *@param identificadorActuacio Enter amb el valor que identifica una actuacio de manera unica
   */
      public void obtePecesActuacio(int identificadorActuacio)
      {
      
         llistat=new Llistat(persistencia.obtePerCampNumeric("actuacio_identificadorActuacio",identificadorActuacio));
      
      }
   
   /**
   * Metode que cerca les peces que composen una actuacio donada l'identificador d'aquesta i les 
   * ordena per el camp indicat
   *@param identificadorActuacio Enter amb el valor que identifica una actuacio de manera unica
   *@param campOrdre Nom del camp per el que volem ordenar els resultats
   */
      public void obtePecesActuacio(int identificadorActuacio,String campOrdre)
      {
      
         ObjectePersistencia campActuacioABuscar = new ObjectePersistencia();
         campActuacioABuscar.afegirCampNumeric("actuacio_identificadorActuacio",identificadorActuacio);
         llistat=new Llistat(persistencia.obtePerDiversosCampsAscPer(campActuacioABuscar,campOrdre));
         campActuacioABuscar.destructorObjectePersistencia();
      }
   
   /**
   * Metode que ens informa si hi ha peces en l'iterador que encara no hem consultat
   *@return Boolea amb valor positiu si hi ha elements i negatiu si no n'hi ha.
   */
      public boolean hiHaPeces() 
      {
      
         return llistat.hiHaElements();
      
      }
   
   /**
   * Metode que ens permet obtenir l'identificador de la primera pesa que encara
   * no hem consultat. L'element no s'elimina del sistema pero si d'aquest iterador. Per tant,
   * haurem de tornar a executar obtePecesActuacio si volem tornar a tractar tots els elements.
   *@return Enter amb l'identificador de la pesa
   */
      public int obtePesa()   
      {
      
         resultat=(Resultat)llistat.obtePrimerElement();
         return Integer.parseInt((String)resultat.obteElement("pesa_identificadorPesa"));
      
      }
   
   //////////Fi iterador\\\\\\\
                                                                                                                  
   /**
   * Metode que permet afegir una pesa, es a dir, relacionar una nova pesa amb una actuacio.
   * Per tant, la nova pesa formara part del programa de l'actuacio.
   *@param actuacio Enter amb l'identificador de l'actuacio
   *@param pesa Enter amb l'identificador de la pesa
   */
      public static void afegirPesaAActuacio(int actuacio,int pesa)
      {
      
         Persistencia Persistencia = new Persistencia("entitats","pecesActuacio");
         ObjectePersistencia dadesNovaPesaPerActuacio = new ObjectePersistencia();
      
         dadesNovaPesaPerActuacio.afegirCampNumeric("actuacio_identificadorActuacio",actuacio);
         dadesNovaPesaPerActuacio.afegirCampNumeric("pesa_identificadorPesa",pesa);
         dadesNovaPesaPerActuacio.afegirCampNumeric("ordre",++ordre);
      
         Persistencia.afegeixElement(dadesNovaPesaPerActuacio);
      
         dadesNovaPesaPerActuacio.destructorObjectePersistencia();
         Persistencia.destructorPersistencia();
      
      }
   
   /*
   * Metode que elimina la relacio d'una determinada actuacio amb totes les peces relacionades.
   *@param identificadorActuacio Enter amb el valor que defineix a una actuacio de manera unica en el sistema. 
   */
      public static void eliminaPeces(int identificadorActuacio)
      {
      
         Persistencia Persistencia = new Persistencia("entitats","pecesActuacio");
      
         Persistencia.eliminaPerCampNumeric("actuacio_identificadorActuacio",identificadorActuacio);
      
         Persistencia.destructorPersistencia();
      
      }
   
   }