   package Entitats.Imatge;

   import Entitats.Persistencia.Persistencia;
   import Entitats.Persistencia.Resultat;
   import Entitats.Imatge.Imatge;
   import Entitats.Persistencia.Llistat;
   import Entitats.Persistencia.ObjectePersistencia;

/**
* Classe que permet mantenir una relacio de imatges. Altes, baixes i
* consultes de imatges. Implementa els elements necessaris
* per accedir a la BBDD o qualsevol altre sistema que utilitzem per conservar les dades
* que manipulem de forma permanent. Aixo es gracies a l'us de la classe Persistencia, que 
* ens permet treballar amb imatges sense haver de preocupar-nos de com el sistema
* tracta les dades.
*@author Jordi Moles i Blanco, Mar 2006
*/

   public class RelacioImatges
   
   {
   
      private Persistencia persistencia;
      private Resultat resultat;
      private Llistat llistat;
   
   /**
   * Constructor de la classe. Inicialitza els elements necessaris per tal d'accedir a la BBDD.
   */
      public RelacioImatges()
      {
      
         persistencia = new Persistencia("entitats","imatge");
      
      }
   
   /**
   * Destructor de la classe
   * Metode que elimina les referencies als objectes manipulats per una RelacioImatges
   * per tal de que l'objecte "Gargabe Collector" del Java n'alliberi la memoria assignada
   * quan descobreixi que l'objecte ja no te cap referencia.
   */
      public void destructorRelacioImatges()
      {
      
         persistencia.destructorPersistencia();
         resultat=null;
         llistat.destructorLlistat();
      
      }
   
   //////Els seguents metodes representen l'iterador de imatges recuperades del sistema\\\\\
                                                                                             
      public void obteTotesLesImatges()
      {
      
         llistat = new Llistat(persistencia.obteTots());
      
      }
   
   /**
   * Metode que ens informa si hi ha imatges en l'iterador que encara no hem consultat
   *@return Boolea amb valor positiu si hi ha elements i negatiu si no n'hi ha.
   */
      public boolean hiHaImatges()
      {
      
         return llistat.hiHaElements();
      
      }
   
   /**
   * Metode que ens permet obtenir el primer objecte imatge que encara
   * no hem consultat, amb totes les dades disponibles sobre aquesta en el sistema.
   *@return Objecte imatge amb totes les dades disponibles
   */
      public Imatge obteImatge()
      {
      
         resultat=(Resultat)llistat.obtePrimerElement();
         Imatge imatge = new Imatge();
      
         imatge.estableixIdentificador(Integer.parseInt((String)resultat.obteElement("numImatge")));
         imatge.estableixRuta((String)resultat.obteElement("ruta"));
      
         return imatge;
      
      }
   
   //////////Fi iterador\\\\\\\
                                                                                                                                                          
   /**
   * Metode que ens permet obtenir una imatge a partir de l'identificador d'aquesta
   *@param identificadorImatge Enter amb el valor que identifica una imatge
   *de manera unica
   *@return Objecte imatge amb totes les dades disponibles de la imatge sol.licitada
   */
   
      public static Imatge obteImatgePerIdentificadorImatge(int identificadorImatge)
      {
      
         Imatge Imatge = new Imatge();
         Persistencia Persistencia = new Persistencia("entitats","imatge");
         Resultat Resultat;
         Llistat ll;
         ll = new Llistat(Persistencia.obtePerCampNumeric("numImatge",identificadorImatge));
         if(ll.hiHaElements())
         {
            Resultat=(Resultat)ll.obtePrimerElement();
            Imatge=emplenaImatge(Resultat);
         }
         Persistencia.destructorPersistencia();
         ll.destructorLlistat();
         Resultat=null;
      
         return Imatge;
      
      }
   
      private static Imatge emplenaImatge(Resultat Resultat)
      {
      
         Imatge Imatge = new Imatge();
      
         Imatge.estableixIdentificador(Integer.parseInt((String)Resultat.obteElement("numImatge")));
         Imatge.estableixRuta((String)Resultat.obteElement("ruta"));
      
         return Imatge;
      
      }
   
   /**
   * Metode que permet eliminar una imatge del sistema a partir del seu identificador
   *@param identificadorImatge Enter amb l'identificador de la imatge
   */
      public static void eliminaImatgePerIdentificadorImatge(int identificadorImatge)
      {
      
         Persistencia Persistencia = new Persistencia("entitats","imatge");
         Persistencia.eliminaPerCampNumeric("numImatge",identificadorImatge);
         Persistencia.destructorPersistencia();
      
      }
   
   /**
   * Metode que ens permet afegir una imatge a la relacio
   *@param imatge Objecte Imatge amb les dades de la nova imatge que volem afegir
   *al sistema
   *@return Enter amb el valor que el sistema genera per a aquest nou element, valor
   *que l'identificara de manera unica en el sistema.
   */
      public static int afegirImatge(Imatge imatge)
      {
      
         int identificador;
         Persistencia Persistencia = new Persistencia("entitats","imatge");
         ObjectePersistencia dadesNovaImatge = new ObjectePersistencia();
      
         dadesNovaImatge.afegirCampText("ruta",imatge.obteRuta());
         identificador=Persistencia.afegeixElement(dadesNovaImatge);
      
         dadesNovaImatge.destructorObjectePersistencia();
         Persistencia.destructorPersistencia();
      
         return identificador;
      
      }
   
   }