   package Entitats.Participant;

   import Entitats.Soci.RelacioSocis;
   import Entitats.Persistencia.ObjectePersistencia;
   import Entitats.Persistencia.Resultat; 
   import Entitats.Persistencia.Persistencia;
   import Entitats.Persistencia.Llistat;
   import Entitats.Temps.Data;
   import Entitats.Soci.Soci;

/**
* Classe que permet mantenir una relacio de Participants. Altes, baixes, modificacions i
* consultes de participants per qualsevol camp son possibles. Implementa els elements necessaris
* per accedir a la BBDD o qualsevol altre sistema que utilitzem per conservar les dades
* que manipulem de forma permanent. Aixo es gracies a l'us de la classe Persistencia, que 
* ens permet treballar amb participants sense haver de preocupar-nos de com el sistema
* tracta les dades.
*@author Jordi Moles i Blanco, Agost 2005
*/

   public class RelacioParticipants extends RelacioSocis
   {
   
   /**
   * Constructor de la classe. Inicialitza els elements necessaris per tal d'accedir a la BBDD.
   */
   
      public RelacioParticipants()
      {
      
         super();
      
      }
   
   /**
   * Destructor de la classe
   * Metode que elimina les referencies als objectes manipulats per una RelacioParticipants
   * per tal de que l'objecte "Gargabe Collector" del Java n'alliberi la memoria assignada
   * quan descobreixi que l'objecte ja no te cap referencia.
   */
   
      public void destructorRelacioParticipants()
      {
         super.destructorRelacioSocis();
      
      }
   
   //////Els seguents metodes representen l'iterador de participants recuperats del sistema\\\\\
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
   
   /**
   * Metode que ens permet obtenir tots els participants existents al sistema.
   */
      public void obteTotsElsParticipants()        
      {
      
         Persistencia Persistencia = new Persistencia("entitats","participant");
         llistat = new Llistat(Persistencia.obteTots());
         Persistencia.destructorPersistencia();
      
      }
   
   /**
   * Metode que ens permet obtenir un conjunt de participants l'any de naixement dels quals
   * coincideixi amb el valor donat.
   *@param AnyNaixement Enter amb el valor de l'any
   */
      public void obteParticipantPerAnyNaixement(int AnyNaixement)
      {
         Persistencia Persistencia = new Persistencia("entitats","participant");
         ObjectePersistencia dadesParticipantABuscar = new ObjectePersistencia();
         Resultat Resultat;
         Object sociCandidat;
         Llistat ll = new Llistat(persistencia.obtePerAny("dataNaixement",AnyNaixement));
         Llistat ll2=null;
      
         while(ll.hiHaElements())
         {
            sociCandidat=ll.obtePrimerElement();
            ll2 = new Llistat(Persistencia.obtePerCampNumeric("numSoci",Integer.parseInt((String)(((Resultat)sociCandidat).obteElement("numSoci")))));
            if(ll2.hiHaElements())llistat.afegeix(sociCandidat);
         }
      
         ll2.destructorLlistat();
         ll.destructorLlistat();
         Resultat=null;
         dadesParticipantABuscar.destructorObjectePersistencia();
         Persistencia.destructorPersistencia();
      }
   
   /**
   * Metode que ens permet obtenir un conjunt de participants l'any d'ingres dels quals
   * coincideixi amb el valor donat.
   *@param AnyAltaEntitat Enter amb el valor de l'any 
   */
      public void obteParticipantPerAnyIngresEntitat(int AnyAltaEntitat)
      {
      
         Persistencia Persistencia = new Persistencia("entitats","participant");
         ObjectePersistencia dadesParticipantABuscar = new ObjectePersistencia();
         Resultat Resultat;
         Object sociCandidat;
         Llistat ll = new Llistat(persistencia.obtePerAny("dataAltaEntitat",AnyAltaEntitat));
         Llistat ll2=null;
      
         while(ll.hiHaElements())
         {
            sociCandidat=ll.obtePrimerElement();
            ll2 = new Llistat(Persistencia.obtePerCampNumeric("numSoci",Integer.parseInt((String)(((Resultat)sociCandidat).obteElement("numSoci")))));
            if(ll2.hiHaElements())llistat.afegeix(sociCandidat);
         }
      
         ll2.destructorLlistat();
         ll.destructorLlistat();
         Resultat=null;
         dadesParticipantABuscar.destructorObjectePersistencia();
         Persistencia.destructorPersistencia();
      
      }
   
   /**
   * Metode que ens permet obtenir un conjunt de participants els quals van ser donats d'alta com a
   * tal en una data determinada.
   *@param dia Enter amb el valor del dia
   *@param mes Enter amb el valor del mes
   *@param any Enter amb el valor de l'any
   */
      public void obteParticipantPerDataAltaParticipant(int dia,int mes,int any)     
      {
         Persistencia Persistencia = new Persistencia("entitats","participant");
         Resultat Resultat;
         Llistat ll;
         Llistat llistatCopia = new Llistat();
         ll = new Llistat(Persistencia.obtePerCampData("dataAltaParticipant",dia,mes,any));
      
         while(ll.hiHaElements())
         {
            Resultat=(Resultat)ll.obtePrimerElement();
         
            llistatCopia.afegeix(RelacioSocis.obteSociPerNumSoci(Integer.parseInt((String)Resultat.obteElement("numSoci"))));
         }
      
         Persistencia.destructorPersistencia();
         ll.destructorLlistat();
         Resultat=null;
         llistat=(Llistat)llistatCopia.clone();
         llistatCopia.destructorLlistat();
      }
   
   /**
   * Metode que ens permet obtenir un conjunt de participants els quals van ser donats d'alta com a
   * tal en una data determinada.
   *@param dataAltaParticipant String amb un valor de data en format dd/mm/aaaa o dd-mm-aaaa  
   */
   
      public void obteParticipantPerDataAltaParticipant(String dataAltaParticipant)              
      {
      
         int dia,mes,any;
         int posicio1,posicio2;
      
         posicio1=dataAltaParticipant.indexOf("-");
         if(posicio1<0) posicio1=dataAltaParticipant.indexOf("/");
         dia=Integer.parseInt(dataAltaParticipant.substring(0,posicio1));
         posicio1++;
         posicio2=dataAltaParticipant.indexOf("-",posicio1);
         if(posicio2<0) posicio2=dataAltaParticipant.indexOf("/",posicio1);
         mes=Integer.parseInt(dataAltaParticipant.substring(posicio1,posicio2));
         posicio2++;
         any=Integer.parseInt(dataAltaParticipant.substring(posicio2).trim());
         obteParticipantPerDataAltaParticipant(dia,mes,any);      
      
      }
   
   /**
   * Metode que ens permet obtenir un conjunt amb tots els participants presents al sistema
    *@return Objecte Llistat amb tots els participants  
   */
      public Llistat obteLlistat()
      {
         return llistat;
      }
   
   /**
   * Metode que ens permet obtenir un conjunt de participants els quals compleixen una serie
   * de restriccions que s'estableixen
   *@param DadesParticipantABuscar ObjectePersistencia amb els camps i els valors per els que s'ha de buscar un participant  
   */
      public void obteParticipantPerMultiplesCamps(ObjectePersistencia DadesParticipantABuscar)   
      {
         ObjectePersistencia DadesParticipantABuscarCopia = (ObjectePersistencia)DadesParticipantABuscar.clone();
         Llistat llistatCopia = new Llistat();
         boolean buscarPerDataAltaParticipant = false;
      
         String dataAltaParticipant="";
         boolean nomesDataAlta = true;
         llistat = new Llistat();
         ObjectePersistencia DadesParticipantABuscarCopiaSenseData = new ObjectePersistencia();
         while(DadesParticipantABuscarCopia.hiHaCamps())
         {
            if(DadesParticipantABuscarCopia.obteCamp().trim().compareToIgnoreCase("dataAltaParticipant")==0)
            {
               dataAltaParticipant=DadesParticipantABuscarCopia.obteValor();
               buscarPerDataAltaParticipant = true;
            }
            else
            {
               if(DadesParticipantABuscarCopia.obteTipus().trim().compareToIgnoreCase("textcurt")==0 | DadesParticipantABuscarCopia.obteTipus().trim().compareToIgnoreCase("textllarg")==0)
               {DadesParticipantABuscarCopiaSenseData.afegirCampText(DadesParticipantABuscarCopia.obteCamp(),DadesParticipantABuscarCopia.obteValor());nomesDataAlta=false;}
               if(DadesParticipantABuscarCopia.obteTipus().trim().compareToIgnoreCase("nombre")==0)
               {DadesParticipantABuscarCopiaSenseData.afegirCampNumeric(DadesParticipantABuscarCopia.obteCamp(),Integer.parseInt(DadesParticipantABuscarCopia.obteValor()));nomesDataAlta=false;}
               if(DadesParticipantABuscarCopia.obteTipus().trim().compareToIgnoreCase("data")==0)
               {DadesParticipantABuscarCopiaSenseData.afegirCampDataUS(DadesParticipantABuscarCopia.obteCamp(),DadesParticipantABuscarCopia.obteValor());nomesDataAlta=false;}
               if(DadesParticipantABuscarCopia.obteTipus().trim().compareToIgnoreCase("hora")==0)
               {DadesParticipantABuscarCopiaSenseData.afegirCampHora(DadesParticipantABuscarCopia.obteCamp(),DadesParticipantABuscarCopia.obteValor());nomesDataAlta=false;}
            }
            DadesParticipantABuscarCopia.seguent();
         
         }
      
         Persistencia Persistencia = new Persistencia("entitats","participant");
         ObjectePersistencia dadesParticipantABuscar = new ObjectePersistencia();
         Resultat Resultat;
         Object sociCandidat;
      
         if(DadesParticipantABuscarCopiaSenseData.hiHaCamps())
         {
            Llistat ll = new Llistat(persistencia.obtePerDiversosCamps(DadesParticipantABuscarCopiaSenseData));
            while(ll.hiHaElements())
            {
               sociCandidat=ll.obtePrimerElement();
               ObjectePersistencia dadesParticipantABuscarNumSociIDataAltaParticipant = new ObjectePersistencia();
               dadesParticipantABuscarNumSociIDataAltaParticipant.afegirCampNumeric("numSoci",Integer.parseInt((String)(((Resultat)sociCandidat).obteElement("numSoci"))));
               if(buscarPerDataAltaParticipant)
               {
                  dadesParticipantABuscarNumSociIDataAltaParticipant.afegirCampDataUS("dataAltaParticipant",dataAltaParticipant);
                  Llistat ll2 = new Llistat(Persistencia.obtePerDiversosCamps(dadesParticipantABuscarNumSociIDataAltaParticipant));
                  if(ll2.hiHaElements())llistat.afegeix(sociCandidat);
                  ll2.destructorLlistat();
               
               }
               else
               { 
                  Llistat ll2 = new Llistat(Persistencia.obtePerDiversosCamps(dadesParticipantABuscarNumSociIDataAltaParticipant));
                  if(ll2.hiHaElements())llistat.afegeix(sociCandidat);
                  ll2.destructorLlistat();
               }
            
               dadesParticipantABuscarNumSociIDataAltaParticipant.destructorObjectePersistencia();
            
            }
            ll.destructorLlistat();
         }
      
         Resultat=null;
         dadesParticipantABuscar.destructorObjectePersistencia();
         DadesParticipantABuscarCopiaSenseData.destructorObjectePersistencia();
         DadesParticipantABuscarCopia.destructorObjectePersistencia();
         Persistencia.destructorPersistencia();
      
      }
   
   /**
   * Metode que ens informa si hi ha participants en l'iterador que encara no hem consultat
   *@return Boolea amb valor positiu si hi ha elements i negatiu si no n'hi ha.
   */
   
      public boolean hiHaParticipants()   
      {
      
         return super.hiHaSocis();
      
      }
   
   /**
   * Metode que ens permet obtenir el primer objecte participant que encara
   * no hem consultat, amb totes les dades disponibles sobre aquest en el sistema.
   *@return Objecte Participant amb totes les dades disponibles
   */
   
      public Participant obteParticipant()
      {
         Resultat resultat = (Resultat)llistat.obtePrimerElement();//System.out.println("e");
      
         int numSoci = Integer.parseInt((String)resultat.obteElement("numSoci"));
         Participant participant = obteParticipantPerNumSoci(numSoci);
      
         participant.estableixNumSoci(numSoci);
         participant.estableixNom(participant.obteNom());
         participant.estableixCognom1(participant.obteCognom1());
         participant.estableixCognom2(participant.obteCognom2());
         participant.estableixIdentificadorPersonal(participant.obteIdentificadorPersonal());
         participant.estableixAdresa(participant.obteAdresa());
         participant.estableixTelefon(participant.obteTelefon());
         participant.estableixDataNaixement(participant.obteDataNaixement());
         participant.estableixDataAltaEntitat(participant.obteDataAltaEntitat());
         participant.estableixDataAltaParticipant(participant.obteDataAltaParticipant());
         return participant;
      
      }
   
   //////////Fi iterador\\\\\\\
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          
   /**
   * Metode que ens permet obtenir un participant a partir del numero de soci
   *@param numSoci Enter amb el valor que identifica un soci/participant
   *de manera unica
   *@return Objecte Participant amb totes les dades disponibles del participant sol.licitat
   */
   
      public static Participant obteParticipantPerNumSoci(int numSoci)
      {
      
         Participant Participant = new Participant();
         Persistencia Persistencia = new Persistencia("entitats","soci");
         Resultat Resultat;
         Llistat ll;
         ll = new Llistat(Persistencia.obtePerCampNumeric("numSoci",numSoci));
         if(ll.hiHaElements())
         {
            Resultat=(Resultat)ll.obtePrimerElement();
            Participant=emplenaParticipant(Resultat);
         }
         Persistencia.destructorPersistencia();
         ll.destructorLlistat();
         Resultat=null;
      
         return Participant;
      
      }
   
   /**
   * Metode que ens permet obtenir un participant a partir de l'identificador personal
   *@param identificadorPersonal Enter amb el valor que identifica una persona
   *de manera unica
   *@return Objecte Participant amb totes les dades disponibles del partcipant sol.licitat
   */
   
      public static Participant obteParticipantPerIdentificadorPersonal(String identificadorPersonal)
      {
      
         Participant Participant = new Participant();
         Persistencia Persistencia = new Persistencia("entitats","soci");
         Resultat Resultat;
         Llistat ll;
         ll = new Llistat(Persistencia.obtePerCampText("identificadorPersonal",identificadorPersonal));
         if(ll.hiHaElements())
         {
            Resultat=(Resultat)ll.obtePrimerElement();
            Participant=emplenaParticipant(Resultat);
         }
         Persistencia.destructorPersistencia();
         ll.destructorLlistat();
         Resultat=null;
      
         return Participant;     
      
      }
   
   /**
   * Metode que ens permet saber si ja existeix un participant amb un numero de soci determinat. 
   *@param numSoci Enter amb el valor que identifica un soci de manera unica 
   *@return int Enter amb valor 0 si el participant no existeix o be el valor de numero de soci si aquest ja existeix
   */
      public static int Existeix(int numSoci)
      {
         Participant p = obteParticipantPerNumSoci(numSoci);
      
         if(p.obteNumSoci()==0)
         {
            p.destructorParticipant(); 
            return -1;
         }
         else 
         {
         
            Persistencia Persistencia = new Persistencia("entitats","participant");
            Llistat ll;
            ll = new Llistat(Persistencia.obtePerCampNumeric("numSoci",p.obteNumSoci()));
            if(!ll.hiHaElements())
            {
               p.destructorParticipant();
               Persistencia.destructorPersistencia();
               ll.destructorLlistat();
               return 0;
            }
            else
            { 
               Persistencia.destructorPersistencia();
               ll.destructorLlistat();
               p.destructorParticipant();
               return 1;
            }
         
         }}
   
   /**
   * Metode que ens permet saber si ja existeix un participant amb un nom determinat. 
   *@param nom String amb el valor del nom pel que hem de buscar el participant 
   *@param cognom1 String amb el valor del primer cognom pel que hem de buscar el participant 
   *@param cognom2 String amb el valor del segon cognom pel que hem de buscar el participant 
   *@return boolean amb valor TRUE si existeix i FALSE en cas contrari
   */
      public static boolean ExisteixAmbNoms(String nom,String cognom1,String cognom2)           
      {
      
         ObjectePersistencia o = new ObjectePersistencia();
         o.afegirCampText("nom",nom);
         o.afegirCampText("cognom1",cognom1);
         o.afegirCampText("cognom2",cognom2);
      
         RelacioSocis relacioSocis = new RelacioSocis(); 
         relacioSocis.obteSociPerMultiplesCamps(o);
      
         if(relacioSocis.hiHaSocis()) 
            return true;
         else 
            return false;
      
      }
   
   /**
   * Metode que ens permet saber si ja existeix un participant amb un identificador personal determinat. 
   *@param identificadorPersonal Enter amb el valor que identifica una persona de manera unica 
   *@return int Enter amb valor 0 si el participant no existeix o be el valor de numero de soci si aquest ja existeix
   */
      public static int Existeix(String identificadorPersonal)   
      {
      
         Participant p = obteParticipantPerIdentificadorPersonal(identificadorPersonal);
      
         if(p.obteNumSoci()==0)
         {
            p.destructorParticipant(); 
            return -1;
         }
         else 
         {
         
            Persistencia Persistencia = new Persistencia("entitats","participant");
            Llistat ll;
            ll = new Llistat(Persistencia.obtePerCampNumeric("numSoci",p.obteNumSoci()));
            if(!ll.hiHaElements())
            {
               p.destructorParticipant();
               Persistencia.destructorPersistencia();
               ll.destructorLlistat();
               return 0;
            }
            else
            { 
               Persistencia.destructorPersistencia();
               ll.destructorLlistat();
               p.destructorParticipant();
               return 1;
            }
         
         }
      
      }
   
   
      protected static Participant emplenaParticipant(Resultat Resultat)
      {
      
         Persistencia Persistencia = new Persistencia("entitats","participant");
      
         Participant Participant = new Participant();
      
         Participant.estableixNumSoci(Integer.parseInt((String)Resultat.obteElement("numSoci")));
         Participant.estableixNom((String)Resultat.obteElement("nom"));
         Participant.estableixCognom1((String)Resultat.obteElement("cognom1"));
         Participant.estableixCognom2((String)Resultat.obteElement("cognom2"));
         Participant.estableixIdentificadorPersonal((String)Resultat.obteElement("identificadorPersonal"));
         Participant.estableixAdresa((String)Resultat.obteElement("adresa"));
         Participant.estableixTelefon(Integer.parseInt((String)Resultat.obteElement("telefon")));
         Participant.estableixDataNaixement((String)Resultat.obteElement("dataNaixement"));
         Participant.estableixDataAltaEntitat((String)Resultat.obteElement("dataAltaEntitat"));
      
         Llistat ll = new Llistat(Persistencia.obtePerCampNumeric("numSoci",Integer.parseInt((String)Resultat.obteElement("numSoci"))));
         if(ll.hiHaElements())
         {
            Object Participant2 = ll.obtePrimerElement();
         
            Participant.estableixDataAltaParticipant((String)(((Resultat)Participant2).obteElement("dataAltaParticipant")));
         
         }
         else
         {
            Participant.estableixDataAltaParticipant("01/01/1000");
         }
      
         if((Integer.parseInt((String)(Resultat.obteElement("actiu"))))==0)
            Participant.inactiu();
         else Participant.actiu();
      
         return Participant;
      
      }
   
   /**
   * Metode que permet donar de baixa un participant del sistema a partir del numero de soci.
   * Se'n preserven les dades per raons estadistiques i de consistencia del sistema, pero, 
   * alhora, es "desactiva" per tal que aquestes dades no estiguin disponibles en un consulta
   *@param numSoci Enter amb el numero de soci
   *@return boolean amb valor true si s'ha eliminat correctament
   */
   
      public static boolean eliminaParticipantPerNumSoci(int numSoci)
      {
      
         boolean totOK;
      
         Persistencia Persistencia = new Persistencia("entitats","participant");
         totOK=Persistencia.eliminaPerCampNumeric("numSoci",numSoci);
         Persistencia.destructorPersistencia();
      
         return totOK; 
      
      }
   
   /**
   * Metode que permet donar de baixa un participant del sistema a partir de l'identificador personal.
   * Se'n preserven les dades per raons estadistiques i de consistencia del sistema, pero, 
   * alhora, es "desactiva" per tal que aquestes dades no estiguin disponibles en un consulta
   *@param identificadorPersonal String amb l'identificador personal de la persona
   *@return boolean amb valor true si s'ha eliminat correctament
   */
   
      public static boolean eliminaParticipantPerIdentificadorPersonal(String identificadorPersonal)   
      {
      
         boolean totOK;
      
         Persistencia Persistencia = new Persistencia("entitats","participant");
         totOK=eliminaParticipantPerNumSoci(obteParticipantPerIdentificadorPersonal(identificadorPersonal).obteNumSoci());
         Persistencia.destructorPersistencia();
      
         return totOK;
      
      }
   
   /**
   * Metode que permet actualitzar de cop diversos camps d'un participant
   *@param numSoci Enter amb el valor que ens permetra trobar el participant
   *que volem modificar 
   *@param dadesActualitzadesParticipant ObjectePersistencia amb els noms dels camps a modificar, aixi com
   *els valors d'aquests
   *@return boolean amb valor true si s'ha actualitzat correctament
   */
   
      public static boolean actualitzaDiversosCampsParticipantPerNumSoci(int numSoci,ObjectePersistencia dadesActualitzadesParticipant)
      {
      
         ObjectePersistencia DadesParticipantABuscarCopia = (ObjectePersistencia)dadesActualitzadesParticipant.clone();
         //Llistat llistatCopia = new Llistat();
         boolean actualitzarPerDataAltaParticipant = false;
      
         String dataAltaParticipant="";
         boolean nomesDataAlta = true;
         //llistat = new Llistat();
         ObjectePersistencia DadesParticipantABuscarCopiaSenseData = new ObjectePersistencia();
         while(DadesParticipantABuscarCopia.hiHaCamps())
         {
            if(DadesParticipantABuscarCopia.obteCamp().trim().compareToIgnoreCase("dataAltaParticipant")==0)
            {
               dataAltaParticipant=DadesParticipantABuscarCopia.obteValor();
               actualitzarPerDataAltaParticipant = true;
            }
            else
            {
               if(DadesParticipantABuscarCopia.obteTipus().trim().compareToIgnoreCase("textcurt")==0 | DadesParticipantABuscarCopia.obteTipus().trim().compareToIgnoreCase("textllarg")==0)
               {DadesParticipantABuscarCopiaSenseData.afegirCampText(DadesParticipantABuscarCopia.obteCamp(),DadesParticipantABuscarCopia.obteValor());nomesDataAlta=false;}
               if(DadesParticipantABuscarCopia.obteTipus().trim().compareToIgnoreCase("nombre")==0)
               {DadesParticipantABuscarCopiaSenseData.afegirCampNumeric(DadesParticipantABuscarCopia.obteCamp(),Integer.parseInt(DadesParticipantABuscarCopia.obteValor()));nomesDataAlta=false;}
               if(DadesParticipantABuscarCopia.obteTipus().trim().compareToIgnoreCase("data")==0)
               {DadesParticipantABuscarCopiaSenseData.afegirCampDataUS(DadesParticipantABuscarCopia.obteCamp(),DadesParticipantABuscarCopia.obteValor());nomesDataAlta=false;}
               if(DadesParticipantABuscarCopia.obteTipus().trim().compareToIgnoreCase("hora")==0)
               {DadesParticipantABuscarCopiaSenseData.afegirCampHora(DadesParticipantABuscarCopia.obteCamp(),DadesParticipantABuscarCopia.obteValor());nomesDataAlta=false;}
            }
            DadesParticipantABuscarCopia.seguent();
         
         }
      
         boolean totOK;
      
         Persistencia Persistencia = new Persistencia("entitats","soci");
         totOK = Persistencia.actualitzaDiversosCampsPerCampNumeric("numSoci",numSoci,DadesParticipantABuscarCopiaSenseData);
         Persistencia.destructorPersistencia();
      
         boolean totOK2=true;
      
      
         return totOK&totOK2;
      
      }
   
   /**
   * Metode que permet actualitzar de cop diversos camps d'un participant
   *@param identificadorPersonal String amb el valor que ens permetra trobar el participant
   *que volem modificar 
   *@param dadesActualitzadesParticipant ObjectePersistencia amb els noms dels camps a modificar, aixi com
   *els valors d'aquests
   *@return boolean amb valor true si s'ha actualitzat correctament
   */
   
      public static boolean actualitzaDiversosCampsParticipantPerIdentificadorPersonal(String identificadorPersonal,ObjectePersistencia dadesActualitzadesParticipant)
      {
      
         return actualitzaDiversosCampsParticipantPerNumSoci(obteParticipantPerIdentificadorPersonal(identificadorPersonal).obteNumSoci(),dadesActualitzadesParticipant);
      
      }
   
   /**
   * Metode que ens permet afegir un participant a la relacio quan aquest ja es soci
   *@param numSoci Enter amb el numero de soci
   */
      public static void afegirNomesParticipant(int numSoci)
      {
      
         Persistencia Persistencia = new Persistencia("entitats","participant");
         ObjectePersistencia dadesNouParticipant = new ObjectePersistencia();
         dadesNouParticipant.afegirCampData("dataAltaParticipant",(new Data()).obteData());
         dadesNouParticipant.afegirCampNumeric("numSoci",numSoci);
         Persistencia.afegeixElement(dadesNouParticipant);
      
         Persistencia.destructorPersistencia();
         dadesNouParticipant.destructorObjectePersistencia();
      
      }
   
   /**
   * Metode que ens permet afegir un participant a la relacio
   *@param participant Objecte Participant amb les dades del nou participant que volem afegir
   *al sistema
   *@return Enter amb el valor que el sistema genera per a aquest nou element, valor
   *que l'identificara de manera unica en el sistema.
   */
   
      public static int afegirParticipant(Participant participant)
      {
      
         int identificador;
         Persistencia Persistencia = new Persistencia("entitats","soci");
         Persistencia Persistencia2 = new Persistencia("entitats","usuariAplicacio");
         Persistencia Persistencia3 = new Persistencia("entitats","participant");
      
         ObjectePersistencia dadesNouSoci = new ObjectePersistencia();
         ObjectePersistencia dadesNouUsuari = new ObjectePersistencia();
         ObjectePersistencia dadesNouParticipant = new ObjectePersistencia();
      
         dadesNouSoci.afegirCampText("nom",participant.obteNom());
         dadesNouSoci.afegirCampText("cognom1",participant.obteCognom1());
         dadesNouSoci.afegirCampText("cognom2",participant.obteCognom2());
         dadesNouSoci.afegirCampText("identificadorPersonal",participant.obteIdentificadorPersonal());
         dadesNouSoci.afegirCampText("adresa",participant.obteAdresa());
         dadesNouSoci.afegirCampNumeric("telefon",participant.obteTelefon());
         dadesNouSoci.afegirCampData("dataNaixement",participant.obteDataNaixement());
         dadesNouSoci.afegirCampData("dataAltaEntitat",participant.obteDataAltaEntitat());
      
         dadesNouUsuari.afegirCampText("usuari",participant.obteNom());
         dadesNouUsuari.afegirCampText("contrasenya",participant.obteIdentificadorPersonal());
         dadesNouUsuari.afegirCampNumeric("nivell",1);
      
         identificador=Persistencia.afegeixElement(dadesNouSoci);
         Persistencia2.afegeixElement(dadesNouUsuari);
      
         dadesNouParticipant.afegirCampNumeric("numSoci",identificador);
         dadesNouParticipant.afegirCampData("dataAltaParticipant",participant.obteDataAltaParticipant());
         Persistencia3.afegeixElement(dadesNouParticipant);
      
         dadesNouSoci.destructorObjectePersistencia();
         dadesNouUsuari.destructorObjectePersistencia();
         dadesNouParticipant.destructorObjectePersistencia();
         Persistencia.destructorPersistencia();
         Persistencia2.destructorPersistencia();
         Persistencia3.destructorPersistencia();
      
         return identificador;
      
      }
   
   }