   package Entitats.Persistencia;

   import java.util.Vector;

/**
* Classe que gestiona un llistat d'elements, independentment de la naturalesa
* de l'objecte amb el que treballem. Un cop generat el llistat, es poden afegir
* i treure elements. 
*@author Jordi Moles i Blanco, Mar 2005
*/

   public class Llistat implements Cloneable 
   {
   
      private Vector llistat=null;
   
   /**
   * Constructor de la classe. La llista es crea a partir d'un conjunt d'elements que es proporcionen 
   */
      public Llistat(Vector llistat)
      {
      
         this.llistat=llistat;
      
      }
   
   /**
   * Constructor de la classe
   */
      public Llistat()
      
      {
      
         this.llistat=new Vector(50,50);
      
      }
   
   /**
   *Metode per obtenir una copia exacte del llistat amb el que treballem.
   *@return Object amb els elements que han de formar part del llistat
   */
      public Object clone()               
      
      {
      
         Object o=null;
      
         try
         {
         
            o = super.clone();
         }
         
         
            catch(Exception e) {
            
            
            
            
               System.err.println("(ObjectePersistencia1)Error al fer la copia de l'objecte: " + e.getMessage());
            
            }
      
         return o;
      
      }
   
      /**
   	*Destructor de la classe
   	*Metode que elimina les referencies als objectes manipulats per un llistat
   	*per tal de que l'objecte "Gargabe Collector" del Java n'alliberi la memoria assignada
   	*quan descobreixi que l'objecte ja no te cap referencia.
   	*/
      public void destructorLlistat()
      {
      
         if(llistat!=null)llistat.removeAllElements();
         llistat=null;
      
      }
   
   /**
   * Metode que afegeix un objecte al llistat, a la ultima posicio
   *@param o Objecte a afegir
   *@return boolea amb la confirmacio o no de que s'ha afegit. 
   */
      public boolean afegeix(Object o)   
      {
      
         return llistat.add(o);
      
      }
   
   /**
   * Metode que afegeix un objecte al llistat, a la posicio indicada
   *@param o Objecte a afegir
   *@param posicio Enter amb el valor de la posicio  
   */
      public void afegeixAPosicio(Object o,int posicio)   
      
      {
      
         if(llistat.size()<=posicio)
         {
            while(!(llistat.size()==posicio+1)) llistat.add("");
         }	
         llistat.add(posicio,o);
      }
   
   /**
   * Metode que ens informa si hi ha elements en el llistat que encara no hem consultat
   *@return Boolea amb valor positiu si hi ha elements i negatiu si no n'hi ha.
   */
      public boolean hiHaElements()
      {
      
         return llistat.size()>0;
      
      }
   
   /**
   * Metode que ens permet obtenir el primer element de la llista. En el moment que l'obtenim,
   * aquest deixa de ser part del llistat.
   *@return El primer objecte de la llista.
   */
      public Object obtePrimerElement()
      {
      
         Object objecte;
      
         objecte=llistat.elementAt(0);
      
         llistat.remove(0);
      
         return objecte;
      
      }
   
   /**
   * Metode que ens permet eliminar un objecte de la llista. 
   *@param o Objecte que volem eliminar del llistat.
   */
      public void eliminaElement(Object o)
      {
      
         if(llistat.contains(o)) llistat.remove(o);
      
      } 
   
   /**
   * Metode que ens permet eliminar un objecte de la llista. 
   *@param i Enter amb la posicio de l'element que volem eliminar.
   */
      public void eliminaElement(int i)           
      {
      
         llistat.remove(i);
      
      }
   
   /**
   * Metode que ens permet obtenir un objecte de la llista. 
   *@param i Enter amb la posicio de l'element que volem obtenir.
   *@return Object amb l'element demanat 
   */
      public Object obteElement(int i)       
      {
      
         return llistat.elementAt(i);
      
      }
   
   /**
   * Metode que ens permet obtenir la mida del llistat. 
   *@return Enter amb el nombre d'elements del llistat.
   */
      public int quants()
      {
         return llistat.size();
      }
   
   /**
   * Metode que ens permet obtenir la posicio d'un element. 
   *@return Enter amb la posicio.
   *@param o Object amb l'objecte a buscar
   */
      public int buscaElement(Object o)
      {
      
         return llistat.indexOf(o);
      
      } 
   
   }