   package Entitats.Persistencia;

   import java.util.Vector;
   import java.lang.Cloneable;

/**
* Classe indispensable per transferir les dades que volem afegir/actualitzar
* del sistema a la classe Persistencia. L'objecte conte un conjunt d'elements que contenen
* el nom del camp a afegir/modificar i el seu valor i tambe el tipus. El tipus es potser el
* menys clar de tots, pero te sentit quan pensem que depenent del sistema de BBDD amb el que 
* treballem les dades es manipulen de manera diferent. Per exemple, en diversos sistemes, quan 
* es fa un INSERT, les dates s'han de presentar en el format aaaa-mm-dd. En canvi, en altres,
* ho hem de fer amb el format mm-dd-aaaa. Per tant, l'idea d'aquesta classe, juntament
* amb Persistencia es fer la resta de classes de l'aplicacio totalment independents de la BBDD
* sobre la que funciona la nostra aplicacio. Finalment, val la pena comentar que les dades
* amb que aquest objecte pot contenir poden ser xifres, dates, hores, texts curts i 
* texts llargs. La distincio entre texts curts i llargs, com el seu nom indica nomes te sentit
* per a la llargada del contingut. En certs sistemes, com ara MySQL, els camps de text 
* amb llargades inferiors s'han d'especificar de diferent manera que la resta. Si intentem
* inserir mes de 250 caracters en un camp text especificat com a "text curt" obtindrem 
* un error de la BBDD. L'usuari, pero, no s'ha de preocupar d'aixo.
*@author Jordi Moles i Blanco, Mar 2005
*/


   public class ObjectePersistencia implements Cloneable
   {
   
      private Vector objectePersistencia;
      int llargadaTextCurt; //Valor que obte un valor diferent depenent del sistema de BBDD.
   
   /**
   * Constructor de la classe
   */
      public ObjectePersistencia()
      {
      
         objectePersistencia = new Vector();
         llargadaTextCurt=250;
      
      }
   
   /**
   *Destructor de la classe
   *Metode que elimina les referencies als objectes manipulats per un ObjectePersistencia
   *per tal de que l'objecte "Gargabe Collector" del Java n'alliberi la memoria assignada
   *quan descobreixi que l'objecte ja no te cap referencia.
   */
      public void destructorObjectePersistencia()
      {
      
         objectePersistencia.removeAllElements();
         objectePersistencia=null;
      
      }
   
      public Object clone()               
      {
      
         Object o=null;
      
         try
         {
         
            o = super.clone();
         }
         
            catch(Exception e) {
            
            
            
               System.err.println("(ObjectePersistencia1)Error al fer la copia de l'objecte: " + e.getMessage());
            
            }
      
         return o;
      
      }
   
   /**
   * Metode que ens permet afegir un campt text, independentment de si es curt o llarg
   *@param camp nom del camp a modificar/afegir
   *@param valor valor del camp a modificar/afegir
   */
      public void afegirCampText(String camp,String valor)
      {
      
         if(valor.length()<=llargadaTextCurt)
            objectePersistencia.add(camp+";;"+valor+";;textCurt;;");
         else
            afegirCampDescripcio(camp,valor);	
      
      }
   
      private void afegirCampDescripcio(String camp,String valor)
      {
      
         objectePersistencia.add(camp+";;"+valor+";;textLlarg;;");
      
      }
   
   /**
   * Metode que ens permet afegir un camp numeric.
   *@param camp String amb el nom del camp a modificar/afegir
   *@param valor Enter amb el valor del camp a modificar/afegir
   */
      public void afegirCampNumeric(String camp,int valor)
      {
      
         objectePersistencia.add(camp+";;"+valor+";;nombre;;");
      
      }
   
   /**
   * Metode que ens permet afegir una data
   *@param camp String amb el nom del camp a modificar/afegir 
   *@param dia Enter amb el valor del dia
   *@param mes Enter amb el valor del mes
   *@param any Enter amb el valor de l'any
   */
      public void afegirCampData(String camp,int dia,int mes,int any)
      {
      
         objectePersistencia.add(camp+";;"+any+"-"+mes+"-"+dia+";;data;;");
      
      }
   
   /**
   * Metode que ens permet afegir una data
   *@param camp String amb el nom del camp a modificar/afegir
   *@param valor String amb el valor de la data. Aquest ha de tenir el format
   * dd-mm-aaaa o be dd/mm/aaaa
   */
      public void afegirCampData(String camp,String valor)
      {
      
         String dia,mes,any;
         int posicio1,posicio2;
      
         posicio1=valor.indexOf("-");
         if(posicio1<0) posicio1=valor.indexOf("/");
         dia=valor.substring(0,posicio1);
      
         posicio1++;
         posicio2=valor.indexOf("-",posicio1);
         if(posicio2<0) posicio2=valor.indexOf("/",posicio1);
         mes=valor.substring(posicio1,posicio2);
      
         posicio2++;
         any=valor.substring(posicio2).trim();
      
      
         objectePersistencia.add(camp+";;"+any+"-"+mes+"-"+dia+";;data;;");
      
      }
   
   /**
   * Metode que ens permet afegir una data
   *@param camp String amb el nom del camp a modificar/afegir
   *@param valor String amb el valor de la data. Aquest ha de tenir el format
   * aaaa-mm-dd o be aaaa/dd/mm
   */
   
   
      public void afegirCampDataUS(String camp,String valor)
      
      {
      
         String dia,mes,any;
         int posicio1,posicio2;
      
         posicio1=valor.indexOf("-");
         if(posicio1<0) posicio1=valor.indexOf("/");
         any=valor.substring(0,posicio1);
      
         posicio1++;
         posicio2=valor.indexOf("-",posicio1);
         if(posicio2<0) posicio2=valor.indexOf("/",posicio1);
         mes=valor.substring(posicio1,posicio2);
      
         posicio2++;
         dia=valor.substring(posicio2).trim();
      
      
         objectePersistencia.add(camp+";;"+any+"-"+mes+"-"+dia+";;data;;");
      
      }	
   
   /**
   * Metode que ens permet afegir una hora o temps, segons la necessitat,
   * fins a detall de segons. Pot ser l'hora que comensa un determinat esdeveniment
   * o be el temps de durada, per el qual sera interessant poder enregistrar els segons.
   *@param camp String amb el nom del camp a modificar/afegir
   *@param hores Enter amb el valor de les hores que un esdeveniment dura o be l'hora en 
   *que comensa
   *@param minuts Enter amb el valor dels minuts que un esdeveniment dura o be el minut en que
   *comensa
   *@param segons Enter amb el valor dels segons que un esdeveniment dura 
   */
      public void afegirCampHora(String camp,int hores,int minuts,int segons)
      {
      
         objectePersistencia.add(camp+";;"+hores+":"+minuts+":"+segons+";;hora;;");
      
      }
   
   /**
   * Metode que ens permet afegir una hora o temps, segons la necessitat,
   * fins a detall de segons. Pot ser la hora que comensa un determinat esdeveniment
   * o be el temps de durada, per el qual sera interessant poder enregistrar els segons.
   *@param camp String amb el nom del camp a modificar/afegir
   *@param valor String amb el valor de la hora en un dels seguents formats:
   * hh:mm:ss o hh.mm.ss
   */
      public void afegirCampHora(String camp,String valor)
      {
      
         String hores,minuts,segons;
         int posicio1,posicio2;
      
         posicio1=valor.indexOf(".");
         if(posicio1<0) posicio1=valor.indexOf(":");
         hores=valor.substring(0,posicio1);
      
         posicio1++;
         posicio2=valor.indexOf(".",posicio1);
         if(posicio2<0) posicio2=valor.indexOf(":",posicio1);
         minuts=valor.substring(posicio1,posicio2);
      
         posicio2++;
         segons=valor.substring(posicio2).trim();
      
         objectePersistencia.add(camp+";;"+hores+":"+minuts+":"+segons+";;hora;;");
      
      }
   
   /**
   * Metode que ens permet afegir una hora o temps, segons la necessitat,
   * fins a detall de minuts. Pot ser la hora que comensa un determinat esdeveniment
   * o be el temps de durada.
   *@param camp String amb el nom del camp a modificar/afegir
   *@param valor String amb el valor de la hora en un dels seguents formats:
   * hh:mm o hh.mm
   */
      public void afegirCampHoraSenseSegons(String camp,String valor)
      
      {
      
         afegirCampHora(camp,valor+":00");
      
      }
   
   //////Els seguents metodes representen l'iterador d'elements recuperats de la BBDD\\\\\
                                                                                                                                                                                                                                       
   /**
   * Metode que ens informa si hi ha elements en l'iterador que encara no hem consultat
   *@return Boolea amb valor positiu si hi ha elements i negatiu si no n'hi ha.
   */
      public boolean hiHaCamps()
      {
      
         return objectePersistencia.size()>0;
      
      }
   
   /**
   * Metode que elimina el primer element del conjunt d'elements que hem obtingut
   * de la BBDD.
   */
      public void seguent()
      {
      
         objectePersistencia.remove(0);
      
      }
   
   /**
   * Metode que ens permet obtenir el nom del camp del primer element que encara
   * no s'ha tractat.
   *@return String amb el nom del camp
   */
      public String obteCamp()
      {
      
         String camp,aux;
         int posicio;
      
         aux=(String)objectePersistencia.elementAt(0);
         posicio=aux.indexOf(";;");
      
         camp=aux.substring(0,posicio);
      
         return camp;
      
      }
   
   /**
   * Metode que ens permet obtenir el valor del camp del primer element
   * sense tractar
   *@return String amb el valor del camp
   */
      public String obteValor()
      {
      
         String valor,aux;
         int posicio1,posicio2;
      
         aux=(String)objectePersistencia.elementAt(0);
         posicio1=aux.indexOf(";;");
         posicio1+=2;
         posicio2=aux.indexOf(";;",posicio1);
      
         valor=aux.substring(posicio1,posicio2);
      
         return valor;
      
      }
   
   /**
   * Metode que ens permet obtenir el tipus de camp del primer element del conjunt
   * que encara no hem tractat
   *@return String amb el valor del tipus de camp: text curt, text llarg, nombre, data, hora.
   */
      public String obteTipus()
      
      {
      
         String valor,aux;
         int posicio1,posicio2,posicio3;
      
         aux=(String)objectePersistencia.elementAt(0);
         posicio1=aux.indexOf(";;");
         posicio1+=2;
         posicio2=aux.indexOf(";;",posicio1);
         posicio2+=2;
         posicio3=aux.indexOf(";;",posicio2);
      
         valor=aux.substring(posicio2,posicio3);
      
         return valor;
      
      }
   
   	//////Fi de l'iterador\\\\\\                                                                                                                          
   }