   package Entitats.Persistencia;

   import java.sql.*;
   import java.util.Date;
   import java.text.SimpleDateFormat;
   import java.util.Vector;
   import Entitats.Persistencia.ObjectePersistencia;
   import java.io.File;
   import java.io.FileInputStream;
   import java.io.InputStreamReader;
   import java.io.BufferedReader;
   import java.sql.SQLException;
   import ch.abacus.lib.ui.renderer.abaRenderer.AbaRenderer;
   import javax.swing.JButton;
   import java.awt.*;
   import java.awt.event.*;
   import javax.swing.*;

/**
* Classe que implementa el codi necessari per tal de consultar informacio de la BBDD, 
* aixi com tambe modificar aquella informacio que desitjem o afegir-ne de nova.
* La classe esta pensada per fer de "missatger" entre l'aplicacio i la BBDD, 
* la qual cosa significa que, per exemple, no es comprova que els camps que es volen
* consultar d'una determinada taula existeixin. Posem per cas que volem fer una consulta: A 
* traves d'un objecte ObjectePersistencia les demes classes de l'aplicacio li comuniquen 
* a aquesta els elements que es volen consultar. Persistencia llegeix d'aquest objecte
* un nom de camp i un valor associat a aquest i executa una sentencia SQL sense 
* saber si aquell camp realment existeix o si el valor es valid per aquell camp.
* Un cop s'executa una alta,baixa o modificacio d'una actuacio, els 
* canvis realitzats s'executen a la BBDD. Per tant, la possibilitat de desfer canvis depen
* exclusivament del sistema de gestio de BBDD amb que es treballa. En qualsevol cas, aquesta
* classe no implementa aquesta possibilitat.
*@author Jordi Moles i Blanco, Mar 2005
*/

   public class Persistencia
   {
   
      private String baseDeDades; //Base de dades a la que l'objecte s'ha de conectar
      private String taula; //Nom de la taula a la que s'ha d'accedir
      private Connection conexio;
      private ResultSet resultat;
      private SimpleDateFormat formatData;
      private SimpleDateFormat formatHora;
   
   /**
   * Constructor de la classe. En aquest punt s'intenta crear conexio a la BBDD. 
   * L'objecte romandra conectat a la BBDD fins que no s'executi el destructor de la classe
   */
      public Persistencia(String baseDeDades,String taula)
      {
      
         this.baseDeDades=baseDeDades;
         this.taula=taula;
         resultat=null;
         formatData=new SimpleDateFormat();
         formatHora=new SimpleDateFormat();
         formatData.applyPattern("dd/MM/yyyy");
         formatHora.applyPattern("HH:mm:ss");
      
         try
         {
         
         //ara intentarem obtenir el login i password de la bbdd.
         
            File f = new File("../Persistencia/config.dat");
            FileInputStream fis = new FileInputStream(f.getAbsolutePath());
            InputStreamReader isr = new InputStreamReader(fis);
            BufferedReader br = new BufferedReader(isr);
            String linia,login,pas,server,port;
         
            int poslogin,pospas,posdeflogin,posdefpas,posserver,posport,posdefserver,posdefport;
         
            poslogin = pospas = posdeflogin = posdefpas = posserver = posport = posdefserver = posdefport = -1;
            login = pas = "";
            server = port = "";
         
            do
            {
               linia = br.readLine();
               if(linia!=null)
               {
               
                  poslogin = linia.trim().indexOf("login=");
                  pospas = linia.trim().indexOf("pas=");
                  posserver = linia.trim().indexOf("ip=");
                  posport = linia.trim().indexOf("port=");
               
                  if(poslogin>=0)
                  {
                     login = linia.trim().substring(poslogin+6);
                     posdeflogin = poslogin;
                  }
                  if(pospas>=0)
                  {
                     pas = linia.trim().substring(pospas+4);
                     posdefpas = pospas;
                  }
                  if(posserver>=0)
                  {
                     server = linia.trim().substring(posserver+3);
                     posdefserver = posserver;
                  }
                  if(posport>=0)
                  {
                     port = linia.trim().substring(posport+5);
                     posdefport = posport;
                  }
               
               }
            }while(linia!=null);
         
            if(posdeflogin>=0 & posdefpas>=0){
               Class.forName("com.mysql.jdbc.Driver").newInstance();
               conexio = DriverManager.getConnection("jdbc:mysql://"+server+":"+port+"/"+baseDeDades, login, pas);
            }
            else System.out.println("Error al conectar a la Base de Dades amb l'usuari i contrasenya guardat.\nSiusplau,revisa les dades.");
         
         }
            catch(SQLException e) {
            
               errorConexio();
               System.err.println("(Persistencia1)Error acces a BBDD: " + e.getMessage());
            
            }
            catch(Exception e) {
            
               System.err.println("(Persistencia2)Error acces a BBDD: " + e.getMessage());
            
            } 
      
      }
   
   /**
   * Destructor de la classe
   * Metode que elimina les referencies als objectes manipulats per un objecte Persistencia
   * per tal de que l'objecte "Gargabe Collector" del Java n'alliberi la memoria assignada
   * quan descobreixi que l'objecte ja no te cap referencia. Tambe s'executa
   * la desconexio de la BBDD.
   */
      public void destructorPersistencia()
      {
      
         try {
         
            if(conexio != null)
               conexio.close();
         
         } 
         
            catch(Exception e) {
            
               System.err.println("(Persistencia3)Error acces a BBDD: " + e.getMessage());
            
            }
      
         conexio=null;
         resultat=null;
         formatHora=null;
         formatData=null;
      
      }
   
      private void errorConexio()
      {
      
         try
         {
            JButton acceptar;
         
            File docFile = new File(".././GUI/SGEC_GUI_errorbbdd.jar");
            String nomFitxer = docFile.getAbsolutePath();
            docFile=null;
            final AbaRenderer pantalla = new AbaRenderer(nomFitxer, false, null);
            pantalla.load();
            pantalla.renderInterface();
         
         	///////////BOTO ACCEPTAR\\\\\\\\\\\
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            
            acceptar=(JButton)(pantalla.getObject("acceptar"));
         
            acceptar.addActionListener(
                                 
                                    new ActionListener()
                                    {
                                    
                                       public void actionPerformed(ActionEvent ae)
                                       {
                                       
                                          ((JFrame)(pantalla.getObject("errorbbdd"))).dispose();
                                       
                                       }
                                    
                                    });
         }
            catch(Exception e) {
               System.err.println("(Persistencia4)Error acces a BBDD: " + e.getMessage());
            
            }
      
      }
   
      private Resultat emplenaResultat(ResultSetMetaData resMD,int columnes)
      {
      
         Resultat res = new Resultat();
         int comptador;
         Statement consulta;
         String dataString;
         String horaString;
      
         try
         {
         
            for(comptador=1;comptador<=columnes;comptador++)
            {
            
               if(0==((resMD.getColumnTypeName(comptador)).compareToIgnoreCase("date")))
               {
               
                  dataString=formatData.format(resultat.getDate(comptador));
                  //System.out.println(dataString);
                  res.afegeixElement(resMD.getColumnName(comptador),dataString);
               
               }
               if(0==((resMD.getColumnTypeName(comptador)).compareToIgnoreCase("time")))
               {
                  horaString=formatHora.format(resultat.getTime(comptador));
                  //System.out.println(horaString);
                  res.afegeixElement(resMD.getColumnName(comptador),horaString);
               
               }
               else{
                  //System.out.println(resultat.getString(comptador));
                  res.afegeixElement(resMD.getColumnName(comptador),resultat.getString(comptador));
               
               }
            
            }
         
         }
         
            catch(Exception e) {
            
               System.err.println("(Persistencia5)Error acces a BBDD: " + e.getMessage());
            
            }
      
         consulta=null;
      
         return res;
      
      }
   
   /*
   * Metode que permet recuperar tots els elements de la taula amb la que esta relacionada aquest objecte
   *@return Vector amb tots els elements relacionats amb aquesta taula
   */
   
      public Vector obteTots()
      {
      
         Resultat res;
         Vector vectorDeResultats = new Vector();
         ResultSetMetaData resMD;
         Statement consulta;
         int columnes;
      
         columnes=0;
      
         try
         {
         
            consulta = conexio.createStatement(); 
            resultat = consulta.executeQuery("SELECT * FROM "+taula);
         
            if(resultat.first())
            {
            
               resMD=resultat.getMetaData();
               columnes=resMD.getColumnCount();
            
               while(!resultat.isAfterLast())
               {
               
                  res=emplenaResultat(resMD,columnes);
               
                  vectorDeResultats.add(res);
               
                  resultat.next();
               
               }
            
            }
         
         }
         
         
            catch(Exception e) {
            
            
               System.err.println("(Persistencia6)Error acces a BBDD: " + e.getMessage());
            
            }
      
         res=null;
         resMD=null;
         consulta=null;
      
         return vectorDeResultats;
      
      }
   
   /*
   * Metode que permet recuperar tots els elements de la taula amb la que esta relacionada aquest objecte, ordenats descendentment
   *@return Vector amb tots els elements relacionats amb aquesta taula
   *@param campsAOrdenar ObjectePersistencia amb els camps per els que volem ordenar la cerca
   */
   
      public Vector obteTotsOrdenatsDescendentment(ObjectePersistencia campsAOrdenar)
      {
      
         Resultat res;
         Vector vectorDeResultats = new Vector();
         ResultSetMetaData resMD;
         Statement consulta;
         int columnes;
         String cadenaAExecutar="";
         String cadenaAExecutar2="";
      
         cadenaAExecutar="SELECT * FROM "+taula+" ORDER BY ";
      
         if(campsAOrdenar.hiHaCamps())
         {
         
            cadenaAExecutar2=campsAOrdenar.obteCamp()+" DESC";
            campsAOrdenar.seguent();
         
         }
      
         while(campsAOrdenar.hiHaCamps())
         {
         
            cadenaAExecutar2+=","+campsAOrdenar.obteCamp()+" DESC";
            campsAOrdenar.seguent();
         
         }
      
         cadenaAExecutar+=cadenaAExecutar2;
      
         columnes=0;
      
         try
         {
         
            consulta = conexio.createStatement(); 
            resultat = consulta.executeQuery(cadenaAExecutar);
         
            if(resultat.first())
            {
            
               resMD=resultat.getMetaData();
               columnes=resMD.getColumnCount();
            
               while(!resultat.isAfterLast())
               {
               
                  res=emplenaResultat(resMD,columnes);
               
                  vectorDeResultats.add(res);
               
                  resultat.next();
               
               }
            
            }
         
         }
         
            catch(Exception e) {
            
               System.err.println("(Persistencia7)Error acces a BBDD: " + e.getMessage());
            
            }
      
         res=null;
         resMD=null;
         consulta=null;
      
         return vectorDeResultats;
      
      }
   
   /*
   * Metode que permet recuperar tots els elements de la taula amb la que esta relacionada aquest objecte, ordenats ascendentment
   *@return Vector amb tots els elements relacionats amb aquesta taula
   *@param campsAOrdenar ObjectePersistencia amb els camps per els que hem d'ordenar els resultats
   */
   
      public Vector obteTotsOrdenatsAscendentment(ObjectePersistencia campsAOrdenar)
      {
      
         Resultat res;
         Vector vectorDeResultats = new Vector();
         ResultSetMetaData resMD;
         Statement consulta;
         int columnes;
         String cadenaAExecutar="";
         String cadenaAExecutar2="";
      
         cadenaAExecutar="SELECT * FROM "+taula+" ORDER BY ";
      
         if(campsAOrdenar.hiHaCamps())
         {
         
            cadenaAExecutar2=campsAOrdenar.obteCamp()+" ASC";
            campsAOrdenar.seguent();
         
         }
      
         while(campsAOrdenar.hiHaCamps())
         {
         
            cadenaAExecutar2+=","+campsAOrdenar.obteCamp()+" ASC";
            campsAOrdenar.seguent();
         
         }
      
         cadenaAExecutar+=cadenaAExecutar2;
      
         columnes=0;
      
         try
         {
         
            consulta = conexio.createStatement(); 
            resultat = consulta.executeQuery(cadenaAExecutar);
         
            if(resultat.first())
            {
            
               resMD=resultat.getMetaData();
               columnes=resMD.getColumnCount();
            
               while(!resultat.isAfterLast())
               {
               
                  res=emplenaResultat(resMD,columnes);
               
                  vectorDeResultats.add(res);
               
                  resultat.next();
               
               }
            
            }
         
         }
         
         
            catch(Exception e) {
            
            
               System.err.println("(Persistencia8)Error acces a BBDD: " + e.getMessage());
            
            }
      
         res=null;
         resMD=null;
         consulta=null;
      
         return vectorDeResultats;
      
      }
   
   /**
   * Metode que ens permet recuperar elements de la BBDD a partir d'un camp de text
   * determinat que especifiquem. Ja que els camps de text solen contenir valors repetits
   * al llarg de la taula (EX. localitat de residencia), el metode retorna un vector amb 
   * tots els elements el camp dels quals coincideix amb el valor aportat.
   *@param camp String amb el nom del camp per el qual hem de cerca l'element
   *@param valor String amb el valor del camp
   *@return Vector amb tots els elements que coincideixen amb els valors aportats
   */
      public Vector obtePerCampText(String camp,String valor)
      {
      
         Resultat res;
         Vector vectorDeResultats = new Vector();
         ResultSetMetaData resMD;
         Statement consulta;
         int columnes;
      
         columnes=0;
      
         try
         {
         
            consulta = conexio.createStatement(); 
            resultat = consulta.executeQuery("SELECT * FROM "+taula+" WHERE "+camp+"='"+valor+"'");
         
            if(resultat.first())
            {
            
               resMD=resultat.getMetaData();
               columnes=resMD.getColumnCount();
            
               while(!resultat.isAfterLast())
               {
               
                  res=emplenaResultat(resMD,columnes);
               
                  vectorDeResultats.add(res);
               
                  resultat.next();
               
               }
            
            }
         
         }
         
            catch(Exception e) {
            
               System.err.println("(Persistencia9)Error acces a BBDD: " + e.getMessage());
            
            }
      
         res=null;
         resMD=null;
         consulta=null;
      
         return vectorDeResultats;
      
      }
   
   /**
   * Metode que ens permet recuperar elements de la BBDD a partir d'un camp numeric
   * determinat que especifiquem. Ja que a priori no sabem si els valors d'aquest camp
   * son unics, el metode retorna un vector amb 
   * tots els elements el camp dels quals coincideix amb el valor aportat.
   *@param camp String amb el nom del camp per el qual hem de cerca l'element
   *@param valor Enter amb el valor del camp
   *@return Vector amb tots els elements que coincideixen amb els valors aportats
   */
      public Vector obtePerCampNumeric(String camp,int valor)
      {
      
         return obtePerCampText(camp,new Integer(valor).toString());
      
      }
   
   /**
   * Metode que ens permet recuperar elements de la BBDD a partir d'un camp data
   * determinat que especifiquem. Ja que a priori no sabem si els valors d'aquest camp
   * son unics(normalment no ho son, pensem per exemple en una data de naixement), 
   * el metode retorna un vector amb tots els elements el camp dels quals 
   * coincideix amb el valor aportat.
   *@param camp String amb el nom del camp per el qual hem de cerca l'element
   *@param dia Enter amb el valor del dia
   *@param mes Enter amb el valor del mes
   *@param any Enter amb el valor de l'any
   *@return Vector amb tots els elements que coincideixen amb els valors aportats
   */
      public Vector obtePerCampData(String camp,int dia,int mes,int any)
      {
      
         return obtePerCampText(camp,any+"-"+mes+"-"+dia);
      
      }
   
   	/**
   * Metode que ens permet recuperar elements de la BBDD a partir d'un camp data
   * determinat que especifiquem. Ja que a priori no sabem si els valors d'aquest camp
   * son unics(normalment no ho son, pensem per exemple en una data de naixement), 
   * el metode retorna un vector amb tots els elements el camp dels quals 
   * coincideix amb el valor aportat.
   *@param camp String amb el nom del camp per el qual hem de cerca l'element
   *@param data String amb el valor de la data a cercar
   *@return Vector amb tots els elements que coincideixen amb els valors aportats
   */
   
      public Vector obtePerCampData(String camp,String data)
      {
      
         int dia,mes,any;
         int posicio1,posicio2;
      
         posicio1=data.indexOf("-");
         if(posicio1<0) posicio1=data.indexOf("/");
         dia=Integer.parseInt(data.substring(0,posicio1));
      
         posicio1++;
         posicio2=data.indexOf("-",posicio1);
         if(posicio2<0) posicio2=data.indexOf("/",posicio1);
         mes=Integer.parseInt(data.substring(posicio1,posicio2));
      
         posicio2++;
         any=Integer.parseInt(data.substring(posicio2).trim());
      
         return obtePerCampData(camp,dia,mes,any);
      
      }
   
   /**
   * Metode que ens permet recuperar elements de la BBDD a partir d'un camp data
   * determinat que especifiquem. Ja que a priori no sabem si els valors d'aquest camp
   * son unics(normalment no ho son, pensem per exemple en una data de naixement), 
   * el metode retorna un vector amb tots els elements el camp dels quals 
   * coincideix amb el valor aportat.
   *@param camp String amb el nom del camp per el qual hem de cerca l'element
   *@param valor String amb el valor de la data a cercar, en format aaaa-mm-dd
   *@return Vector amb tots els elements que coincideixen amb els valors aportats
   */
   
      public Vector obtePerCampDataUS(String camp,String valor)
      
      
      {
      
         String dia,mes,any;
         int posicio1,posicio2;
      
         posicio1=valor.indexOf("-");
         if(posicio1<0) posicio1=valor.indexOf("/");
         any=valor.substring(0,posicio1);
      
         posicio1++;
         posicio2=valor.indexOf("-",posicio1);
         if(posicio2<0) posicio2=valor.indexOf("/",posicio1);
         mes=valor.substring(posicio1,posicio2);
      
         posicio2++;
         dia=valor.substring(posicio2).trim();
      
      
         return obtePerCampData(camp,Integer.parseInt(dia),Integer.parseInt(mes),Integer.parseInt(any));
      
      }
   
   /**
   * Metode que ens permet obtenir elements de la BBDD a partir de l'any d'un
   * camp data. Aquest metode esta pensat per obtenir de manera simple informacio
   * del tipus: Persones que van neixer a partir de l'any X.
   *@param camp String amb el nom del camp a consultar
   *@param any Enter amb el valor a partir del qual buscarem, cap endavant
   *@return Vector amb un conjunt d'elements el camp dels quals coincideix amb els valors
   *aportats.
   */
   
      public Vector obtePerAnyAPartirDe(String camp,int any)        
      {
      
         Resultat res;
         Vector vectorDeResultats = new Vector();
         ResultSetMetaData resMD;
         Statement consulta;
         int columnes;
      
         columnes=0;
      
         try
         {
         
            consulta = conexio.createStatement(); 
            resultat = consulta.executeQuery("SELECT * FROM "+taula+" WHERE YEAR("+camp+")>='"+any+"'");
         
            if(resultat.first())
            {
            
               resMD=resultat.getMetaData();
               columnes=resMD.getColumnCount();
            
               while(!resultat.isAfterLast())
               {
               
                  res=emplenaResultat(resMD,columnes);
               
                  vectorDeResultats.add(res);
               
                  resultat.next();
               
               }
            
            }
         
         }
         
         
            catch(Exception e) {
            
            
            
               System.err.println("(Persistencia10)Error acces a BBDD: " + e.getMessage());
            
            }
      
         res=null;
         resMD=null;
         consulta=null;
      
         return vectorDeResultats;
      
      }
   
   /**
   * Metode que ens permet obtenir elements de la BBDD a partir de l'any d'un
   * camp data. Aquest metode esta pensat per obtenir de manera simple informacio
   * del tipus: Persones que van neixer abans de l'any X.
   *@param camp String amb el nom del camp a consultar
   *@param any Enter amb el valor a partir del qual buscarem, cap enradera
   *@return Vector amb un conjunt d'elements el camp dels quals coincideix amb els valors
   *aportats.
   */
   
      public Vector obtePerAnyMenorOIgualQue(String camp,int any)        
      {
      
         Resultat res;
         Vector vectorDeResultats = new Vector();
         ResultSetMetaData resMD;
         Statement consulta;
         int columnes;
      
         columnes=0;
      
         try
         {
         
            consulta = conexio.createStatement(); 
            resultat = consulta.executeQuery("SELECT * FROM "+taula+" WHERE YEAR("+camp+")<='"+any+"'");
         
            if(resultat.first())
            {
            
               resMD=resultat.getMetaData();
               columnes=resMD.getColumnCount();
            
               while(!resultat.isAfterLast())
               {
               
                  res=emplenaResultat(resMD,columnes);
               
                  vectorDeResultats.add(res);
               
                  resultat.next();
               
               }
            
            }
         
         }
         
         
            catch(Exception e) {
            
            
            
               System.err.println("(Persistencia11)Error acces a BBDD: " + e.getMessage());
            
            }
      
         res=null;
         resMD=null;
         consulta=null;
      
         return vectorDeResultats;
      
      }
   
   /**
   * Metode que ens permet obtenir elements de la BBDD a partir de l'any d'un
   * camp data. Aquest metode esta pensat per obtenir de manera simple informacio
   * del tipus: Persones que van neixer l'any X.
   *@param camp String amb el nom del camp a consultar
   *@param any Enter amb el valor de l'any
   *@return Vector amb un conjunt d'elements el camp dels quals coincideix amb els valors
   *aportats.
   */
      public Vector obtePerAny(String camp,int any)
      {
      
         Resultat res;
         Vector vectorDeResultats = new Vector();
         ResultSetMetaData resMD;
         Statement consulta;
         int columnes;
      
         columnes=0;
      
         try
         {
         
            consulta = conexio.createStatement(); 
            resultat = consulta.executeQuery("SELECT * FROM "+taula+" WHERE YEAR("+camp+")='"+any+"'");
         
            if(resultat.first())
            {
            
               resMD=resultat.getMetaData();
               columnes=resMD.getColumnCount();
            
               while(!resultat.isAfterLast())
               {
               
                  res=emplenaResultat(resMD,columnes);
               
                  vectorDeResultats.add(res);
               
                  resultat.next();
               
               }
            
            }
         
         }
         
            catch(Exception e) {
            
            
               System.err.println("(Persistencia12)Error acces a BBDD: " + e.getMessage());
            
            }
      
         res=null;
         resMD=null;
         consulta=null;
      
         return vectorDeResultats;
      
      }
   
   
   
   
   /**
   * Metode que ens permet obtenir elements de la BBDD a partir d'un o mes camps que coneixem.
   * El camps poden ser de qualsevol naturalesa i tants com es vulgui.
   *@param consul ObjectePersistencia amb els valors dels camps amb els que treballar i els 
   *seus valors corresponents
   *@return Vector amb els elements obtinguts
   */
      public Vector obtePerDiversosCamps(ObjectePersistencia consul)
      {
      
         Resultat res;
         Vector vectorDeResultats = new Vector();
         ResultSetMetaData resMD;
         int columnes;
         String consultaSQL;
         Statement consulta;
      
         columnes=0;
         consultaSQL="SELECT * FROM "+taula+" WHERE";
      
         if(consul.hiHaCamps())
         {
         
            consultaSQL+=" "+consul.obteCamp()+"='"+consul.obteValor()+"'";
            consul.seguent();
         
         }
      
         while(consul.hiHaCamps())
         {
         
            consultaSQL+=" AND "+consul.obteCamp()+"='"+consul.obteValor()+"'";
            consul.seguent();
         
         }
         //System.out.println(consultaSQL);
         try
         {
         
            consulta = conexio.createStatement(); 
            resultat = consulta.executeQuery(consultaSQL);
         
            if(resultat.first())
            {
            
               resMD=resultat.getMetaData();
               columnes=resMD.getColumnCount();
            
               while(!resultat.isAfterLast())
               {
               
                  res=emplenaResultat(resMD,columnes);
               
                  vectorDeResultats.add(res);
               
                  resultat.next();
               
               }
            
            }
         
         }
         
            catch(Exception e) {
            
               System.err.println("(Persistencia13)Error acces a BBDD: " + e.getMessage());
            
            }
      
         res=null;
         resMD=null;
         consulta=null;
      
         return vectorDeResultats;
      
      }
   
   /**
   * Metode que ens permet obtenir elements de la BBDD a partir d'un o mes camps que coneixem, ordenats ascendentment per un camp determinat
   * El camps poden ser de qualsevol naturalesa i tants com es vulgui.
   *@param consul ObjectePersistencia amb els valors dels camps amb els que treballar i els 
   *seus valors corresponents
   *@param campOrdre String amb el nom del camp per el que hem d'ordenar ascendentment
   *@return Vector amb els elements obtinguts
   */
   
      public Vector obtePerDiversosCampsAscPer(ObjectePersistencia consul,String campOrdre)
      
      {
      
         Resultat res;
         Vector vectorDeResultats = new Vector();
         ResultSetMetaData resMD;
         int columnes;
         String consultaSQL;
         Statement consulta;
      
         columnes=0;
         consultaSQL="SELECT * FROM "+taula+" WHERE";
      
         if(consul.hiHaCamps())
         {
         
            consultaSQL+=" "+consul.obteCamp()+"='"+consul.obteValor()+"'";
            consul.seguent();
         
         }
      
         while(consul.hiHaCamps())
         {
         
            consultaSQL+=" AND "+consul.obteCamp()+"='"+consul.obteValor()+"'";
            consul.seguent();
         
         }
      
         consultaSQL+=" ORDER BY "+campOrdre+" ASC";
      
         try
         {
         
            consulta = conexio.createStatement(); 
            resultat = consulta.executeQuery(consultaSQL);
         
            if(resultat.first())
            {
            
               resMD=resultat.getMetaData();
               columnes=resMD.getColumnCount();
            
               while(!resultat.isAfterLast())
               {
               
                  res=emplenaResultat(resMD,columnes);
               
                  vectorDeResultats.add(res);
               
                  resultat.next();
               
               }
            
            }
         
         }
         
         
            catch(Exception e) {
            
            
               System.err.println("(Persistencia14)Error acces a BBDD: " + e.getMessage());
            
            }
      
         res=null;
         resMD=null;
         consulta=null;
      
         return vectorDeResultats;
      
      }
   
   /**
   * Metode que ens permet buscar valors inferiors a un valor donat.
   *@param camp String amb el nom del camp
   *@param valor Enter amb el valor numeric del camp
   *@return Vector amb tot els resultats
   */
   
      public Vector obteMenorsQue(String camp,int valor)
      {
      
         Resultat res;
         Vector vectorDeResultats = new Vector();
         ResultSetMetaData resMD;
         int columnes;
         String consultaSQL;
         Statement consulta;
      
         columnes=0;
         consultaSQL="SELECT * FROM "+taula+" WHERE "+camp+" < '"+valor+"'";
      
         try
         {
         
            consulta = conexio.createStatement(); 
            resultat = consulta.executeQuery(consultaSQL);
         
            if(resultat.first())
            {
            
               resMD=resultat.getMetaData();
               columnes=resMD.getColumnCount();
            
               while(!resultat.isAfterLast())
               {
               
                  res=emplenaResultat(resMD,columnes);
               
                  vectorDeResultats.add(res);
               
                  resultat.next();
               
               }
            
            }
         
         }
         
         
         
            catch(Exception e) {
            
            
            
               System.err.println("(Persistencia15)Error acces a BBDD: " + e.getMessage());
            
            }
      
         res=null;
         resMD=null;
         consulta=null;
      
         return vectorDeResultats;
      
      }
   
   /**
   * Metode que ens permet buscar valors superiors a un valor donat.
   *@param camp String amb el nom del camp
   *@param valor Enter amb el valor del camp
   *@return Vector amb tots els resultats
   */           
      public Vector obteMajorsQue(String camp,int valor)
      
      {
      
         Resultat res;
         Vector vectorDeResultats = new Vector();
         ResultSetMetaData resMD;
         int columnes;
         String consultaSQL;
         Statement consulta;
      
         columnes=0;
         consultaSQL="SELECT * FROM "+taula+" WHERE "+camp+" > '"+valor+"'";
      
         try
         {
         
            consulta = conexio.createStatement(); 
            resultat = consulta.executeQuery(consultaSQL);
         
            if(resultat.first())
            {
            
               resMD=resultat.getMetaData();
               columnes=resMD.getColumnCount();
            
               while(!resultat.isAfterLast())
               {
               
                  res=emplenaResultat(resMD,columnes);
               
                  vectorDeResultats.add(res);
               
                  resultat.next();
               
               }
            
            }
         
         }
         
         
         
         
            catch(Exception e) {
            
            
            
            
               System.err.println("(Persistencia16)Error acces a BBDD: " + e.getMessage());
            
            }
      
         res=null;
         resMD=null;
         consulta=null;
      
         return vectorDeResultats;
      
      }
   
   /**
   * Metode que ens permet eliminar elements de la BBDD a partir d'un camp de text. El sistema
   * eliminara tots els elements el camp dels quals coincideixi amb el valor donat. Cal
   * recordar que aquesta classe no coneix la naturalesa dels camps amb els que treballa. En 
   * altres paraules, si per exemple volem eliminar nomes un element de la BBDD a partir 
   * d'un camp text d'aquest, primer s'haura d'assegurar que en tota la taula no hi ha mes
   * d'un element amb el mateix valor. 
   *@param camp String amb el nom del camp
   *@param valor String amb el valor del camp
   *@return boolean amb valor TRUE si s'ha realitzat correctament
   */
      public boolean eliminaPerCampText(String camp,String valor)
      {
      
         Statement eliminacio;
      
         try
         {
         
            eliminacio = conexio.createStatement(); 
            eliminacio.executeUpdate("DELETE FROM "+taula+" WHERE "+camp+"='"+valor+"'");
            eliminacio = null;
            return true;
         
         }
         
            catch(Exception e) {
            
            
               System.err.println("(Persistencia17)Error acces a BBDD: " + e.getMessage());
               eliminacio = null;
               return false;
            
            } 
      
      }
   
   /**
   * Metode que ens permet eliminar elements de la BBDD a partir d'un camp numeric. El sistema
   * eliminara tots els elements el camp dels quals coincideixi amb el valor donat. Cal
   * recordar que aquesta classe no coneix la naturalesa dels camps amb els que treballa. En 
   * altres paraules, si per exemple volem eliminar nomes un element de la BBDD a partir 
   * d'un camp numeric d'aquest, primer s'haura d'assegurar que en tota la taula no hi ha mes
   * d'un element amb el mateix valor. 
   *@param camp String amb el nom del camp
   *@param valor Enter amb el valor del camp
   *@return boolean amb valor TRUE si s'ha executat correctament
   */
      public boolean eliminaPerCampNumeric(String camp,int valor)
      {
      
         return eliminaPerCampText(camp,""+valor+"");
      
      }
   
   /**
   * Metode que ens permet actualitzar un camp text d'un element de la BBDD. El metode funciona
   * de la seguent manera: el primer atribut representa el nom del camp que volem
   * modificar, el segon el valor que la BBDD conte actualment i el darrer el nou valor
   * que volem assignar al camp. El primer i segon camp, doncs, s'utilitzen per localitzar
   * l'element del qual volem canviar el valor d'un camp. Cal recordar que si no assegurem
   * que el camp amb el que treballem te valors que son unics, el sistema executara sobre
   * tots els elements que tinguin el valor valorAntic en el camp amb nom "camp".
   *@param camp String amb el nom del camp que volem modificar
   *@param valorAntic String amb el valor actual del camp
   *@param valorNou String amb el valor que volem assignar al camp
   *@return boolean amb valor true si s'executa correctament
   */
      public boolean actualitzaCampText(String camp,String valorAntic,String valorNou)
      {
      
         Statement actualitzacio;
      
         try
         {
         
            actualitzacio = conexio.createStatement(); 
         
            actualitzacio.executeUpdate("UPDATE "+taula+" SET "+camp+"='"+valorNou+"' WHERE "+camp+"='"+valorAntic+"'");
            actualitzacio = null;
            return true;
         
         }
         
            catch(Exception e) {
            
               System.err.println("(Persistencia18)Error acces a BBDD: " + e.getMessage());
               actualitzacio = null;
               return false;
            
            }
      
      }
   
   /**
   * Metode que ens permet actualitzar un camp numeric d'un element de la BBDD. El metode funciona
   * de la seguent manera: el primer atribut representa el nom del camp que volem
   * modificar, el segon el valor que la BBDD conte actualment i el darrer el nou valor
   * que volem assignar al camp. El primer i segon camp, doncs, s'utilitzen per localitzar
   * l'element del qual volem canviar el valor d'un camp. Cal recordar que si no assegurem
   * que el camp amb el que treballem te valors que son unics, el sistema executara sobre
   * tots els elements que tinguin el valor valorAntic en el camp amb nom "camp".
   *@param camp String amb el nom del camp que volem modificar
   *@param valorAntic Enter amb el valor actual del camp
   *@param valorNou Enter amb el valor que volem assignar al camp
   *@return boolean amb valor TRUE si s'ha realitzat correctament
   */
      public boolean actualitzaCampNumeric(String camp,int valorAntic,int valorNou)       
      {
      
         return actualitzaCampText(camp,""+valorAntic+"",""+valorNou+"");
      
      }
   
   /**
   * Metode que ens permet actualitzar diversos camps d'un element de la BBDD. El metode funciona
   * de la seguent manera: el primer atribut representa el nom d'un camp de l'element a
   * modificar, el segon el valor que la BBDD conte actualment i el darrer el conjunt de nous valors
   * que volem assignar al camp. El primer i segon camp, doncs, s'utilitzen per localitzar
   * l'element del qual volem canviar el valor d'un camp. Cal recordar que si no assegurem
   * que el camp amb el que treballem te valors que son unics, el sistema executara sobre
   * tots els elements que tinguin el valor valorAntic en el camp amb nom "camp".
   *@param camp String amb el nom del camp que volem modificar
   *@param valorAntic String amb el valor actual del camp
   *@param dadesActualitzadesObjecte ObjectePersistencia amb un conjunt de noms de camp
   *que volem actualitzar aixi com el nou valor per aquell camp
   *@return boolean amb valor TRUE quan s'executa correctament
   */
      public boolean actualitzaDiversosCampsPerCampText(String camp,String valorAntic,ObjectePersistencia dadesActualitzadesObjecte)
      {
      
         Statement actualitzacio;
         String expressio;
      
         expressio="UPDATE "+taula+" SET ";
      
         if(dadesActualitzadesObjecte.hiHaCamps())
         {
         
            expressio+=dadesActualitzadesObjecte.obteCamp()+"='"+dadesActualitzadesObjecte.obteValor()+"'";
            dadesActualitzadesObjecte.seguent();
         
         }
      
         while(dadesActualitzadesObjecte.hiHaCamps())
         {
         
            expressio+=","+dadesActualitzadesObjecte.obteCamp()+"='"+dadesActualitzadesObjecte.obteValor()+"'";
            dadesActualitzadesObjecte.seguent();
         
         }
      
         expressio+=" WHERE "+camp+"='"+valorAntic+"'";
      
         try
         {
         
            actualitzacio = conexio.createStatement(); 
            actualitzacio.executeUpdate(expressio);
            actualitzacio = null;
            return true;
         
         }
         
            catch(Exception e) {        
            
               System.err.println("(Persistencia19)Error acces a BBDD: " + e.getMessage());
               actualitzacio = null;
               return false;
            
            }
      
      }
   
   /**
   * Metode que ens permet actualitzar diversos camps d'un element de la BBDD. El metode funciona
   * de la seguent manera: el primer atribut representa el nom d'un camp de l'element a
   * modificar, el segon el valor que la BBDD conte actualment i el darrer el conjunt de nous valors
   * que volem assignar al camp. El primer i segon camp, doncs, s'utilitzen per localitzar
   * l'element del qual volem canviar el valor d'un camp. Cal recordar que si no assegurem
   * que el camp amb el que treballem te valors que son unics, el sistema executara sobre
   * tots els elements que tinguin el valor valorAntic en el camp amb nom "camp".
   *@param camp String amb el nom del camp que volem modificar
   *@param valorAntic Enter amb el valor actual del camp
   *@param dadesActualitzadesObjecte ObjectePersistencia amb un conjunt de noms de camp
   *que volem actualitzar aixi com el nou valor per aquell camp
   *@return boolean amb valor true si s'ha realitzat correctament
   */
      public boolean actualitzaDiversosCampsPerCampNumeric(String camp,int valorAntic,ObjectePersistencia dadesActualitzadesObjecte)
      {
      
         return actualitzaDiversosCampsPerCampText(camp,""+valorAntic+"",dadesActualitzadesObjecte);
      
      }
   
   /**
   * Metode que ens permet afegir un element. Cal tenir en compte que aquesta classe no
   * fa cap comprovacio, i per tant, l'execucio pot fallar
   * facilment si, per exemple, l'objecte dadesNouObjecte no conte cap valor per un camp
   * que la BBDD considera que no pot ser nul.
   *@param dadesNouObjecte Objecte del tipus ObjectePersistencia amb les dades del nou element
   *a afegir en el sistema
   *@return Enter amb el valor que el sistema ha generat automaticament per aquest element
   */
      public int afegeixElement(ObjectePersistencia dadesNouObjecte)
      {
      
         Statement altaElement;
         String nouElement1,nouElement2,nouElementFinal;
         ResultSet rs;
         int identificador;
      
         identificador=0;
         nouElement1="INSERT INTO "+taula+" (";
         nouElement2="VALUES(";
      
         if(dadesNouObjecte.hiHaCamps())
         {
         
            nouElement1+=dadesNouObjecte.obteCamp();
            if( (dadesNouObjecte.obteTipus().compareToIgnoreCase("textCurt"))==0 | (dadesNouObjecte.obteTipus().compareToIgnoreCase("data"))==0 | (dadesNouObjecte.obteTipus().compareToIgnoreCase("hora"))==0)
               nouElement2+="'"+dadesNouObjecte.obteValor()+"'";
            if( (dadesNouObjecte.obteTipus().compareToIgnoreCase("textLlarg"))==0 )
            {
               nouElement2+=">"+dadesNouObjecte.obteValor()+">";
               nouElement2=nouElement2.replace('>','"');
            }
            if( (dadesNouObjecte.obteTipus().compareToIgnoreCase("nombre"))==0 )
               nouElement2+=""+dadesNouObjecte.obteValor()+"";
         
            dadesNouObjecte.seguent();
         
         }
      
         while(dadesNouObjecte.hiHaCamps())
         {
         
            nouElement1+=","+dadesNouObjecte.obteCamp();
            if( (dadesNouObjecte.obteTipus().compareToIgnoreCase("textCurt"))==0 | (dadesNouObjecte.obteTipus().compareToIgnoreCase("data"))==0 | (dadesNouObjecte.obteTipus().compareToIgnoreCase("hora"))==0)
               nouElement2+=",'"+dadesNouObjecte.obteValor()+"'";
            if( (dadesNouObjecte.obteTipus().compareToIgnoreCase("textLlarg"))==0 )
            {
               nouElement2+=",>"+dadesNouObjecte.obteValor()+">";
               nouElement2=nouElement2.replace('>','"');
            }
            if( (dadesNouObjecte.obteTipus().compareToIgnoreCase("nombre"))==0 )
               nouElement2+=","+dadesNouObjecte.obteValor()+"";
            dadesNouObjecte.seguent();
         
         }
      
         nouElementFinal=nouElement1+") "+nouElement2+")";
      
         try
         {
         
            altaElement = conexio.createStatement(ResultSet.TYPE_FORWARD_ONLY,ResultSet.CONCUR_UPDATABLE);
            altaElement.executeUpdate(nouElementFinal);
            rs = altaElement.getGeneratedKeys();
            if(rs.next())
               identificador = rs.getInt(1);
         
         }
         
            catch(Exception e) {        
            
            
               System.err.println("(Persistencia20)Error acces a BBDD: " + e.getMessage());
            
            }
      
         altaElement = null;
         rs = null;
      
         return identificador;
      
      }
   
   }