   package Entitats.Pesa;

   import Entitats.Temps.Data;
   import Entitats.Temps.Hora;    

	/**
	* Classe que permet la creacio i manipulacio de peces del repertori d'un entitat cultural, amb 
	* dades com el nom, l'autor, etc.
	* No s'implementa en aquest punt cap validacio de les dades assignades a l'objecte, doncs
	* es creu funcio indispensable de les capes superiors de l'aplicacio, les capes que 
	* utilitzen aquesta classe. 
	*@author Jordi Moles i Blanco, Mar 2005
	*/      

   public class Pesa
   {
   
      private int identificadorPesa; //Valor que identifica la pesa de manera unica
      private String nom; //Nom de la pesa
      private Hora durada; //Durada de la pesa
      private String procedencia; //Procedencia de la pesa (normalment poble o area geografica)
      private String autor; //Autor de la pesa 
      private Data dataCreacio; //Data de creacio de la pesa
      private String descripcio; //Extens comentari/descripcio sobre la pesa
      private int imatge;
   
   /**
   *Constructor de la classe.
   *Inicialitza tots els atributs numerics a 0 i els strings a "" (Blanc)
   *La data de creacio de la pesa s'inicialitza a 00/00/0000
   */
      public Pesa()
      {
      
         identificadorPesa=0;
         nom="";
         durada=new Hora();
         durada.estableixHora(0,0,0);
         procedencia="";
         autor="";
         dataCreacio=new Data();
         dataCreacio.estableixData(0,0,0);
         descripcio="";
      
      }
   
   /**
   *Destructor de la classe
   *Metode que elimina les referencies als objectes manipulats per una pesa
   *per tal de que l'objecte "Gargabe Collector" del Java n'alliberi la memoria assignada
   *quan descobreixi que l'objecte ja no te cap referencia.
   */
      public void destructorPesa()
      {
      
         durada.destructorHora();
         dataCreacio.destructorData();
      
      }
   
   /**
   * Metode que assigna el valor que identifica la pesa de manera unica en el sistema
   *@param identificadorPesa Enter amb el valor de l'identificador de la pesa
   */
      public void estableixIdentificador(int identificadorPesa)
      {
      
         this.identificadorPesa=identificadorPesa;
      
      }
   
   	/**
   	*Metode que assigna el nom de la pesa.
   	*@param nom Nom que volem assignar a la pesa
   	*/
      public void estableixNom(String nom)
      {
      
         this.nom=nom;
      
      }
   
   	/**
   	*Metode que assigna la durada de la pesa.
   	*@param hores Enter amb el valor de les hores que dura la pesa
   	*@param minuts Enter amb el valor dels minuts que dura la pesa
   	*@param segons Enter amb el valor dels segons que dura la pesa
   	*/           
      public void estableixDurada(int hores,int minuts,int segons)              
      {
      
         durada.estableixHora(hores,minuts,segons);
      
      }
   
   /**
   * Metode que assigna la durada de la pesa
   *@param durada String amb el temps que la pesa dura, en un dels seguents formats:
   *hh.mm.ss o hh:mm:ss
   */
      public void estableixDurada(String durada)
      {
      
         int hores,minuts,segons;
         int posicio1,posicio2;
      
         posicio1=durada.indexOf(".");
         if(posicio1<0) posicio1=durada.indexOf(":");
         hores=Integer.parseInt(durada.substring(0,posicio1));
      
         posicio1++;
         posicio2=durada.indexOf(".",posicio1);
         if(posicio2<0) posicio2=durada.indexOf(":",posicio1);
         minuts=Integer.parseInt(durada.substring(posicio1,posicio2));
      
         posicio2++;
         segons=Integer.parseInt(durada.substring(posicio2).trim());
      
         estableixDurada(hores,minuts,segons);
      
      }
   
   	/**
   	*Metode que assigna la procedencia de la pesa.
   	*@param procedencia Nom del poble o area geografica d'on procedeix la pesa
   	*/           
      public void estableixProcedencia(String procedencia)              
      {
      
         this.procedencia=procedencia;
      
      }
   
   	/**
   	*Metode que assigna el nom de l'autor de la pesa.
   	*@param autor Autor de la pesa
   	*/           
      public void estableixAutor(String autor)              
      {
      
         this.autor=autor;
      
      }
   
   	/**
   	*Metode que assigna la data de creacio de la pesa.
   	*@param dia Enter amb el valor del dia del mes que ha de formar part de la data.
   	*@param mes Enter amb el valor del mes de l'any que ha de formar part de la data.
   	*@param any Enter amb el valor de l'any que ha de formar part de la data
   	*/           
      public void estableixDataCreacio(int dia,int mes,int any)              
      {
      
         dataCreacio.estableixData(dia,mes,any);
      
      }
   
   /**
   * Metode que assigna la data de creacio de la pesa
   *@param dataCreacio String amb la data de creacio de la pesa, en un dels seguents
   *formats: dd-mm-aaaa o dd/mm/aaaa
   */
      public void estableixDataCreacio(String dataCreacio)
      {
      
         int dia,mes,any;
         int posicio1,posicio2;
      
         posicio1=dataCreacio.indexOf("-");
         if(posicio1<0) posicio1=dataCreacio.indexOf("/");
         dia=Integer.parseInt(dataCreacio.substring(0,posicio1));
      
         posicio1++;
         posicio2=dataCreacio.indexOf("-",posicio1);
         if(posicio2<0) posicio2=dataCreacio.indexOf("/",posicio1);
         mes=Integer.parseInt(dataCreacio.substring(posicio1,posicio2));
      
         posicio2++;
         any=Integer.parseInt(dataCreacio.substring(posicio2).trim());
      
         estableixDataCreacio(dia,mes,any);
      
      }
   
   	/**
   	*Metode que assigna una descripcio/comentari a la pesa.
   	*@param descripcio Descripcio/comentari de la pesa
   	*/                 
      public void estableixDescripcio(String descripcio)                    
      {
      
         this.descripcio=descripcio;
      
      }
   
   	/**
   	* Metode que estableix la ruta on es troba la imatge associada a la pesa
   	*@param imatge Enter amb l'identificador de la imatge
   	*/
   
      public void estableixImatge(int imatge)
      
      {
         this.imatge = imatge;
      }
   
   /**
   * Metode que obte el valor que identifica la pesa de manera unica en el sistema
   *@return Enter amb el valor de l'identificador
   */
      public int obteIdentificador()
      {
      
         return identificadorPesa;
      
      }
   
   	/**
   	*Metode que obte el nom de la pesa
   	*@return String amb el nom de la pesa
   	*/
      public String obteNom()
      {
      
         return nom.trim();
      
      }
   
   	/**
   	*Metode que obte la durada de la pesa
   	*@return String amb la durada de la pesa
   	*/    
      public String obteDurada()      
      {
      
         return durada.obteHora().trim();
      
      }
   
   	/**
   	*Metode que obte la procedencia de la pesa (normalment poblacio o area geografica)
   	*@return String amb la procedencia de la pesa
   	*/    
      public String obteProcedencia()      
      {
      
         return procedencia.trim();
      
      }
   
   	/**
   	*Metode que obte el nom de l'autor de la pesa
   	*@return String amb el nom de l'autor de la pesa
   	*/    
      public String obteAutor()      
      {
      
         return autor.trim();
      
      }
   
   	/**
   	*Metode que obte la data de creacio de la pesa
   	*@return String amb la data de creacio de la pesa
   	*/    
      public String obteDataCreacio()      
      {
      
         return dataCreacio.obteData().trim();
      
      }
   
   	/**
   	*Metode que obte la descripcio de la pesa
   	*@return String amb la descripcio/comentaris de la pesa
   	*/         
      public String obteDescripcio()              
      {
      
         return descripcio.trim();
      
      }
   
   
   /**
   	*Metode que obte la imatge associada a la pesa
   	*@return Enter amb l'identificador de la pesa associada
   	*/ 
      public int obteImatge()
      {
         return imatge;
      }
   
   }