   package Entitats.Pesa;

   import Entitats.Persistencia.Persistencia;
   import Entitats.Persistencia.Llistat;
   import Entitats.Persistencia.Resultat;
   import Entitats.Persistencia.ObjectePersistencia;

/**
* Classe que permet mantenir una relacio entre peces i les imatges associades a aquestes
*@author Jordi Moles i Blanco, Mar 2006
*/

   public class RelacioImatgesPesa
   {
   
      private Persistencia persistencia; //Objecte que ens permet obtenir informacio de la BBDD.
      private Llistat llistat; 
      private Resultat resultat;
      private static int ordre=0; 
   
   /**
   * Constructor de la classe. Inicialitza els elements necessaris per tal d'accedir a la BBDD.
   */
      public RelacioImatgesPesa()
      {
      
         persistencia = new Persistencia("entitats","imatgesPesa");
         llistat=null;
      
      }
   
   /**
   * Destructor de la classe
   * Metode que elimina les referencies als objectes manipulats per una RelacioImatgesPesa
   * per tal de que l'objecte "Gargabe Collector" del Java n'alliberi la memoria assignada
   * quan descobreixi que l'objecte ja no te cap referencia.
   */
      public void destructorRelacioImatgesPesa()
      {
      
         persistencia.destructorPersistencia();
         if(llistat!=null)llistat.destructorLlistat();
         resultat=null;
      
      }
   
   //////Els seguents metodes representen l'iterador de imatges associades a una pesa\\\\\
                                                                                                                                                   
   /**
   * Metode que cerca totes les imatges sobre socis   
   */
   
      public void obteImatgesPeces()
      
      {
      
         llistat=new Llistat(persistencia.obteTots());
      
      }
   
   /**
   * Metode que cerca les imatges relacionades a una pesa donat l'identificador d'aquesta
   *@param identificadorPesa Enter amb el valor que identifica la pesa
   */
      public void obteImatgesPesa(int identificadorPesa)
      {
      
         llistat=new Llistat(persistencia.obtePerCampNumeric("pesa_identificadorPesa",identificadorPesa));
      
      }
   
   
   /**
   * Metode que ens informa si hi ha imatges en l'iterador que encara no hem consultat
   *@return Boolea amb valor positiu si hi ha elements i negatiu si no n'hi ha.
   */
      public boolean hiHaImatges() 
      {
      
         return llistat.hiHaElements();
      
      }
   
   /**
   * Metode que ens permet obtenir l'identificador de la primera imatge que encara
   * no hem consultat. L'element no s'elimina del sistema pero si d'aquest iterador. Per tant,
   * haurem de tornar a executar obteImatgesPesa si volem tornar a tractar tots els elements.
   *@return Enter amb l'identificador de la imatge
   */
      public int obteImatge()   
      {
      
         resultat=(Resultat)llistat.obtePrimerElement();
         return Integer.parseInt((String)resultat.obteElement("imatge_numImatge"));
      
      }
   
   //////////Fi iterador\\\\\\\
                                                                                                                                                                                                                                                                                    
   /**
   * Metode que permet afegir una imatge, es a dir, relacionar una nova imatge amb un pesa.
   *@param identificadorPesa Enter amb l'identificador de la pesa
   *@param imatge Enter amb l'identificador de la imatge
   */
      public static void afegirImatgeAPesa(int identificadorPesa,int imatge)
      {
      
         Persistencia Persistencia = new Persistencia("entitats","imatgesPesa");
         ObjectePersistencia dadesNovaImatgePerPesa = new ObjectePersistencia();
      
         dadesNovaImatgePerPesa.afegirCampNumeric("pesa_identificadorPesa",identificadorPesa);
         dadesNovaImatgePerPesa.afegirCampNumeric("imatge_numImatge",imatge);
      
         Persistencia.afegeixElement(dadesNovaImatgePerPesa);
      
         dadesNovaImatgePerPesa.destructorObjectePersistencia();
         Persistencia.destructorPersistencia();
      
      }
   
   /*
   * Metode que elimina la relacio d'una determinada pesa amb totes les imatges relacionades.
   *@param identificadorPesa Enter amb el valor corresponent 
   */
      public static void eliminaImatges(int identificadorPesa)
      {
      
         Persistencia Persistencia = new Persistencia("entitats","imatgesPesa");
      
         Persistencia.eliminaPerCampNumeric("pesa_identificadorPesa",identificadorPesa);
      
         Persistencia.destructorPersistencia();
      
      }
   
   }