   package Entitats.Pesa;

   import Entitats.Persistencia.Persistencia;
   import Entitats.Persistencia.Resultat;
   import Entitats.Pesa.Pesa;
   import Entitats.Pesa.RelacioImatgesPesa;
   import Entitats.Persistencia.Llistat;
   import Entitats.Persistencia.ObjectePersistencia;

/**
* Classe que permet mantenir una relacio de peces. Altes, baixes, modificacions i
* consultes de peces per qualsevol camp son possibles. Implementa els elements necessaris
* per accedir a la BBDD o qualsevol altre sistema que utilitzem per conservar les dades
* que manipulem de forma permanent. Aixo es gracies a l'us de la classe Persistencia, que 
* ens permet treballar amb peces sense haver de preocupar-nos de com el sistema
* tracta les dades.
*@author Jordi Moles i Blanco, Mar 2005
*/

   public class RelacioPeces
   
   {
   
      private Persistencia persistencia;
      private Resultat resultat;
      private Llistat llistat;
   
   /**
   * Constructor de la classe. Inicialitza els elements necessaris per tal d'accedir a la BBDD.
   */
      public RelacioPeces()
      {
      
         persistencia = new Persistencia("entitats","pesa");
      
      }
   
   /**
   * Destructor de la classe
   * Metode que elimina les referencies als objectes manipulats per una RelacioPeces
   * per tal de que l'objecte "Gargabe Collector" del Java n'alliberi la memoria assignada
   * quan descobreixi que l'objecte ja no te cap referencia.
   */
      public void destructorRelacioPeces()
      {
      
         persistencia.destructorPersistencia();
         resultat=null;
         llistat.destructorLlistat();
      
      }
   
   //////Els seguents metodes representen l'iterador de peces recuperades del sistema\\\\\
                 
   /*
   * Metode que obte totes les peces presents al sistema
   */                                           
      public void obteTotesLesPeces()
      {
      
         llistat = new Llistat(persistencia.obteTots());
      
      }
   
   /**
   * Metode que ens permet obtenir un conjunt de peces que coincideixen amb els valors
   * donats. Es pot cercar una pesa aportant valors de tots els seus camps
   * o nomes d'un si es desitja, i a mes, aquest camp pot ser qualsevol.
   *@param dadesPesaAConsultar ObjectePersistencia amb tots els valors dels camps
   *per els que volem cercar una pesa.
   */
      public void obtePesaPerMultiplesCamps(ObjectePersistencia dadesPesaAConsultar)
      {
      
         llistat = new Llistat(persistencia.obtePerDiversosCamps(dadesPesaAConsultar));
      
      }
   
   /**
   * Metode que ens informa si hi ha peces en l'iterador que encara no hem consultat
   *@return Boolea amb valor positiu si hi ha elements i negatiu si no n'hi ha.
   */
      public boolean hiHaPeces()
      {
      
         return llistat.hiHaElements();
      
      }
   
   /**
   * Metode que ens permet obtenir el primer objecte pesa que encara
   * no hem consultat, amb totes les dades disponibles sobre aquesta en el sistema.
   *@return Pesa amb totes les dades disponibles
   */
      public Pesa obtePesa()
      {
      
         resultat=(Resultat)llistat.obtePrimerElement();
         Pesa pesa = new Pesa();
      
         pesa.estableixIdentificador(Integer.parseInt((String)resultat.obteElement("identificadorPesa")));
         pesa.estableixNom((String)resultat.obteElement("nom"));
         pesa.estableixDurada((String)resultat.obteElement("durada"));
         pesa.estableixProcedencia((String)resultat.obteElement("procedencia"));
         pesa.estableixAutor((String)resultat.obteElement("autor"));
         pesa.estableixDataCreacio((String)resultat.obteElement("dataCreacio"));
         pesa.estableixDescripcio((String)resultat.obteElement("descripcio"));
      
      
         return pesa;
      
      }
   
   //////////Fi iterador\\\\\\\
                                                                                                                         
   /**
   * Metode que ens permet obtenir una pesa a partir de l'identificador d'aquesta
   *@param identificadorPesa Enter amb el valor que identifica una pesa
   *de manera unica
   *@return Pesa amb totes les dades disponibles de la pesa sol.licitada
   */
      public static Pesa obtePesaPerIdentificadorPesa(int identificadorPesa)
      {
      
         Pesa Pesa= new Pesa();
         Persistencia Persistencia = new Persistencia("entitats","pesa");
         Resultat Resultat;
         Llistat ll;
         ll = new Llistat(Persistencia.obtePerCampNumeric("identificadorPesa",identificadorPesa));
         if(ll.hiHaElements())
         {
            Resultat=(Resultat)ll.obtePrimerElement();
            Pesa=emplenaPesa(Resultat);
         }
         Persistencia.destructorPersistencia();
         ll.destructorLlistat();
         Resultat=null;
      
         return Pesa;
      
      }
   
      private static Pesa emplenaPesa(Resultat Resultat)
      {
      
         Pesa Pesa = new Pesa();
      
         Pesa.estableixIdentificador(Integer.parseInt((String)Resultat.obteElement("identificadorPesa")));
         Pesa.estableixNom((String)Resultat.obteElement("nom"));
         Pesa.estableixDurada((String)Resultat.obteElement("durada"));
         Pesa.estableixProcedencia((String)Resultat.obteElement("procedencia"));
         Pesa.estableixAutor((String)Resultat.obteElement("autor"));
         Pesa.estableixDataCreacio((String)Resultat.obteElement("dataCreacio"));
         Pesa.estableixDescripcio((String)Resultat.obteElement("descripcio"));
      
         return Pesa;
      
      }
   
   /**
   * Metode que permet eliminar una pesa del sistema a partir del seu identificador
   *@param identificadorPesa Enter amb l'identificador de la pesa
   */
      public static void eliminaPesaPerIdentificadorPesa(int identificadorPesa)
      {
      
         Persistencia Persistencia = new Persistencia("entitats","pesa");
         Persistencia.eliminaPerCampNumeric("identificadorPesa",identificadorPesa);
         Persistencia.destructorPersistencia();
      
      }
   
   /**
   * Metode que permet actualitzar de cop diversos camps d'una pesa
   *@param identificadorPesa Enter amb el valor que ens permetra trobar la pesa
   *que volem modificar 
   *@param dadesActualitzadesPesa ObjectePersistencia amb els noms dels camps a modificar, aixi com
   *els valors d'aquests
   */
      public static void actualitzaDiversosCampsPesaPerIdentificadorPesa(int identificadorPesa,ObjectePersistencia dadesActualitzadesPesa)
      {
      
         Persistencia Persistencia = new Persistencia("entitats","pesa");
         Persistencia.actualitzaDiversosCampsPerCampNumeric("identificadorPesa",identificadorPesa,dadesActualitzadesPesa);
         Persistencia.destructorPersistencia();
      
      }
   
   /**
   * Metode que ens permet afegir una pesa a la relacio
   *@param pesa Objecte Pesa amb les dades de la nova pesa que volem afegir
   *al sistema
   *@return Enter amb el valor que el sistema genera per a aquest nou element, valor
   *que l'identificara de manera unica en el sistema.
   */
      public static int afegirPesa(Pesa pesa)
      {
      
         int identificador;
         Persistencia Persistencia = new Persistencia("entitats","pesa");
         ObjectePersistencia dadesNovaPesa = new ObjectePersistencia();
      
         dadesNovaPesa.afegirCampText("nom",pesa.obteNom());
         dadesNovaPesa.afegirCampHora("durada",pesa.obteDurada());
         dadesNovaPesa.afegirCampText("procedencia",pesa.obteProcedencia());
         dadesNovaPesa.afegirCampText("autor",pesa.obteAutor());
         dadesNovaPesa.afegirCampData("dataCreacio",pesa.obteDataCreacio());
         dadesNovaPesa.afegirCampText("descripcio",pesa.obteDescripcio());
      
         identificador=Persistencia.afegeixElement(dadesNovaPesa);
      
         if(pesa.obteImatge()!=0)
         {
            RelacioImatgesPesa relPesa = new RelacioImatgesPesa();
            relPesa.afegirImatgeAPesa(identificador,pesa.obteImatge());  
            relPesa.destructorRelacioImatgesPesa();
         }
      
         dadesNovaPesa.destructorObjectePersistencia();
         Persistencia.destructorPersistencia();
      
         return identificador;
      
      }
   
   }