   package Entitats.PropostaModificacioSoci;

   import Entitats.PropostaModificacioSoci.PropostaModificacioSoci;
   import Entitats.Soci.RelacioSocis;
   import Entitats.Persistencia.Persistencia;
   import Entitats.Persistencia.ObjectePersistencia;
   import Entitats.Persistencia.Llistat;
   import Entitats.Persistencia.Resultat;

/**
* Classe que permet mantenir una relacio de propostes de modificacio de dades del socis, que realitzen aquests mateixo. 
* Implementa els elements necessaris per accedir a la BBDD o qualsevol altre sistema que utilitzem per conservar les dades
* que manipulem de forma permanent. Aixo es gracies a l'us de la classe Persistencia, que 
* ens permet treballar amb peces sense haver de preocupar-nos de com el sistema
* tracta les dades.
*@author Jordi Moles i Blanco, Abril 2005
*/

   public class RelacioPropostesModificacioSoci
   {
   
      private Persistencia persistencia;
      private Llistat llistat;
      private Resultat resultat;
   
   /**
   * Constructor de la classe. Inicialitza els elements necessaris per tal d'accedir a la BBDD.
   */
   
      public RelacioPropostesModificacioSoci()
      
      {
      
         persistencia = new Persistencia("entitats","propostaModificacio");
      
      }
   
   /**
   * Destructor de la classe
   * Metode que elimina les referencies als objectes manipulats per una RelacioPropostesModificacioSoci
   * per tal de que l'objecte "Gargabe Collector" del Java n'alliberi la memoria assignada
   * quan descobreixi que l'objecte ja no te cap referencia.
   */
   
      public void destructorRelacioPropostesModificacioSoci()
      
      {
      
         persistencia.destructorPersistencia();
      
      }
   
   //////Els seguents metodes representen l'iterador de propostes recuperades del sistema\\\\\
                                                                                                                                                                                                                                                
   
      public void obteTotesLesPropostes()
      
      {
      
         llistat = new Llistat(persistencia.obteTots());
      
      }
   
   /**
   * Metode que ens permet obtenir un conjunt de propostes que coincideixen amb els valors
   * donats. Es pot cercar una proposta aportant valors de tots els seus camps
   * o nomes d'un si es desitja, i a mes, aquest camp pot ser qualsevol.
   *@param dadesPropostaAConsultar ObjectePersistencia amb tots els valors dels camps
   *per els que volem cercar una proposta.
   */
   
      public void obtePropostaPerMultiplesCamps(ObjectePersistencia dadesPropostaAConsultar)
      
      {
      
         llistat = new Llistat(persistencia.obtePerDiversosCamps(dadesPropostaAConsultar));
      
      }
   
   /**
   * Metode que ens informa si hi ha propostes en l'iterador que encara no hem consultat
   *@return Boolea amb valor positiu si hi ha elements i negatiu si no n'hi ha.
   */
   
      public boolean hiHaPropostes()
      
      {
      
         return llistat.hiHaElements();
      
      }
   
   /**
   * Metode que ens permet obtenir el primer objecte proposta que encara
   * no hem consultat, amb totes les dades disponibles sobre aquesta en el sistema.
   *@return PropostaModificacioSoci amb totes les dades disponibles
   */
   
      public PropostaModificacioSoci obteProposta()
      
      {
      
         resultat=(Resultat)llistat.obtePrimerElement();
         PropostaModificacioSoci proposta = new PropostaModificacioSoci();
      
         proposta.estableixIdentificador(Integer.parseInt((String)resultat.obteElement("identificadorProposta")));
         proposta.estableixNom((String)resultat.obteElement("nom"));
         proposta.estableixCognom1((String)resultat.obteElement("cognom1"));
         proposta.estableixCognom2((String)resultat.obteElement("cognom2"));
         proposta.estableixIdentificadorPersonal((String)resultat.obteElement("identificadorPersonal"));
         proposta.estableixAdresa((String)resultat.obteElement("adresa"));
         proposta.estableixTelefon(Integer.parseInt((String)resultat.obteElement("telefon")));
         proposta.estableixDataNaixement((String)resultat.obteElement("dataNaixement"));
         proposta.estableixActiu( ((String)resultat.obteElement("actiu")).compareToIgnoreCase("1")==0 );
         proposta.estableixNumSoci(Integer.parseInt((String)resultat.obteElement("numSoci")));
      
      
         return proposta;
      
      }
   
   //////////Fi iterador\\\\\\\
                                                                                                                                                                                                                        
   
   /*
   * Metode que ens permet aplicar els canvis d'una determinada proposta de forma automatitzada.
   *@param identificadorProposta Enter amb el identificador de la proposta que hem acceptat. 
   */
   
      public static void confirmarPropostaModificacioSociPerIdentificadorProposta(int identificadorProposta)
      {
      
         Persistencia persistenciaProposta = new Persistencia("entitats","propostaModificacio");
         ObjectePersistencia obj = new ObjectePersistencia();
         ObjectePersistencia objBuscaSoci = new ObjectePersistencia();
         ObjectePersistencia objActualitzaSoci = new ObjectePersistencia();
         RelacioSocis relacioSocis = new RelacioSocis();
         Llistat Llistat;
         Resultat Resultat;
      
         objBuscaSoci.afegirCampNumeric("identificadorProposta",identificadorProposta);
         Llistat = new Llistat(persistenciaProposta.obtePerDiversosCamps(objBuscaSoci));
         if(Llistat.hiHaElements())
         {
         
            Resultat=(Resultat)Llistat.obtePrimerElement();
            int numSoci = Integer.parseInt((String)Resultat.obteElement("numSoci"));	
         
            if(((String)Resultat.obteElement("nom")).compareToIgnoreCase("")!=0)
            {
            
               objActualitzaSoci.afegirCampText("nom",(String)Resultat.obteElement("nom"));
            
            }
         
            if(((String)Resultat.obteElement("cognom1")).compareToIgnoreCase("")!=0)
            {
            
               objActualitzaSoci.afegirCampText("cognom1",(String)Resultat.obteElement("cognom1"));
            
            }
         
            if(((String)Resultat.obteElement("cognom2")).compareToIgnoreCase("")!=0)
            {
            
               objActualitzaSoci.afegirCampText("cognom2",(String)Resultat.obteElement("cognom2"));
            
            }
         
            if(((String)Resultat.obteElement("identificadorPersonal")).compareToIgnoreCase("")!=0)
            {
            
               objActualitzaSoci.afegirCampText("identificadorPersonal",(String)Resultat.obteElement("identificadorPersonal"));
            
            }
         
            if(((String)Resultat.obteElement("adresa")).compareToIgnoreCase("")!=0)
            {
            
               objActualitzaSoci.afegirCampText("adresa",(String)Resultat.obteElement("adresa"));
            
            }
         
            if(((String)Resultat.obteElement("telefon")).compareToIgnoreCase("0")!=0)
            {
            
               objActualitzaSoci.afegirCampNumeric("telefon",Integer.parseInt((String)Resultat.obteElement("telefon")));
            
            }
         
         //implementar modificar data naixement
         
            relacioSocis.actualitzaDiversosCampsSociPerNumSoci(numSoci,objActualitzaSoci);
         
         }
      
         obj.afegirCampNumeric("actiu",0);
         persistenciaProposta.actualitzaDiversosCampsPerCampNumeric("identificadorProposta",identificadorProposta,obj);
         obj.destructorObjectePersistencia();
         persistenciaProposta.destructorPersistencia();
      
      }
   
   /*
   * Metode que elimina del sistema una determinada proposta que hem descartat
   *@param identificadorProposta Enter amb el valor que identifica una determinada proposta
   */
   
      public static void eliminarPropostaModificacioSociPerIdentificadorProposta(int identificadorProposta)
      
      {
      
         Persistencia Persistencia = new Persistencia("entitats","propostaModificacio");
         Persistencia.eliminaPerCampNumeric("identificadorProposta",identificadorProposta);
         Persistencia.destructorPersistencia();
      
      }
   
   /**
   * Metode que ens permet afegir una proposta a la relacio
   *@param propostaModificacioSoci Objecte PropostaModificacioSoci amb les dades de la nova proposta que volem afegir
   *al sistema
   *@return Enter amb el valor que el sistema genera per a aquest nou element, valor
   *que l'identificara de manera unica en el sistema.
   */
   
      public static int afegirPropostaModificacioSoci(PropostaModificacioSoci propostaModificacioSoci)
      {
      
         int identificador;
         Persistencia Persistencia = new Persistencia("entitats","propostaModificacio");
         ObjectePersistencia dadesNovaPropostaModificacio = new ObjectePersistencia();
      
         dadesNovaPropostaModificacio.afegirCampText("nom",propostaModificacioSoci.obteNom());
         dadesNovaPropostaModificacio.afegirCampText("cognom1",propostaModificacioSoci.obteCognom1());
         dadesNovaPropostaModificacio.afegirCampText("cognom2",propostaModificacioSoci.obteCognom2());
         dadesNovaPropostaModificacio.afegirCampText("identificadorPersonal",propostaModificacioSoci.obteIdentificadorPersonal());
         dadesNovaPropostaModificacio.afegirCampText("adresa",propostaModificacioSoci.obteAdresa());
         dadesNovaPropostaModificacio.afegirCampNumeric("telefon",propostaModificacioSoci.obteTelefon());
         dadesNovaPropostaModificacio.afegirCampData("dataNaixement",propostaModificacioSoci.obteDataNaixement());
         dadesNovaPropostaModificacio.afegirCampNumeric("numSoci",propostaModificacioSoci.obteNumSoci());
      
         identificador=Persistencia.afegeixElement(dadesNovaPropostaModificacio);
      
         dadesNovaPropostaModificacio.destructorObjectePersistencia();
         Persistencia.destructorPersistencia();
      
         return identificador;
      
      }
   
   }