   package Entitats.Soci;

   import Entitats.Persistencia.Persistencia;
   import Entitats.Persistencia.Llistat;
   import Entitats.Persistencia.Resultat;
   import Entitats.Persistencia.ObjectePersistencia;

/**
* Classe que permet mantenir una relacio entre socis i les imatges associades a aquests
*@author Jordi Moles i Blanco, Mar 2006
*/

   public class RelacioImatgesSoci
   {
   
      private Persistencia persistencia; //Objecte que ens permet obtenir informacio de la BBDD.
      private Llistat llistat; 
      private Resultat resultat;
      private static int ordre=0; 
   
   /**
   * Constructor de la classe. Inicialitza els elements necessaris per tal d'accedir a la BBDD.
   */
      public RelacioImatgesSoci()
      {
      
         persistencia = new Persistencia("entitats","imatgesSoci");
         llistat=null;
      
      }
   
   /**
   * Destructor de la classe
   * Metode que elimina les referencies als objectes manipulats per una RelacioImatgesSoci
   * per tal de que l'objecte "Gargabe Collector" del Java n'alliberi la memoria assignada
   * quan descobreixi que l'objecte ja no te cap referencia.
   */
      public void destructorRelacioImatgesSoci()
      {
      
         persistencia.destructorPersistencia();
         if(llistat!=null)llistat.destructorLlistat();
         resultat=null;
      
      }
   
   //////Els seguents metodes representen l'iterador de imatges associades a un soci\\\\\
                                                                                                      
   /**
   * Metode que cerca totes les imatges sobre socis   
   */
   
      public void obteImatgesSocis()
      
      {
      
         llistat=new Llistat(persistencia.obteTots());
      
      }
   
   /**
   * Metode que cerca les imatges relacionades a un soci donat l'identificador d'aquest
   *@param numSoci Enter amb el valor que identifica el soci
   */
      public void obteImatgesSoci(int numSoci)
      {
      
         llistat=new Llistat(persistencia.obtePerCampNumeric("soci_numSoci",numSoci));
      
      }
   
   
   /**
   * Metode que ens informa si hi ha imatges en l'iterador que encara no hem consultat
   *@return Boolea amb valor positiu si hi ha elements i negatiu si no n'hi ha.
   */
      public boolean hiHaImatges() 
      {
      
         return llistat.hiHaElements();
      
      }
   
   /**
   * Metode que ens permet obtenir l'identificador de la primera imatge que encara
   * no hem consultat. L'element no s'elimina del sistema pero si d'aquest iterador. Per tant,
   * haurem de tornar a executar obteImatgesSoci si volem tornar a tractar tots els elements.
   *@return Enter amb l'identificador de la imatge
   */
      public int obteImatge()   
      {
      
         resultat=(Resultat)llistat.obtePrimerElement();
         return Integer.parseInt((String)resultat.obteElement("imatge_numImatge"));
      
      }
   
   //////////Fi iterador\\\\\\\
                                                                                                                                                                                                                                       
   /**
   * Metode que permet afegir una imatge, es a dir, relacionar una nova imatge amb un soci.
   *@param numSoci Enter amb el identificador de soci
   *@param imatge Enter amb el valor que identifica una imatge
   */
      public static void afegirImatgeASoci(int numSoci,int imatge)
      {
      
         Persistencia Persistencia = new Persistencia("entitats","imatgesSoci");
         ObjectePersistencia dadesNovaImatgePerSoci = new ObjectePersistencia();
      
         dadesNovaImatgePerSoci.afegirCampNumeric("soci_numSoci",numSoci);
         dadesNovaImatgePerSoci.afegirCampNumeric("imatge_numImatge",imatge);
      
         Persistencia.afegeixElement(dadesNovaImatgePerSoci);
      
         dadesNovaImatgePerSoci.destructorObjectePersistencia();
         Persistencia.destructorPersistencia();
      
      }
   
   /*
   * Metode que elimina la relacio d'un determinat soci amb totes les imatges relacionades.
   *@param numSoci Enter amb el valor corresponent 
   */
      public static void eliminaImatges(int numSoci)
      {
      
         Persistencia Persistencia = new Persistencia("entitats","imatgesSoci");
      
         Persistencia.eliminaPerCampNumeric("soci_numSoci",numSoci);
      
         Persistencia.destructorPersistencia();
      
      }
   
   }