   package Entitats.Soci;

   import Entitats.Persistencia.Persistencia;
   import Entitats.Persistencia.Resultat;
   import Entitats.Soci.Soci;
   import Entitats.Soci.RelacioImatgesSoci;
   import Entitats.Persistencia.Llistat;
   import Entitats.Persistencia.ObjectePersistencia;
   import java.util.Random;

/**
* Classe que permet mantenir una relacio de Socis. Altes, baixes, modificacions i
* consultes de socis per qualsevol camp son possibles. Implementa els elements necessaris
* per accedir a la BBDD o qualsevol altre sistema que utilitzem per conservar les dades
* que manipulem de forma permanent. Aixo es gracies a l'us de la classe Persistencia, que 
* ens permet treballar amb socis sense haver de preocupar-nos de com el sistema
* tracta les dades.
*@author Jordi Moles i Blanco, Mar 2005
*/

   public class RelacioSocis
   {
   
      protected Persistencia persistencia;
      protected Resultat resultat;
      protected Llistat llistat;
   
   /**
   * Constructor de la classe. Inicialitza els elements necessaris per tal d'accedir a la BBDD.
   */
      public RelacioSocis()
      {
      
         persistencia = new Persistencia("entitats","soci");
         llistat=null;
      
      }
   
   /**
   * Destructor de la classe
   * Metode que elimina les referencies als objectes manipulats per una RelacioSocis
   * per tal de que l'objecte "Gargabe Collector" del Java n'alliberi la memoria assignada
   * quan descobreixi que l'objecte ja no te cap referencia.
   */
      public void destructorRelacioSocis()
      {
      
         persistencia.destructorPersistencia();
         resultat=null;
         if(llistat!=null)llistat.destructorLlistat();
      
      }
   
   //////Els seguents metodes representen l'iterador de socis recuperats del sistema\\\\\
                                   
   /*
   * Metode que permet recuperar aquells socis per l'any que van neixer
   *@param AnyNaixement Enter amb el valor de l'any
   */                                                                                                                                                                                                                                                                                                                                                                
      public void obteSociPerAnyNaixement(int AnyNaixement)
      {
      
         llistat = new Llistat(persistencia.obtePerAny("dataNaixement",AnyNaixement));
      
      }
   
   /*
   * Metode que permet recuperar aquells socis que van neixer a partir d'un any determinat
   *@param AnyNaixement Enter amb el valor de l'any
   */
      public void obteSociPerAnyNaixementAPartirDe(int AnyNaixement)        
      {
      
         llistat = new Llistat(persistencia.obtePerAnyAPartirDe("dataNaixement",AnyNaixement));
      
      }
   
   
   /*
   * Metode que permet recuperar aquells socis que van neixer abans d'un any determinat
   *@param AnyNaixement Enter amb el valor de l'any
   */
      public void obteSociPerAnyNaixementMenorOIgualQue(int AnyNaixement)
      
      {
      
         llistat = new Llistat(persistencia.obtePerAnyMenorOIgualQue("dataNaixement",AnyNaixement));
      
      }
   
   /*
   * Metode que permet recuperar aquells socis que van ingresar a l'intentitat a un any determinat
   *@param AnyAltaEntitat Enter amb el valor de l'any
   */
      public void obteSociPerAnyIngresEntitat(int AnyAltaEntitat)
      {
      
         llistat = new Llistat(persistencia.obtePerAny("dataAltaEntitat",AnyAltaEntitat));
      
      }
   
   /*
   * Metode que permet recuperar aquells socis que van ingresar a partir d'un any determinat
   *@param AnyAltaEntitat Enter amb el valor de l'any
   */
      public void obteSociPerAnyIngresEntitatAPartirDe(int AnyAltaEntitat)
      
      {
      
         llistat = new Llistat(persistencia.obtePerAnyAPartirDe("dataAltaEntitat",AnyAltaEntitat));
      
      }
   
   /*
   * Metode que permet recuperar aquells socis que van ingresar abans d'un any determinat
   *@param AnyAltaEntitat Enter amb el valor de l'any
   */
      public void obteSociPerAnyIngresEntitatMenorOIgualQue(int AnyAltaEntitat)
      
      {
      
         llistat = new Llistat(persistencia.obtePerAnyMenorOIgualQue("dataAltaEntitat",AnyAltaEntitat));
      
      }
   
   /**
   * Metode que ens permet obtenir un conjunt de socis que coincideixen amb els valors
   * donats. Es pot cercar un soci aportant valors de tots els seus camps
   * o nomes d'un si es desitja, i a mes, aquest camp pot ser qualsevol.
   *@param DadesSociABuscar ObjectePersistencia amb tots els valors dels camps
   *per els que volem cercar un soci.
   */
      public void obteSociPerMultiplesCamps(ObjectePersistencia DadesSociABuscar)
      {
      
         llistat = new Llistat(persistencia.obtePerDiversosCamps(DadesSociABuscar));
      
      }
   
   /**
   * Metode que ens informa si hi ha socis en l'iterador que encara no hem consultat
   *@return Boolea amb valor positiu si hi ha elements i negatiu si no n'hi ha.
   */
      public boolean hiHaSocis()
      {
      
         return llistat.hiHaElements();
      
      }
   
   /**
   * Metode que ens permet obtenir el primer objecte soci que encara
   * no hem consultat, amb totes les dades disponibles sobre aquest en el sistema.
   *@return Soci amb totes les dades disponibles
   */
      public Soci obteSoci()
      {
      
         resultat=(Resultat)llistat.obtePrimerElement();
         Soci soci = new Soci();
      
         soci.estableixNumSoci(Integer.parseInt((String)resultat.obteElement("numSoci")));
         soci.estableixNom((String)resultat.obteElement("nom"));
         soci.estableixCognom1((String)resultat.obteElement("cognom1"));
         soci.estableixCognom2((String)resultat.obteElement("cognom2"));
         soci.estableixIdentificadorPersonal((String)resultat.obteElement("identificadorPersonal"));
         soci.estableixAdresa((String)resultat.obteElement("adresa"));
         soci.estableixTelefon(Integer.parseInt((String)resultat.obteElement("telefon")));
         soci.estableixDataNaixement((String)resultat.obteElement("dataNaixement"));
         soci.estableixDataAltaEntitat((String)resultat.obteElement("dataAltaEntitat"));
         if(Integer.parseInt((String)resultat.obteElement("actiu"))==1)
            soci.actiu();
         else soci.inactiu();
      
         return soci;
      
      }
   
   //////////Fi iterador\\\\\\\
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      
   /**
   * Metode que ens permet obtenir un soci a partir del numero de soci
   *@param numSoci Enter amb el valor que identifica una soci
   *de manera unica
   *@return Soci amb totes les dades disponibles del soci sol.licitat
   */
      public static Soci obteSociPerNumSoci(int numSoci)
      {
      
         Soci Soci = new Soci();
         Persistencia Persistencia = new Persistencia("entitats","soci");
         Resultat Resultat;
         Llistat ll;
         ll = new Llistat(Persistencia.obtePerCampNumeric("numSoci",numSoci));
         if(ll.hiHaElements())
         {
            Resultat=(Resultat)ll.obtePrimerElement();
            Soci=emplenaSoci(Resultat);
         }
         Persistencia.destructorPersistencia();
         ll.destructorLlistat();
         Resultat=null;
      
         return Soci;
      
      }
   
   /**
   * Metode que ens permet obtenir un soci a partir de l'identificador personal
   *@param identificadorPersonal Enter amb el valor que identifica una persona
   *de manera unica
   *@return Soci amb totes les dades disponibles del soci sol.licitat
   */
      public static Soci obteSociPerIdentificadorPersonal(String identificadorPersonal)
      {
      
         Soci Soci = new Soci();
         Persistencia Persistencia = new Persistencia("entitats","soci");
         Resultat Resultat;
         Llistat ll;
         ll = new Llistat(Persistencia.obtePerCampText("identificadorPersonal",identificadorPersonal));
         if(ll.hiHaElements())
         {
            Resultat=(Resultat)ll.obtePrimerElement();
            Soci=emplenaSoci(Resultat);
         }
         Persistencia.destructorPersistencia();
         ll.destructorLlistat();
         Resultat=null;
      
         return Soci;
      
      }
   
   /*
   * Metode que ens permet saber si existeix un soci amb un numero concret
   *@param numSoci Enter amb el numero de soci
   *@return Boolean amb valor true si no existeix i false en cas contrari
   */
      public static boolean noExisteix(int numSoci)
      {
      
         if(obteSociPerNumSoci(numSoci).obteNumSoci()==0) 
            return true;
         else 
            return false;
      
      }
   
   /*
   * Metode que ens permet saber si existeix un determinat soci a partir del seu nom
   *@param nom String amb el nom del soci
   *@param cognom1 String amb el primer cognom del soci
   *@param cognom2 String amb el segon cognom del soci
   *@return Boolean amb valor true si no existeix i false en cas contrari
   */
      public static boolean noExisteixAmbNoms(String nom,String cognom1,String cognom2)
      {
      
         ObjectePersistencia o = new ObjectePersistencia();
         o.afegirCampText("nom",nom);
         o.afegirCampText("cognom1",cognom1);
         o.afegirCampText("cognom2",cognom2);
      
         RelacioSocis relacioSocis = new RelacioSocis(); 
         relacioSocis.obteSociPerMultiplesCamps(o);
      
         if(relacioSocis.hiHaSocis()) 
            return false;
         else 
            return true;
      
      }
   
   /*
   * Metode que ens permet saber si existeix un soci amb un identificador concret
   *@param identificadorPersonal String amb el valor del identificador
   *@return Boolean amb valor true si no existeix i false en cas contrari
   */
      public static boolean noExisteix(String identificadorPersonal)
      {
      
         if(obteSociPerIdentificadorPersonal(identificadorPersonal).obteNumSoci()==0) 
            return true;
         else 
            return false;
      
      }
   
      protected static Soci emplenaSoci(Resultat Resultat)
      {
      
         Soci Soci = new Soci();
      
         Soci.estableixNumSoci(Integer.parseInt((String)Resultat.obteElement("numSoci")));
         Soci.estableixNom((String)Resultat.obteElement("nom"));
         Soci.estableixCognom1((String)Resultat.obteElement("cognom1"));
         Soci.estableixCognom2((String)Resultat.obteElement("cognom2"));
         Soci.estableixIdentificadorPersonal((String)Resultat.obteElement("identificadorPersonal"));
         Soci.estableixAdresa((String)Resultat.obteElement("adresa"));
         Soci.estableixTelefon(Integer.parseInt((String)Resultat.obteElement("telefon")));
         Soci.estableixDataNaixement((String)Resultat.obteElement("dataNaixement"));
         Soci.estableixDataAltaEntitat((String)Resultat.obteElement("dataAltaEntitat"));
         if((Integer.parseInt((String)(Resultat.obteElement("actiu"))))==0)
            Soci.inactiu();
         else Soci.actiu();
      
         return Soci;
      
      }
   
   /**
   * Metode que permet donar de baixa un soci del sistema a partir del numero de soci.
   * Se'n preserven les dades per raons estadistiques i de consistencia del sistema, pero, 
   * alhora, es "desactiva" per tal que aquestes dades no estiguin disponibles en un consulta
   *@param numSoci Enter amb el numero de soci
   *@return Boolean amb valor true si l'operaci ha finalitzat correctament i false en cas contrari
   */
      public static boolean eliminaSociPerNumSoci(int numSoci)
      {
      
         boolean totOK;
      
         Persistencia Persistencia = new Persistencia("entitats","soci");
         ObjectePersistencia obj = new ObjectePersistencia();
         obj.afegirCampNumeric("actiu",0);
         totOK=Persistencia.actualitzaDiversosCampsPerCampNumeric("numSoci",numSoci,obj);
         obj.destructorObjectePersistencia();
         Persistencia.destructorPersistencia();
      
         return totOK;
      
      }
   
   /**
   * Metode que permet donar de baixa un soci del sistema a partir de l'identificador personal.
   * Se'n preserven les dades per raons estadistiques i de consistencia del sistema, pero, 
   * alhora, es "desactiva" per tal que aquestes dades no estiguin disponibles en un consulta
   *@param identificadorPersonal String amb l'identificador personal de la persona
   *@return Boolean amb valor true si l'operaci ha sigut realitzada correctament i false en cas contrari
   */
      public static boolean eliminaSociPerIdentificadorPersonal(String identificadorPersonal)
      {
      
         boolean totOK;
      
         Persistencia Persistencia = new Persistencia("entitats","soci");
         totOK=eliminaSociPerNumSoci(obteSociPerIdentificadorPersonal(identificadorPersonal).obteNumSoci());
         Persistencia.destructorPersistencia();
      
         return totOK;
      
      }
   
   /**
   * Metode que permet actualitzar de cop diversos camps d'un soci
   *@param numSoci Enter amb el valor que ens permetra trobar el soci
   *que volem modificar 
   *@param dadesActualitzadesSoci ObjectePersistencia amb els noms dels camps a modificar, aixi com
   *els valors d'aquests
   *@return Boolean amb valor true si l'operaci s'ha dut a terme correctament i false en cas contrari.
   */
      public static boolean actualitzaDiversosCampsSociPerNumSoci(int numSoci,ObjectePersistencia dadesActualitzadesSoci)
      {
      
         boolean totOK;
      
         Persistencia Persistencia = new Persistencia("entitats","soci");
         totOK = Persistencia.actualitzaDiversosCampsPerCampNumeric("numSoci",numSoci,dadesActualitzadesSoci);
         Persistencia.destructorPersistencia();
      
         return totOK;
      
      }
   
   /**
   * Metode que permet actualitzar de cop diversos camps d'un soci
   *@param identificadorPersonal String amb el valor que ens permetra trobar el soci
   *que volem modificar 
   *@param dadesActualitzadesSoci ObjectePersistencia amb els noms dels camps a modificar, aixi com
   *els valors d'aquests
   *@return Boolean amb valor true si l'operaci ha finalitzat correctament i false en cas contrari. 
   */
      public static boolean actualitzaDiversosCampsSociPerIdentificadorPersonal(String identificadorPersonal,ObjectePersistencia dadesActualitzadesSoci)
      
      {
      
         boolean totOK;
      
         Persistencia Persistencia = new Persistencia("entitats","soci");
         totOK = Persistencia.actualitzaDiversosCampsPerCampNumeric("numSoci",obteSociPerIdentificadorPersonal(identificadorPersonal).obteNumSoci(),dadesActualitzadesSoci);
         Persistencia.destructorPersistencia();
      
         return totOK;
      
      }
   
   /**
   * Metode que ens permet afegir una soci a la relacio
   *@param soci Objecte Soci amb les dades del nou soci que volem afegir
   *al sistema
   *@return Enter amb el valor que el sistema genera per a aquest nou element, valor
   *que l'identificara de manera unica en el sistema.
   */
      public static int afegirSoci(Soci soci)
      {
      
         int identificador;
         Persistencia Persistencia = new Persistencia("entitats","soci");
         Persistencia Persistencia2 = new Persistencia("entitats","usuariAplicacio");
         ObjectePersistencia dadesNouSoci = new ObjectePersistencia();
         ObjectePersistencia dadesNouUsuari = new ObjectePersistencia();
      
         dadesNouSoci.afegirCampText("nom",soci.obteNom());
         dadesNouSoci.afegirCampText("cognom1",soci.obteCognom1());
         dadesNouSoci.afegirCampText("cognom2",soci.obteCognom2());
         dadesNouSoci.afegirCampText("identificadorPersonal",soci.obteIdentificadorPersonal());
         dadesNouSoci.afegirCampText("adresa",soci.obteAdresa());
         dadesNouSoci.afegirCampNumeric("telefon",soci.obteTelefon());
         dadesNouSoci.afegirCampData("dataNaixement",soci.obteDataNaixement());
         dadesNouSoci.afegirCampData("dataAltaEntitat",soci.obteDataAltaEntitat());
      
         Random generador = new Random();
      
         dadesNouUsuari.afegirCampText("usuari",soci.obteNom().substring(0,2)+soci.obteCognom1().substring(0,2)+(generador.nextInt()%1000));
         dadesNouUsuari.afegirCampText("contrasenya",soci.obteIdentificadorPersonal());
         dadesNouUsuari.afegirCampNumeric("nivell",1);
      
         identificador=Persistencia.afegeixElement(dadesNouSoci);
         Persistencia2.afegeixElement(dadesNouUsuari);
      
         RelacioImatgesSoci relSoci = new RelacioImatgesSoci();
      
         if(soci.obteImatge()!=0)
         {
            relSoci.afegirImatgeASoci(identificador,soci.obteImatge());  
         }
      
         dadesNouSoci.destructorObjectePersistencia();
         dadesNouUsuari.destructorObjectePersistencia();
         Persistencia.destructorPersistencia();
         Persistencia2.destructorPersistencia();
         relSoci.destructorRelacioImatgesSoci();
      
         return identificador;
      
      }
   
   }