   package Entitats.Soci;

   import Entitats.Temps.Data;

/**
* Classe que permet la creacio i manipulacio de socis, els quals poden ser usats en qualsevol
* entitat/institucio que en tingui.
* No s'implementa en aquest punt cap validacio de les dades assignades a l'objecte, doncs
* es creu funcio indispensable de les capes superiors de l'aplicacio, les capes que 
* utilitzen aquesta classe. 
*@author Jordi Moles i Blanco, Mar 2005
*/

   public class Soci
   {
   
      private int numSoci; //Enter amb el valor que identifica el soci de manera unica
   								//en el sistema.
      private String nom; //Nom del soci
      private String cognom1; //Primer cognom del soci
      private String cognom2; //Segon cognom del soci
      private String identificadorPersonal; //String amb el valor d'algun document
   													  //que identifica la persona de manera
   													  //unica, com ara el numero de passaport
      private String adresa; //Adresa del soci
      private int telefon; //Telefon de contacte del soci
      private Data dataNaixement; //Data de naixement del soci.
      private Data dataAltaEntitat; //Data d'alta del soci a l'entitat
      private int actiu; //Indica si el soci forma part de l'entitat en aquest moment
      private int imatge; 
   
   /**
   *Constructor de la classe.
   *Inicialitza tots els atributs numerics a 0 i els strings a "" (Blanc)
   *La data d'alta del soci a l'entitat s'inicialitza amb la data que el sistema te 
   *en el moment de la creacio de l'objecte.
   *La data de naixement del soci s'estableix a 00/00/0000
   */
      public Soci() 
      {
      
         numSoci=0;
         identificadorPersonal="";
         nom="";
         cognom1="";
         cognom2="";
         adresa="";
         telefon=0;
         dataNaixement=new Data();
         dataAltaEntitat=new Data();
         actiu=1;
         imatge = 0;
      
      }
   
   /**
   *Destructor de la classe
   *Metode que elimina les referencies als objectes manipulats per un soci
   *per tal de que l'objecte "Gargabe Collector" del Java n'alliberi la memoria assignada
   *quan descobreixi que l'objecte ja no te cap referencia.
   */
      public void destructorSoci()
      {
      
         dataNaixement.destructorData();
         dataAltaEntitat.destructorData();
      
      }
   
   /**
   * Metode que assigna el numero de soci
   *@param numSoci Enter amb el valor que identifica el soci de manera unica en el sistema
   */
      public void estableixNumSoci(int numSoci)
      {
      
         this.numSoci=numSoci;
      
      }
   
   /**
   *Metode que estableix un valor per al nom soci. 
   *La informacio que hi havia en aquest atribut es perdra de forma definitiva
   *@param nom String que conte el valor que es vol assignar a nom
   */
      public void estableixNom(String nom) 
      {
      
         this.nom=nom;
      
      }
   
   /**Metode que estableix un valor per al primer cognom del soci. 
   *La informacio que hi havia en aquest atribut es perdra de forma definitiva
   *@param cognom1 String que conte el valor que es vol assignar al primer cognom del soci
   */
      public void estableixCognom1(String cognom1) 
      {
      
         this.cognom1=cognom1;
      
      }
   
   /**Metode que estableix un valor per al segon cognom del soci. 
   *La informacio que hi havia en aquest atribut es perdra de forma definitiva
   *@param cognom2 String que conte el valor que es vol assignar al segon cognom del soci
   */
      public void estableixCognom2(String cognom2) 
      {
      
         this.cognom2=cognom2;
      
      }
   
   /**Metode que estableix un valor per a identificador. 
   *La informacio que hi havia en aquest atribut es perdra de forma definitiva
   *@param identificadorPersonal String que conte el valor que es vol assignar al identificador del soci
   */       
      public void estableixIdentificadorPersonal(String identificadorPersonal)       
      {
      
         this.identificadorPersonal=identificadorPersonal;
      
      }
   
   /**Metode que estableix un valor per a l'adresa del soci. 
   *La informacio que hi havia en aquest atribut es perdra de forma definitiva
   *@param adresa String que conte el valor que es vol assignar a l'adresa del soci
   */
      public void estableixAdresa(String adresa) 
      {
      
         this.adresa=adresa;
      
      }
   
   /**Metode que estableix un valor per al telefon de contacte del soci. 
   *La informacio que hi havia en aquest atribut es perdra de forma definitiva
   *@param telefon Enter que conte el valor que es vol assignar al telefon de contacte del soci
   */
      public void estableixTelefon(int telefon) 	
      {
      
         this.telefon=telefon;
      
      }
   
   /**Metode que estableix un valor per a la data de naixement del soci. 
   *La informacio que hi havia en aquest atribut es perdra de forma definitiva
   *@param dia Enter amb el valor del dia del mes que ha de formar part de la data.
   *@param mes Enter amb el valor del mes de l'any que ha de formar part de la data.
   *@param any Enter amb el valor de l'any que ha de formar part de la data
   */
      public void estableixDataNaixement(int dia,int mes,int any)
      {
      
         dataNaixement.estableixData(dia,mes,any);
      
      }
   
   /**
   * Metode que estableix un valor per a la data de naixement del soci.
   *@param dataNaixement String amb la data de naixement del soci, en un dels seguents
   *formats: dd/mm/aaaa o dd-mm-aaaa
   */
      public void estableixDataNaixement(String dataNaixement)        
      {
      
         int dia,mes,any;
         int posicio1,posicio2;
      
         posicio1=dataNaixement.indexOf("-");
         if(posicio1<0) posicio1=dataNaixement.indexOf("/");
         dia=Integer.parseInt(dataNaixement.substring(0,posicio1));
      
         posicio1++;
         posicio2=dataNaixement.indexOf("-",posicio1);
         if(posicio2<0) posicio2=dataNaixement.indexOf("/",posicio1);
         mes=Integer.parseInt(dataNaixement.substring(posicio1,posicio2));
      
         posicio2++;
         any=Integer.parseInt(dataNaixement.substring(posicio2).trim());
      
         estableixDataNaixement(dia,mes,any);
      
      }
   
   /**Metode que estableix un valor per a la data d'alta del soci a l'entitat. 
   *La informacio que hi havia en aquest atribut es perdra de forma definitiva
   *@param dia Enter amb el valor del dia del mes que ha de formar part de la data.
   *@param mes Enter amb el valor del mes de l'any que ha de formar part de la data.
   *@param any Enter amb el valor de l'any que ha de formar part de la data
   */
      public void estableixDataAltaEntitat(int dia,int mes,int any)
      {
      
         dataAltaEntitat.estableixData(dia,mes,any);
      
      }
   
   /**
   * Metode que estableix un valor per a la data d'alta del soci a l'entitat.
   *@param dataAltaEntitat String amb la data d'alta del soci a l'entitat, en un dels seguents
   *formats: dd/mm/aaaa o dd-mm-aaaa
   */
      public void estableixDataAltaEntitat(String dataAltaEntitat)        
      {
      
         int dia,mes,any;
         int posicio1,posicio2;
      
         posicio1=dataAltaEntitat.indexOf("-");
         if(posicio1<0) posicio1=dataAltaEntitat.indexOf("/");
         dia=Integer.parseInt(dataAltaEntitat.substring(0,posicio1));
      
         posicio1++;
         posicio2=dataAltaEntitat.indexOf("-",posicio1);
         if(posicio2<0) posicio2=dataAltaEntitat.indexOf("/",posicio1);
         mes=Integer.parseInt(dataAltaEntitat.substring(posicio1,posicio2));
      
         posicio2++;
         any=Integer.parseInt(dataAltaEntitat.substring(posicio2).trim());
      
         estableixDataAltaEntitat(dia,mes,any);
      
      }
   
   	/**
   	* Metode que estableix la ruta on es troba la imatge associada al soci
   	*@param imatge Enter amb el identificador de la imatge 
   	*/
      public void estableixImatge(int imatge)
      {
         this.imatge = imatge;
      }
   
   
   /**
   * Metode que marca el soci com a soci actiu, es a dir, que actualment forma part 
   *de l'entitat
   */
      public void actiu()
      {
      
         actiu=1;
      
      }
   
   /**
   * Metode que marca el soci com a soci inactiu, es a dir, que actualment no forma part
   *de l'entitat
   */
      public void inactiu()
      {
      
         actiu=0;
      
      }
   
   /**
   * Metode per obtenir el numero de soci
   *@return Enter amb el numero de soci
   */
      public int obteNumSoci()
      {
      
         return numSoci;
      
      }
   
   /**
   *Metode per obtenir el nom del soci
   *@return String amb el nom del soci.
   */
      public String obteNom()
      {
      
         return nom.trim();
      
      }
   
   /**
   *Metode per obtenir el primer cognom del soci
   *@return String amb el primer cognom del soci.
   */        
      public String obteCognom1()
      {
      
         return cognom1.trim();
      
      }
   
   /**
   *Metode per obtenir el segon cognom del soci
   *@return String amb el segon cognom del soci.
   */                
      public String obteCognom2()
      {
      
         return cognom2.trim();
      
      }
   
   /**
   *Metode per obtenir el valor que identifica una persona de manera unica 
   *(Document nacional d'identitat, passaport, etc.)
   *@return String amb el valor de l'identificador.
   */        
      public String obteIdentificadorPersonal()        
      {
      
         return identificadorPersonal.trim();
      
      }
   
   /**
   *Metode per obtenir l'adresa del soci
   *@return String amb l'adresa del soci.
   */             
      public String obteAdresa()
      {
      
         return adresa.trim();
      
      }
   
   /**
   *Metode per obtenir el telefon de contacte del soci
   *@return Enter amb el telefon de contacte del soci.
   */                
      public int obteTelefon()
      {
      
         return telefon;
      
      }
   
   /**
   *Metode per obtenir la data de naixement del soci
   *@return String amb la data completa del soci en format dd/mm/aaaa.
   */               
      public String obteDataNaixement()       
      {
      
         return dataNaixement.obteData().trim();
      
      }
   
   /**
   *Metode per obtenir la data d'alta del soci a l'entitat
   *@return String amb la data completa d'alta en format dd/mm/aaaa.
   */               
      public String obteDataAltaEntitat()       
      {
      
         return dataAltaEntitat.obteData().trim();
      
      }
   
   /**
   *Metode que ens permet obtenir el identificador de la imatge associada al soci
   *@return Enter amb el valor que identifica una imatge de manera unica al sistema
   */
      public int obteImatge()
      {
         return imatge;
      }
   
   /**
   * Metode que ens permet saber si el soci es actiu i per tant en podem consultar les dades
   * o be nomes conservem l'element per raons estadistiques o de consistencia del sistema. 
   *@return Boolea amb valor positiu si el soci es actiu
   */
      public boolean esActiu()
      {
      
         return actiu==1;
      
      }
   
   /**
   * Metode que ens permet establir l'estat del soci com a esborrar, es a dir, 
   *que la propia persona ens va demanar que li eliminessim les dades
   */
      public void esborrat()
      {
      
         actiu=2;
      
      }
   
   /**
   * Metode que ens permet saber si el soci esta "esborrat"
   *@return Boolean amb valor true si el soci esta esborrar i false en cas contrari
   */
      public boolean estaEsborrat()
      {
         return actiu==2;
      }
   }
