   package Entitats.Temps;

   import java.util.GregorianCalendar;
   import java.util.Date;
   import java.text.SimpleDateFormat;

/**
* Classe "eina" per a la creacio i manipulacio de dates. Les classes implementades per l'usuari
* normalment treballen amb el format de data dd/mm/aaaa. Per contra, el sistema normalment
* treballa amb els milisegons que han trasncorregut des del 1970. Per tant, aquesta classe
* crea la capa necessaria per poder treballar en ambdos sentits d'una manera totalment
* transparent a l'usuari.
*@author Jordi Moles i Blanco, Mar 2005
*/

   public class Data
   {
   
      private GregorianCalendar calendari;
      private SimpleDateFormat sdf;
   
   	/**
   	*Constructor de la classe. Inicialitza l'objecte amb la data actual del sistema.
   	*/
      public Data()
      {
      
         calendari=new GregorianCalendar();
         sdf=new SimpleDateFormat();
      
      }
   
   /**
   *Constructor de la classe. Inicialitza l'objecte amb la data recollida per parametre.
   *@param dia Enter amb el valor del dia del mes que ha de formar part de la nova data.
   *@param mes Enter amb el valor del mes de l'any que ha de formar part de la nova data.
   *@param any Enter amb el valor de l'any que ha de formar part de la nova data
   */
      public Data(int dia,int mes,int any)
      {
      
         calendari=new GregorianCalendar(any,mes-1,dia);
         sdf=new SimpleDateFormat();
      
      }
   
   /**
   	*Destructor de la classe
   	*Metode que elimina les referencies als objectes manipulats per una Data
   	*per tal de que l'objecte "Gargabe Collector" del Java n'alliberi la memoria assignada
   	*quan descobreixi que l'objecte ja no te cap referencia.
   	*/
      public void destructorData()
      {
      
         calendari=null;
         sdf=null;
      
      }
   
   	/**
   	*Metode que ens permet recuperar la data completa, en format dd/mm/aaaa que guardem
   	*en l'objecte
   	*@return Un string amb la data completa en format dd/mm/aaaa
   	*/
      public String obteData()
      {
      
         sdf.applyPattern("dd/MM/yyyy");
         return sdf.format(calendari.getTime());
      
      }
   
   /**
   *Metode per a obtenir un enter amb el valor de l'any que l'objecte te guardat
   *@return Un enter amb el valor numeric de l'any, 4 xifres.
   */
      public int obteAny()
      {
      
         sdf.applyPattern("yyyy");
         return Integer.parseInt(sdf.format(calendari.getTime()));
      
      }
   
   /**
   *Metode per a la modificacio d'una data. El contingut anterior es perdra de forma definitiva.
   *La data s'ha de modificar en tots els seus parametres (dia, mes i any), per tant, s'ha
   *d'introduir la data completa, tal i com volem que quedi guardada de forma definitiva
   *@param dia Enter amb el valor del dia del mes que ha de formar part de la nova data.
   *@param mes Enter amb el valor del mes de l'any que ha de formar part de la nova data.
   *@param any Enter amb el valor de l'any que ha de formar part de la nova data
   */
      public void estableixData(int dia,int mes,int any)
      {
      
         calendari.set(any,mes-1,dia);
      
      }
   
   }