   package Entitats.Temps;

   import java.util.GregorianCalendar;
   import java.text.SimpleDateFormat;

/**
* Classe "eina" per a la creacio i manipulacio de hores. Les classes implementades per l'usuari
* normalment treballen amb el format de data hh:mm:ss (format 24h). Per contra, el sistema normalment
* treballa amb els milisegons que han transcorregut des del 1970. Per tant, aquesta classe
* crea la capa necessaria per poder treballar en ambdos sentits d'una manera totalment
* transparent a l'usuari.
*@author Jordi Moles i Blanco, Mar 2005
*/

   public class Hora
   {
   
      private GregorianCalendar calendari;
      private SimpleDateFormat sdf;
   
   /**
   *Constructor de la classe. Inicialitza l'objecte amb l'hora actual.
   */         
      public Hora()    
      {
      
         calendari=new GregorianCalendar();
         sdf=new SimpleDateFormat();
      
      }
   
   /**
   *Constructor de la classe. Inicialitza l'objecte amb la hora recollida per parametre.
   *@param hora Enter amb el valor de la hora.
   *@param minut Enter amb el valor del minut.
   *@param segon Enter amb el valor del segon
   */
      public Hora(int hora,int minut,int segon)
      {
      
         calendari=new GregorianCalendar(0,0,0,hora,minut,segon);
         sdf=new SimpleDateFormat();
      
      }
   
   /**
   	*Destructor de la classe
   	*Metode que elimina les referencies als objectes manipulats per una hora
   	*per tal de que l'objecte "Gargabe Collector" del Java n'alliberi la memoria assignada
   	*quan descobreixi que l'objecte ja no te cap referencia.
   	*/
      public void destructorHora()
      {
      
         calendari=null;
         sdf=null;
      
      }
   
   	/**
   	*Metode que ens permet recuperar la hora completa, en format hh:mm:ss que guardem
   	*en l'objecte
   	*@return Un string amb la hora completa en format hh:mm:ss
   	*/
      public String obteHora()
      {
      
         sdf.applyPattern("HH:mm:ss");
         return sdf.format(calendari.getTime());
      
      }
   
   /**
   *Metode per a la modificacio d'una hora. El contingut anterior es perdra de forma definitiva.
   *La hora s'ha de modificar en tots els seus parametres (hora i minut), per tant, s'ha
   *d'introduir la hora completa, tal i com volem que quedi guardada de forma definitiva
   *@param hora Enter amb el valor de la hora.
   *@param minut Enter amb el valor del minut.
   *@param segon Enter amb el valor del segon
   */
      public void estableixHora(int hora,int minut,int segon)
      {
      
         calendari.set(0,0,0,hora,minut,segon);
      
      }
   
   }