   package Entitats.UsuariAplicacio;

   import Entitats.Temps.Hora;
   import Entitats.Temps.Data;
   import Entitats.Persistencia.Persistencia;
   import Entitats.Persistencia.Resultat;
   import Entitats.Persistencia.Llistat;
   import Entitats.Persistencia.ObjectePersistencia;

/**
* Classe "eina" per al registre de l'activitat dels usuaris. D'aquesta manera, podem per exempe loguejar totes les connexions 
* dels usuaris amb privilegis a la nostra aplicaci.
*@author Jordi Moles i Blanco, Mar 2005
*/

   public class RegistreConnexions
   {
   
      private Persistencia persistencia;
      private Resultat resultat;
      private Llistat llistat;
   
   /**
   * Constructor de la classe. Inicialitza els elements necessaris per tal d'accedir a la BBDD.
   */
   
      public RegistreConnexions()
      {
      
         persistencia = new Persistencia("entitats","registreConnexions");
      
      }
   
   /**
   * Destructor de la classe
   * Metode que elimina les referencies als objectes manipulats per un RegistreConnexions
   * per tal de que l'objecte "Gargabe Collector" del Java n'alliberi la memoria assignada
   * quan descobreixi que l'objecte ja no te cap referencia.
   */
   
      public void destructorRegistreConnexions()
      {
      
         persistencia.destructorPersistencia();
         resultat=null;
         llistat.destructorLlistat();
      
      }
   
   	//////Els seguents metodes representen l'iterador de connexions recuperades del sistema\\\\\
      
   /**
   * Metode que permet obtenir totes les connexions que s'han enregistrat al sistema
   */                            
      public void obteTotesLesConnexions()
      {
      
         llistat = new Llistat(persistencia.obteTots());
      
      }
   
   /**
   * Metode que permet obtenir totes les connexions que s'han enregistrart al sistema, ordenades descendentment per els
   *valors entrats
   *@param campsAOrdenar ObjectePersistencia amb el camps per els que hem d'ordenar els resultats
   */
      public void obteTotesLesConnexionsOrdenatsDescendentment(ObjectePersistencia campsAOrdenar)
      
      {
      
         llistat = new Llistat(persistencia.obteTotsOrdenatsDescendentment(campsAOrdenar));
      
      }
   
   /**
   * Metode que ens permet obtenir un conjunt de connexions que coincideixen amb els valors
   * donats. Es pot cercar una connexio aportant valors de tots els seus camps
   * o nomes d'un si es desitja, i a mes, aquest camp pot ser qualsevol.
   *@param dadesConnexioAConsultar ObjectePersistencia amb tots els valors dels camps
   *per els que volem cercar una connexio.
   */
   
      public void obteConnexioPerMultiplesCamps(ObjectePersistencia dadesConnexioAConsultar)
      
      {
      
         llistat = new Llistat(persistencia.obtePerDiversosCamps(dadesConnexioAConsultar));
      
      }
   
   /**
   * Metode que ens informa si hi ha connexions en l'iterador que encara no hem consultat
   *@return Boolea amb valor positiu si hi ha elements i negatiu si no n'hi ha.
   */
   
      public boolean hiHaConnexions()
      
      {
      
         return llistat.hiHaElements();
      
      }
   
   /**
   * Metode que ens permet obtenir el primer registre de connexio que encara
   * no hem consultat, amb totes les dades disponibles sobre aquest en el sistema.
   *@return String amb totes les dades disponibles
   */
   
      public String obteRegistreConnexio()
      
      {
      
         resultat=(Resultat)llistat.obtePrimerElement();
         String solucio="";
      
         solucio+=(String)resultat.obteElement("usuari")+" / ";
         solucio+=(String)resultat.obteElement("dataConnexio")+" / ";
         solucio+=(String)resultat.obteElement("horaConnexio");
      
         return solucio;
      
      }
   
   //////////Fi iterador\\\\\\\
                 
   
   /**
   * Metode que permet afegir (enregistrar) una nova activitat d'un usuari
   *@param usuari String amb el nom d'usuari
   */                                                 
      public static void afegirRegistreConnexio(String usuari)       
      {
      
         Persistencia Persistencia = new Persistencia("entitats","registreConnexions");
         ObjectePersistencia dadesNouRegistreConnexio = new ObjectePersistencia();
      
         dadesNouRegistreConnexio.afegirCampText("usuari",usuari);
         dadesNouRegistreConnexio.afegirCampData("dataConnexio",new Data().obteData());
         dadesNouRegistreConnexio.afegirCampHora("horaConnexio",new Hora().obteHora());
      
         Persistencia.afegeixElement(dadesNouRegistreConnexio);
      
         dadesNouRegistreConnexio.destructorObjectePersistencia();
         Persistencia.destructorPersistencia();
      
      }
   
   }