   package Entitats.UsuariAplicacio;

   import Entitats.Persistencia.Persistencia;
   import Entitats.Persistencia.Resultat;
   import Entitats.Persistencia.Llistat;
   import Entitats.Persistencia.ObjectePersistencia;
   import Entitats.UsuariAplicacio.UsuariAplicacio;
   import Entitats.Soci.RelacioSocis;

/**
* Classe que gestiona l'alta, baixa i modificacio d'usuaris de la nostra aplicacio.
*@author Jordi Moles i Blanco, Mar 2006
*/

   public class RelacioUsuarisAplicacio
   
   {
   
      private Persistencia persistencia;
      private Resultat resultat;
      private Llistat llistat;
   
   /**
   * Constructor de la classe. Inicialitza els elements necessaris per tal d'accedir a la BBDD.
   */
      public RelacioUsuarisAplicacio()
      {
      
         persistencia = new Persistencia("entitats","usuariAplicacio");
      
      }
   
   /**
   * Destructor de la classe
   * Metode que elimina les referencies als objectes manipulats per una RelacioUsuarisAplicacio
   * per tal de que l'objecte "Gargabe Collector" del Java n'alliberi la memoria assignada
   * quan descobreixi que l'objecte ja no te cap referencia.
   */
      public void destructorRelacioUsuarisAplicacio()
      {
      
         persistencia.destructorPersistencia();
         resultat=null;
         llistat.destructorLlistat();
      
      }
   
   //////Els seguents metodes representen l'iterador d'usuaris recuperats del sistema\\\\\
                                                        
   /**
   * Metode que ens permet obtenir tots els usuaris que poden accedir a la nostra aplicaci
   */             
      public void obteTotsElsUsuaris()
      {
      
         llistat = new Llistat(persistencia.obteTots());
      
      }
   
   /**
   * Metode que ens permet obtenir un conjunt d'usuaris que coincideixen amb els valors
   * donats. Es pot cercar un usuari aportant valors de tots els seus camps
   * o nomes d'un si es desitja, i a mes, aquest camp pot ser qualsevol.
   *@param dadesUsuariAConsultar ObjectePersistencia amb tots els valors dels camps
   *per els que volem cercar un usuari.
   */
      public void obteUsuariAplicacioPerMultiplesCamps(ObjectePersistencia dadesUsuariAConsultar)
      {
      
         llistat = new Llistat(persistencia.obtePerDiversosCamps(dadesUsuariAConsultar));
      
      }
   
   /**
   * Metode que ens permet saber si l'usuari que intenta identificar-se s un soci, que ja t el login/password creats per defecte
   *@param usuari String amb el valor de l'usuari
   *@param contrasenya String amb el valor de la contrasenya
   *@return Boolean amb valor True si correspon a un soci
   */
   
      public boolean esSoci(String usuari,String contrasenya)
      {
      
         ObjectePersistencia op = new ObjectePersistencia();
      
         op.afegirCampText("nom",usuari);
         op.afegirCampText("identificadorPersonal",contrasenya);
      
         RelacioSocis relacioSocis = new RelacioSocis();
         relacioSocis.obteSociPerMultiplesCamps(op);
      
         op.destructorObjectePersistencia();
      
         if(relacioSocis.hiHaSocis()) 
            return true;
         else 
            return false;
      
      }
   
   /**
   * Metode que ens informa si hi ha usuaris en l'iterador que encara no hem consultat
   *@return Boolea amb valor positiu si hi ha elements i negatiu si no n'hi ha.
   */
      public boolean hiHaUsuaris()
      {
      
         return llistat.hiHaElements();
      
      }
   
   /**
   * Metode que ens permet obtenir el primer objecte usuari que encara
   * no hem consultat, amb totes les dades disponibles sobre aquest en el sistema.
   *@return UsuariAplicacio amb totes les dades disponibles
   */
      public UsuariAplicacio obteUsuari()
      {
      
         resultat=(Resultat)llistat.obtePrimerElement();
         UsuariAplicacio usuariAplicacio = new UsuariAplicacio();
      
         usuariAplicacio.estableixUsuari((String)resultat.obteElement("usuari"));
         usuariAplicacio.estableixContrasenya((String)resultat.obteElement("contrasenya"));
         usuariAplicacio.estableixNivell(Integer.parseInt((String)resultat.obteElement("nivell")));
      
         return usuariAplicacio;
      
      }
   
   //////////Fi iterador\\\\\\\
                                                                                                                                                                                                                                      
   /**
   * Metode que permet eliminar un UsuariAplicacio del sistema a partir del seu nom. Atencio:
   * En circumstancies normals, hi ha un sol usuari amb nom "usuari". Si n'hi hagues mes d'un,
   * el sistema eliminaria tots els usuaris amb el nom d'usuari entrat. 
   *@param usuari String amb el nom d'usuari
   */
      public static void eliminaUsuariActivitatPerNomUsuari(String usuari)
      {
      
         Persistencia Persistencia = new Persistencia("entitats","usuariAplicacio");
         Persistencia.eliminaPerCampText("usuari",usuari);
         Persistencia.destructorPersistencia();
      
      }
   
   /**
   * Metode que permet actualitzar de cop diversos camps d'un usuariAplicacio. Atencio:
   * En circumstancies normals, hi ha un sol usuari amb nom "usuari". Si n'hi hagues mes d'un,
   * el sistema actualitzaria tots els usuaris amb el nom d'usuari entrat. 
   *@param nomUsuari String amb el valor que ens permetra trobar l'usuari
   *que volem modificar 
   *@param dadesActualitzadesUsuari ObjectePersistencia amb els noms dels camps a modificar, aixi com
   *els valors d'aquests
   */
      public static void actualitzaDiversosCampsUsuariAplicacioPerNomUsuari(String nomUsuari,ObjectePersistencia dadesActualitzadesUsuari)
      {
      
         Persistencia Persistencia = new Persistencia("entitats","usuariAplicacio");
         Persistencia.actualitzaDiversosCampsPerCampText("usuari",nomUsuari,dadesActualitzadesUsuari);
         Persistencia.destructorPersistencia();
      
      }
   
   /**
   * Metode que ens permet afegir un UsuariActuacio a la relacio
   *@param usuariAplicacio Objecte UsuariAplicacio amb les dades del nou usuari que volem afegir
   *al sistema
   */
      public static void afegirUsuariAplicacio(UsuariAplicacio usuariAplicacio)
      {
      
         Persistencia Persistencia = new Persistencia("entitats","usuariAplicacio");
         ObjectePersistencia dadesNouUsuariAplicacio = new ObjectePersistencia();
      
         dadesNouUsuariAplicacio.afegirCampText("usuari",usuariAplicacio.obteUsuari());
         dadesNouUsuariAplicacio.afegirCampText("contrasenya",usuariAplicacio.obteContrasenya());
         dadesNouUsuariAplicacio.afegirCampNumeric("nivell",usuariAplicacio.obteNivell());
      
         Persistencia.afegeixElement(dadesNouUsuariAplicacio);
      
         dadesNouUsuariAplicacio.destructorObjectePersistencia();
         Persistencia.destructorPersistencia();
      
      }
   
   }