   package Entitats.Validadors;

   import Entitats.Soci.Soci;
   import java.util.regex.Pattern;

	/**
* Classe "eina" que ens permet validar el contingut de determinades cadenes de text per poder decidir si aquella dada
* pot passar a formar part del sistema o no
*@author Jordi Moles i Blanco, Febrer 2006
*/

   public class ValidadorFormulariEntrada
   {
   
   /**
   * Metode que valida que el text aportat sigui vlid com a nom de persona
   *@param nom String amb el valor a consultar
   *@return Boolean amb valor true si es apte i false en cas contrari
   */
      public static boolean validaNom(String nom)
      {
         final String validaNom="^[a-zA-Z-'\\s]{2,}$";
         if(Pattern.matches(validaNom,nom))
            return true;
         else 
            return false;
      
      }
   
   /**
   * Metode que valida que el text aportat sigui vlid com a identificador personal(dni, passaport,etc)
   *@param identificadorPersonal String amb el valor a consultar
   *@return Boolean amb valor true si es apte i false en cas contrari
   */
      public static boolean validaIdentificadorPersonal(String identificadorPersonal)     
      {
         final String validaIdentificadorPersonal="^[a-zA-Z0-9_-]{2,}$";
         if(Pattern.matches(validaIdentificadorPersonal,identificadorPersonal))
            return true;
         else 
            return false;
      
      }
   
   /**
   * Metode que valida que el text aportat sigui vlid com a adrea
   *@param adresa String amb el valor a consultar
   *@return Boolean amb valor true si es apte i false en cas contrari
   */
      public static boolean validaAdresa(String adresa)           
      {
         final String validaAdresa="^[a-zA-Z0-9-/,'\\s]{2,}$";
         if(Pattern.matches(validaAdresa,adresa))
            return true;
         else 
            return false;
      
      }
   
   /**
   * Metode que valida que el text aportat sigui vlid com a telefon
   *@param telefon String amb el valor a consultar
   *@return Boolean amb valor true si es apte i false en cas contrari
   */
      public static boolean validaTelefon(String telefon)           
      {
         final String validaTelefon="^[6|8|9][0-9]{8}$";
         if(Pattern.matches(validaTelefon,telefon))
            return true;
         else 
            return false;
      
      }
   
   /**
   * Metode que valida que el text aportat sigui vlid com a data
   *@param data String amb el valor a consultar
   *@return Boolean amb valor true si es apte i false en cas contrari
   */
      public static boolean validaData(String data)     
      {
         final String validaData="^(0[0-9]|1[0-9]|2[0-9]|3[01]){1}/(0[0-9]|1[012]){1}/(19[0-9][0-9]|20[0-9][0-9]){1}$";
         if(Pattern.matches(validaData,data))
            return true;
         else 
            return false;
      
      }
   
   /**
   * Metode que valida que el text aportat sigui vlid com a any
   *@param any String amb el valor a consultar
   *@return Boolean amb valor true si es apte i false en cas contrari
   */
      public static boolean validaAny(String any)
      {
      
         final String validaAny="^[0-9]{4}$";
         if(Pattern.matches(validaAny,any))
            return true;
         else 
            return false;
      
      }
   
   /**
   * Metode que valida que el text aportat sigui vlid com a descripcio
   *@param descripcio String amb el valor a consultar
   *@return Boolean amb valor true si es apte i false en cas contrari
   */
      public static boolean validaDescripcio(String descripcio)
      
      {
      
         final String validaDescripcio="^[0-9a-zA-Z,.;'\\s]{2,500}$";
         if(Pattern.matches(validaDescripcio,descripcio))
            return true;
         else 
            return false;
      
      }
   
   /**
   * Metode que valida que el text aportat sigui vlid com a durada de, per exemple, una pea musical
   *@param durada String amb el valor a consultar
   *@return Boolean amb valor true si es apte i false en cas contrari
   */
      public static boolean validaDurada(String durada)
      {
      
         final String validaDurada="^[0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2}$";
         if(Pattern.matches(validaDurada,durada))
            return true;
         else 
            return false;
      
      }
   
   /**
   * Metode que valida que el text aportat sigui vlid , per exemple, com a hora d'inici d'un espectacle
   *@param hora String amb el valor a consultar
   *@return Boolean amb valor true si es apte i false en cas contrari
   */
      public static boolean validaHora(String hora)
      {
      
         final String validaHora="^[0-9]{1,2}:[0-9]{1,2}$";
         if(Pattern.matches(validaHora,hora))
            return true;
         else 
            return false;
      
      }
   
   /**
   * Metode que valida que el text aportat sigui vlid com a xifra, s a dir, que tingui un valo integrament numric
   *@param xifra String amb el valor a consultar
   *@return Boolean amb valor true si es apte i false en cas contrari
   */
      public static boolean validaXifra(String xifra)
      
      {
      
         final String validaXifra="^[0-9]{1,}$";
         if(Pattern.matches(validaXifra,xifra))
            return true;
         else 
            return false;
      
      }
   
   /**
   * Metode que valida que el text aportat sigui vlid com a nom d'usuari per a la nostra aplicaci
   *@param nomUsuari String amb el valor a consultar
   *@return Boolean amb valor true si es apte i false en cas contrari
   */
      public static boolean validaNomUsuari(String nomUsuari)
      {
      
         final String validaNomUsuari="^[a-zA-Z]{3,}$";
         if(Pattern.matches(validaNomUsuari,nomUsuari))
            return true;
         else 
            return false;
      
      }
   
   /**
   * Metode que valida que el text aportat sigui vlid com a password, per exemple, que tingui un mnim de 6 carcters
   *@param contrasenya String amb el valor a consultar
   *@return Boolean amb valor true si es apte i false en cas contrari
   */
      public static boolean validaContrasenya(String contrasenya)
      {
      
         final String validaContrasenya="^[a-zA-Z0-9_-]{6,}$";
         if(Pattern.matches(validaContrasenya,contrasenya))
            return true;
         else 
            return false;
      
      }
   
   }